@@ -18,7 +18,8 @@ const NAME2TOPIC = {
   'IceCat': 'sessionstore-windows-restored',
   'IceCatMobile': 'sessionstore-windows-restored',
   'SeaMonkey': 'sessionstore-windows-restored',
-  'Thunderbird': 'mail-startup-done'
+  'Thunderbird': 'mail-startup-done',
+  'Instantbird': 'xul-window-visible'
 };
 
 var Startup = {
