@@ -13,7 +13,8 @@ TestSuffix(const PrincipalOriginAttributes& attrs)
   attrs.CreateSuffix(suffix);
 
   PrincipalOriginAttributes attrsFromSuffix;
-  attrsFromSuffix.PopulateFromSuffix(suffix);
+  bool success = attrsFromSuffix.PopulateFromSuffix(suffix);
+  EXPECT_TRUE(success);
 
   EXPECT_EQ(attrs, attrsFromSuffix);
 }
@@ -24,13 +25,13 @@ TEST(PrincipalOriginAttributes, Suffix_default)
   TestSuffix(attrs);
 }
 
-TEST(PrincipalOriginAttributes, Suffix_appId_inBrowser)
+TEST(PrincipalOriginAttributes, Suffix_appId_inIsolatedMozBrowser)
 {
   PrincipalOriginAttributes attrs(1, true);
   TestSuffix(attrs);
 }
 
-TEST(PrincipalOriginAttributes, Suffix_maxAppId_inBrowser)
+TEST(PrincipalOriginAttributes, Suffix_maxAppId_inIsolatedMozBrowser)
 {
   PrincipalOriginAttributes attrs(4294967295, true);
   TestSuffix(attrs);
