@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -46,9 +48,9 @@ Crypto::Init(nsIGlobalObject* aParent)
 }
 
 /* virtual */ JSObject*
-Crypto::WrapObject(JSContext* aCx)
+Crypto::WrapObject(JSContext* aCx, JS::Handle<JSObject*> aGivenProto)
 {
-  return CryptoBinding::Wrap(aCx, this);
+  return CryptoBinding::Wrap(aCx, this, aGivenProto);
 }
 
 void
@@ -56,10 +58,14 @@ Crypto::GetRandomValues(JSContext* aCx, const ArrayBufferView& aArray,
                         JS::MutableHandle<JSObject*> aRetval,
                         ErrorResult& aRv)
 {
-  MOZ_ASSERT(NS_IsMainThread(), "Called on the wrong thread");
-
   JS::Rooted<JSObject*> view(aCx, aArray.Obj());
 
+  if (JS_IsTypedArrayObject(view) && JS_GetTypedArraySharedness(view)) {
+    // Throw if the object is mapping shared memory (must opt in).
+    aRv.ThrowTypeError<MSG_TYPEDARRAY_IS_SHARED>(NS_LITERAL_STRING("Argument of Crypto.getRandomValues"));
+    return;
+  }
+
   // Throw if the wrong type of ArrayBufferView is passed in
   // (Part of the Web Crypto API spec)
   switch (JS_GetArrayBufferViewType(view)) {
@@ -87,32 +93,24 @@ Crypto::GetRandomValues(JSContext* aCx, const ArrayBufferView& aArray,
     return;
   }
 
-  uint8_t* data = aArray.Data();
-
-  if (XRE_GetProcessType() != GeckoProcessType_Default) {
-    InfallibleTArray<uint8_t> randomValues;
-    // Tell the parent process to generate random values via PContent
-    ContentChild* cc = ContentChild::GetSingleton();
-    if (!cc->SendGetRandomValues(dataLen, &randomValues) ||
-        randomValues.Length() == 0) {
-      aRv.Throw(NS_ERROR_FAILURE);
-      return;
-    }
-    NS_ASSERTION(dataLen == randomValues.Length(),
-                 "Invalid length returned from parent process!");
-    memcpy(data, randomValues.Elements(), dataLen);
-  } else {
-    uint8_t *buf = GetRandomValues(dataLen);
-
-    if (!buf) {
-      aRv.Throw(NS_ERROR_FAILURE);
-      return;
-    }
-
-    memcpy(data, buf, dataLen);
-    NS_Free(buf);
+  nsCOMPtr<nsIRandomGenerator> randomGenerator =
+    do_GetService("@mozilla.org/security/random-generator;1");
+  if (!randomGenerator) {
+    aRv.Throw(NS_ERROR_DOM_OPERATION_ERR);
+    return;
   }
 
+  uint8_t* buf;
+  nsresult rv = randomGenerator->GenerateRandomBytes(dataLen, &buf);
+  if (NS_FAILED(rv) || !buf) {
+    aRv.Throw(NS_ERROR_DOM_OPERATION_ERR);
+    return;
+  }
+
+  // Copy random bytes to ABV.
+  memcpy(aArray.Data(), buf, dataLen);
+  free(buf);
+
   aRetval.set(view);
 }
 
@@ -125,21 +123,5 @@ Crypto::Subtle()
   return mSubtle;
 }
 
-/* static */ uint8_t*
-Crypto::GetRandomValues(uint32_t aLength)
-{
-  nsCOMPtr<nsIRandomGenerator> randomGenerator;
-  nsresult rv;
-  randomGenerator = do_GetService("@mozilla.org/security/random-generator;1");
-  NS_ENSURE_TRUE(randomGenerator, nullptr);
-
-  uint8_t* buf;
-  rv = randomGenerator->GenerateRandomBytes(aLength, &buf);
-
-  NS_ENSURE_SUCCESS(rv, nullptr);
-
-  return buf;
-}
-
 } // namespace dom
 } // namespace mozilla
