@@ -12,7 +12,6 @@
 #include "nsPresContext.h"
 #include "nsScriptLoader.h"
 #include "nsIParser.h"
-#include "nsAutoPtr.h"
 #include "nsGkAtoms.h"
 #include "nsContentSink.h"
 
@@ -27,7 +26,15 @@ nsScriptElement::ScriptAvailable(nsresult aResult,
                                  int32_t aLineNo)
 {
   if (!aIsInline && NS_FAILED(aResult)) {
-    return FireErrorEvent();
+    nsCOMPtr<nsIParser> parser = do_QueryReferent(mCreatorParser);
+    if (parser) {
+      parser->PushDefinedInsertionPoint();
+    }
+    nsresult rv = FireErrorEvent();
+    if (parser) {
+      parser->PopDefinedInsertionPoint();
+    }
+    return rv;
   }
   return NS_OK;
 }
@@ -117,7 +124,7 @@ nsScriptElement::MaybeProcessScript()
                "You forgot to add self as observer");
 
   if (mAlreadyStarted || !mDoneAddingChildren ||
-      !cont->GetCrossShadowCurrentDoc() || mMalformed || !HasScriptContent()) {
+      !cont->GetComposedDoc() || mMalformed || !HasScriptContent()) {
     return false;
   }
 
