@@ -62,7 +62,12 @@ class MediaChannelStatistics;
  */
 class MediaChannelStatistics {
 public:
-  MediaChannelStatistics() { Reset(); }
+  MediaChannelStatistics()
+    : mAccumulatedBytes(0)
+    , mIsStarted(false)
+  {
+    Reset();
+  }
 
   explicit MediaChannelStatistics(MediaChannelStatistics * aCopyFrom)
   {
@@ -135,8 +140,6 @@ private:
 typedef media::Interval<int64_t> MediaByteRange;
 typedef media::IntervalSet<int64_t> MediaByteRangeSet;
 
-class RtspMediaResource;
-
 /**
  * Provides a thread-safe, seek/read interface to resources
  * loaded from a URI. Uses MediaCache to cache data received over
@@ -339,12 +342,6 @@ public:
   // any thread.
   virtual const nsCString& GetContentType() const = 0;
 
-  // Get the RtspMediaResource pointer if this MediaResource really is a
-  // RtspMediaResource. For calling Rtsp specific functions.
-  virtual RtspMediaResource* GetRtspPointer() {
-    return nullptr;
-  }
-
   // Return true if the stream is a live stream
   virtual bool IsRealTime() {
     return false;
@@ -375,11 +372,10 @@ private:
 
 class BaseMediaResource : public MediaResource {
 public:
-  virtual nsIURI* URI() const override { return mURI; }
-  virtual void SetLoadInBackground(bool aLoadInBackground) override;
+  nsIURI* URI() const override { return mURI; }
+  void SetLoadInBackground(bool aLoadInBackground) override;
 
-  virtual size_t SizeOfExcludingThis(
-                  MallocSizeOf aMallocSizeOf) const override
+  size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const override
   {
     // Might be useful to track in the future:
     // - mChannel
@@ -392,8 +388,7 @@ public:
     return size;
   }
 
-  virtual size_t SizeOfIncludingThis(
-                  MallocSizeOf aMallocSizeOf) const override
+  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const override
   {
     return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
   }
@@ -424,7 +419,7 @@ protected:
     MOZ_COUNT_DTOR(BaseMediaResource);
   }
 
-  virtual const nsCString& GetContentType() const override
+  const nsCString& GetContentType() const override
   {
     return mContentType;
   }
@@ -551,21 +546,21 @@ public:
 
   // Ensure that the media cache writes any data held in its partial block.
   // Called on the main thread.
-  virtual void FlushCache() override;
+  void FlushCache() override;
 
   // Notify that the last data byte range was loaded.
-  virtual void NotifyLastByteRange() override;
+  void NotifyLastByteRange() override;
 
   // Main thread
-  virtual nsresult Open(nsIStreamListener** aStreamListener) override;
-  virtual nsresult Close() override;
-  virtual void     Suspend(bool aCloseImmediately) override;
-  virtual void     Resume() override;
-  virtual already_AddRefed<nsIPrincipal> GetCurrentPrincipal() override;
+  nsresult Open(nsIStreamListener** aStreamListener) override;
+  nsresult Close() override;
+  void     Suspend(bool aCloseImmediately) override;
+  void     Resume() override;
+  already_AddRefed<nsIPrincipal> GetCurrentPrincipal() override;
   // Return true if the stream has been closed.
-  bool IsClosed() const { return mCacheStream.IsClosed(); }
-  virtual bool     CanClone() override;
-  virtual already_AddRefed<MediaResource> CloneData(MediaResourceCallback* aDecoder) override;
+  bool     IsClosed() const { return mCacheStream.IsClosed(); }
+  bool     CanClone() override;
+  already_AddRefed<MediaResource> CloneData(MediaResourceCallback* aDecoder) override;
   // Set statistics to be recorded to the object passed in. If not called,
   // |ChannelMediaResource| will create it's own statistics objects in |Open|.
   void RecordStatisticsTo(MediaChannelStatistics *aStatistics) override {
@@ -575,31 +570,30 @@ public:
       mChannelStatistics = aStatistics;
     }
   }
-  virtual nsresult ReadFromCache(char* aBuffer, int64_t aOffset, uint32_t aCount) override;
-  virtual void     EnsureCacheUpToDate() override;
+  nsresult ReadFromCache(char* aBuffer, int64_t aOffset, uint32_t aCount) override;
+  void     EnsureCacheUpToDate() override;
 
   // Other thread
-  virtual void     SetReadMode(MediaCacheStream::ReadMode aMode) override;
-  virtual void     SetPlaybackRate(uint32_t aBytesPerSecond) override;
-  virtual nsresult ReadAt(int64_t offset, char* aBuffer,
-                          uint32_t aCount, uint32_t* aBytes) override;
-  virtual already_AddRefed<MediaByteBuffer> MediaReadAt(int64_t aOffset, uint32_t aCount) override;
-  virtual int64_t Tell() override;
+  void     SetReadMode(MediaCacheStream::ReadMode aMode) override;
+  void     SetPlaybackRate(uint32_t aBytesPerSecond) override;
+  nsresult ReadAt(int64_t offset, char* aBuffer,
+                  uint32_t aCount, uint32_t* aBytes) override;
+  already_AddRefed<MediaByteBuffer> MediaReadAt(int64_t aOffset, uint32_t aCount) override;
+  int64_t Tell() override;
 
   // Any thread
-  virtual void    Pin() override;
-  virtual void    Unpin() override;
-  virtual double  GetDownloadRate(bool* aIsReliable) override;
-  virtual int64_t GetLength() override;
-  virtual int64_t GetNextCachedData(int64_t aOffset) override;
-  virtual int64_t GetCachedDataEnd(int64_t aOffset) override;
-  virtual bool    IsDataCachedToEndOfResource(int64_t aOffset) override;
-  virtual bool    IsSuspendedByCache() override;
-  virtual bool    IsSuspended() override;
-  virtual bool    IsTransportSeekable() override;
+  void    Pin() override;
+  void    Unpin() override;
+  double  GetDownloadRate(bool* aIsReliable) override;
+  int64_t GetLength() override;
+  int64_t GetNextCachedData(int64_t aOffset) override;
+  int64_t GetCachedDataEnd(int64_t aOffset) override;
+  bool    IsDataCachedToEndOfResource(int64_t aOffset) override;
+  bool    IsSuspendedByCache() override;
+  bool    IsSuspended() override;
+  bool    IsTransportSeekable() override;
 
-  virtual size_t SizeOfExcludingThis(
-                      MallocSizeOf aMallocSizeOf) const override {
+  size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const override {
     // Might be useful to track in the future:
     //   - mListener (seems minor)
     //   - mChannelStatistics (seems minor)
@@ -611,8 +605,7 @@ public:
     return size;
   }
 
-  virtual size_t SizeOfIncludingThis(
-                      MallocSizeOf aMallocSizeOf) const override {
+  size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const override {
     return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
   }
 
@@ -637,7 +630,7 @@ public:
   };
   friend class Listener;
 
-  virtual nsresult GetCachedRanges(MediaByteRangeSet& aRanges) override;
+  nsresult GetCachedRanges(MediaByteRangeSet& aRanges) override;
 
 protected:
   // These are called on the main thread by Listener.
@@ -667,12 +660,12 @@ protected:
 
   void DoNotifyDataReceived();
 
-  static NS_METHOD CopySegmentToCache(nsIInputStream *aInStream,
-                                      void *aClosure,
-                                      const char *aFromSegment,
-                                      uint32_t aToOffset,
-                                      uint32_t aCount,
-                                      uint32_t *aWriteCount);
+  static nsresult CopySegmentToCache(nsIInputStream *aInStream,
+                                     void *aClosure,
+                                     const char *aFromSegment,
+                                     uint32_t aToOffset,
+                                     uint32_t aCount,
+                                     uint32_t *aWriteCount);
 
   // Main thread access only
   int64_t            mOffset;
@@ -699,13 +692,6 @@ protected:
   // isPending state at the time of the suspend request.
   bool mIgnoreResume;
 
-  // Start and end offset of the bytes to be requested.
-  MediaByteRange mByteRange;
-
-  // True if the stream can seek into unbuffered ranged, i.e. if the
-  // connection supports byte range requests.
-  bool mIsTransportSeekable;
-
   ChannelSuspendAgent mSuspendAgent;
 };
 
