@@ -6,7 +6,7 @@
 #include "LoadMonitor.h"
 #include "LoadManager.h"
 #include "nsString.h"
-#include "prlog.h"
+#include "mozilla/Logging.h"
 #include "prtime.h"
 #include "prinrval.h"
 #include "prsystem.h"
@@ -16,6 +16,8 @@
 #include "nsThreadUtils.h"
 #include "nsReadableUtils.h"
 #include "nsNetUtil.h"
+#include "nsIInputStream.h"
+#include "nsIFile.h"
 #include "nsILineInputStream.h"
 #include "nsIObserverService.h"
 #include "nsIServiceManager.h"
@@ -53,18 +55,12 @@
 #pragma comment(lib, "pdh.lib")
 #endif
 
-// NSPR_LOG_MODULES=LoadManager:5
+// MOZ_LOG=LoadManager:5
 #undef LOG
 #undef LOG_ENABLED
-#if defined(PR_LOGGING)
-#define LOG(args) PR_LOG(gLoadManagerLog, PR_LOG_DEBUG, args)
-#define LOG_ENABLED() PR_LOG_TEST(gLoadManagerLog, 4)
-#define LOG_MANY_ENABLED() PR_LOG_TEST(gLoadManagerLog, 5)
-#else
-#define LOG(args)
-#define LOG_ENABLED() (false)
-#define LOG_MANY_ENABLED() (false)
-#endif
+#define LOG(args) MOZ_LOG(gLoadManagerLog, mozilla::LogLevel::Debug, args)
+#define LOG_ENABLED() MOZ_LOG_TEST(gLoadManagerLog, mozilla::LogLevel::Debug)
+#define LOG_MANY_ENABLED() MOZ_LOG_TEST(gLoadManagerLog, mozilla::LogLevel::Verbose)
 
 namespace mozilla {
 
@@ -98,15 +94,15 @@ LoadMonitor::Observe(nsISupports* /* aSubject */,
   return NS_OK;
 }
 
-class LoadMonitorAddObserver : public nsRunnable
+class LoadMonitorAddObserver : public Runnable
 {
 public:
-  explicit LoadMonitorAddObserver(nsRefPtr<LoadMonitor> loadMonitor)
+  explicit LoadMonitorAddObserver(RefPtr<LoadMonitor> loadMonitor)
   {
     mLoadMonitor = loadMonitor;
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     nsCOMPtr<nsIObserverService> observerService =
         mozilla::services::GetObserverService();
@@ -120,18 +116,18 @@ public:
   }
 
 private:
-  nsRefPtr<LoadMonitor> mLoadMonitor;
+  RefPtr<LoadMonitor> mLoadMonitor;
 };
 
-class LoadMonitorRemoveObserver : public nsRunnable
+class LoadMonitorRemoveObserver : public Runnable
 {
 public:
-  explicit LoadMonitorRemoveObserver(nsRefPtr<LoadMonitor> loadMonitor)
+  explicit LoadMonitorRemoveObserver(RefPtr<LoadMonitor> loadMonitor)
   {
     mLoadMonitor = loadMonitor;
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     // remove xpcom shutdown observer
     nsCOMPtr<nsIObserverService> observerService =
@@ -144,7 +140,7 @@ public:
   }
 
 private:
-  nsRefPtr<LoadMonitor> mLoadMonitor;
+  RefPtr<LoadMonitor> mLoadMonitor;
 };
 
 void LoadMonitor::Shutdown()
@@ -162,7 +158,7 @@ void LoadMonitor::Shutdown()
     // collection.  Argh.
     mLoadInfoThread = nullptr;
 
-    nsRefPtr<LoadMonitorRemoveObserver> remObsRunner = new LoadMonitorRemoveObserver(this);
+    RefPtr<LoadMonitorRemoveObserver> remObsRunner = new LoadMonitorRemoveObserver(this);
     if (!NS_IsMainThread()) {
       NS_DispatchToMainThread(remObsRunner);
     } else {
@@ -536,10 +532,10 @@ nsresult RTCLoadInfo::UpdateProcessLoad() {
 
 // Note: This class can't be in the anonymous namespace, because then we can't
 // declare it as a friend of LoadMonitor.
-class LoadInfoCollectRunner : public nsRunnable
+class LoadInfoCollectRunner : public Runnable
 {
 public:
-  LoadInfoCollectRunner(nsRefPtr<LoadMonitor> loadMonitor,
+  LoadInfoCollectRunner(RefPtr<LoadMonitor> loadMonitor,
                         RefPtr<RTCLoadInfo> loadInfo,
                         nsIThread *loadInfoThread)
     : mThread(loadInfoThread),
@@ -550,7 +546,7 @@ public:
     mLoadInfo = loadInfo;
   }
 
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     if (NS_IsMainThread()) {
       if (mThread) {
@@ -590,7 +586,7 @@ public:
 private:
   nsCOMPtr<nsIThread> mThread;
   RefPtr<RTCLoadInfo> mLoadInfo;
-  nsRefPtr<LoadMonitor> mLoadMonitor;
+  RefPtr<LoadMonitor> mLoadMonitor;
   int mLoadUpdateInterval;
   int mLoadNoiseCounter;
 };
@@ -629,7 +625,7 @@ LoadMonitor::GetSystemLoad() {
 }
 
 nsresult
-LoadMonitor::Init(nsRefPtr<LoadMonitor> &self)
+LoadMonitor::Init(RefPtr<LoadMonitor> &self)
 {
   LOG(("Initializing LoadMonitor"));
 
@@ -641,12 +637,12 @@ LoadMonitor::Init(nsRefPtr<LoadMonitor> &self)
     return rv;
   }
 
-  nsRefPtr<LoadMonitorAddObserver> addObsRunner = new LoadMonitorAddObserver(self);
+  RefPtr<LoadMonitorAddObserver> addObsRunner = new LoadMonitorAddObserver(self);
   NS_DispatchToMainThread(addObsRunner);
 
   NS_NewNamedThread("Sys Load Info", getter_AddRefs(mLoadInfoThread));
 
-  nsRefPtr<LoadInfoCollectRunner> runner =
+  RefPtr<LoadInfoCollectRunner> runner =
     new LoadInfoCollectRunner(self, load_info, mLoadInfoThread);
   mLoadInfoThread->Dispatch(runner, NS_DISPATCH_NORMAL);
 
