@@ -27,7 +27,7 @@ public:
 
     NS_DECL_ISUPPORTS
 
-    NS_IMETHODIMP_(txAExprResult*) GetResult() override
+    NS_IMETHOD_(txAExprResult*) GetResult() override
     {
         return mValue;
     }
@@ -39,7 +39,7 @@ protected:
 
     virtual ~txXPathObjectAdaptor() {}
 
-    nsRefPtr<txAExprResult> mValue;
+    RefPtr<txAExprResult> mValue;
 };
 
 #endif // txXPathObjectAdaptor_h__
