@@ -24,9 +24,9 @@ public:
                      char16_t* aDest,
                      int32_t* aDestLength) override;
 
-  MOZ_WARN_UNUSED_RESULT NS_IMETHOD GetMaxLength(const char* aSrc,
-                                                 int32_t aSrcLength,
-                                                 int32_t* aDestLength) override;
+  MOZ_MUST_USE NS_IMETHOD GetMaxLength(const char* aSrc,
+                                       int32_t aSrcLength,
+                                       int32_t* aDestLength) override;
 
   NS_IMETHOD Reset() override;
 
