@@ -3,10 +3,13 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "logging.h"
+#include "nsIGfxInfo.h"
+#include "nsServiceManagerUtils.h"
 
 #include "PeerConnectionImpl.h"
 #include "PeerConnectionMedia.h"
 #include "MediaPipelineFactory.h"
+#include "MediaPipelineFilter.h"
 #include "transportflow.h"
 #include "transportlayer.h"
 #include "transportlayerdtls.h"
@@ -14,12 +17,14 @@
 
 #include "signaling/src/jsep/JsepTrack.h"
 #include "signaling/src/jsep/JsepTransport.h"
+#include "signaling/src/common/PtrVector.h"
 
-#ifdef MOZILLA_INTERNAL_API
+#if !defined(MOZILLA_EXTERNAL_LINKAGE)
 #include "MediaStreamTrack.h"
 #include "nsIPrincipal.h"
 #include "nsIDocument.h"
 #include "mozilla/Preferences.h"
+#include "MediaEngine.h"
 #endif
 
 #include "GmpVideoCodec.h"
@@ -28,26 +33,24 @@
 #include "OMXCodecWrapper.h"
 #endif
 
+#ifdef MOZ_WEBRTC_MEDIACODEC
+#include "MediaCodecVideoCodec.h"
+#endif
+
+#ifdef MOZILLA_INTERNAL_API
+#include "mozilla/Preferences.h"
+#endif
+
+#if !defined(MOZILLA_EXTERNAL_LINKAGE)
+#include "WebrtcGmpVideoCodec.h"
+#endif
+
 #include <stdlib.h>
 
 namespace mozilla {
 
 MOZ_MTLOG_MODULE("MediaPipelineFactory")
 
-// Trivial wrapper class around a vector of ptrs.
-template <class T> class PtrVector
-{
-public:
-  ~PtrVector()
-  {
-    for (auto it = values.begin(); it != values.end(); ++it) {
-      delete *it;
-    }
-  }
-
-  std::vector<T*> values;
-};
-
 static nsresult
 JsepCodecDescToCodecConfig(const JsepCodecDescription& aCodec,
                            AudioCodecConfig** aConfig)
@@ -70,12 +73,41 @@ JsepCodecDescToCodecConfig(const JsepCodecDescription& aCodec,
                                   desc.mName,
                                   desc.mClock,
                                   desc.mPacketSize,
-                                  desc.mChannels,
-                                  desc.mBitrate);
+                                  desc.mForceMono ? 1 : desc.mChannels,
+                                  desc.mBitrate,
+                                  desc.mFECEnabled);
+  (*aConfig)->mMaxPlaybackRate = desc.mMaxPlaybackRate;
+  (*aConfig)->mDtmfEnabled = desc.mDtmfEnabled;
 
   return NS_OK;
 }
 
+static std::vector<JsepCodecDescription*>
+GetCodecs(const JsepTrackNegotiatedDetails& aDetails)
+{
+  // We do not try to handle cases where a codec is not used on the primary
+  // encoding.
+  if (aDetails.GetEncodingCount()) {
+    return aDetails.GetEncoding(0).GetCodecs();
+  }
+  return std::vector<JsepCodecDescription*>();
+}
+
+static nsresult
+NegotiatedDetailsToAudioCodecConfigs(const JsepTrackNegotiatedDetails& aDetails,
+                                     PtrVector<AudioCodecConfig>* aConfigs)
+{
+  std::vector<JsepCodecDescription*> codecs(GetCodecs(aDetails));
+  for (const JsepCodecDescription* codec : codecs) {
+    AudioCodecConfig* config;
+    if (NS_FAILED(JsepCodecDescToCodecConfig(*codec, &config))) {
+      return NS_ERROR_INVALID_ARG;
+    }
+    aConfigs->values.push_back(config);
+  }
+  return NS_OK;
+}
+
 static nsresult
 JsepCodecDescToCodecConfig(const JsepCodecDescription& aCodec,
                            VideoCodecConfig** aConfig)
@@ -96,10 +128,10 @@ JsepCodecDescToCodecConfig(const JsepCodecDescription& aCodec,
     return NS_ERROR_INVALID_ARG;
   }
 
-  ScopedDeletePtr<VideoCodecConfigH264> h264Config;
+  UniquePtr<VideoCodecConfigH264> h264Config;
 
   if (desc.mName == "H264") {
-    h264Config = new VideoCodecConfigH264;
+    h264Config = MakeUnique<VideoCodecConfigH264>();
     size_t spropSize = sizeof(h264Config->sprop_parameter_sets);
     strncpy(h264Config->sprop_parameter_sets,
             desc.mSpropParameterSets.c_str(),
@@ -107,26 +139,49 @@ JsepCodecDescToCodecConfig(const JsepCodecDescription& aCodec,
     h264Config->sprop_parameter_sets[spropSize - 1] = '\0';
     h264Config->packetization_mode = desc.mPacketizationMode;
     h264Config->profile_level_id = desc.mProfileLevelId;
-    h264Config->max_mbps = desc.mMaxMbps;
-    h264Config->max_fs = desc.mMaxFs;
-    h264Config->max_cpb = desc.mMaxCpb;
-    h264Config->max_dpb = desc.mMaxDpb;
-    h264Config->max_br = desc.mMaxBr;
     h264Config->tias_bw = 0; // TODO. Issue 165.
   }
 
   VideoCodecConfig* configRaw;
   configRaw = new VideoCodecConfig(
-      pt, desc.mName, desc.mMaxFs, desc.mMaxFr, h264Config);
+      pt, desc.mName, desc.mConstraints, h264Config.get());
 
   configRaw->mAckFbTypes = desc.mAckFbTypes;
   configRaw->mNackFbTypes = desc.mNackFbTypes;
   configRaw->mCcmFbTypes = desc.mCcmFbTypes;
+  configRaw->mRembFbSet = desc.RtcpFbRembIsSet();
+  configRaw->mFECFbSet = desc.mFECEnabled;
 
   *aConfig = configRaw;
   return NS_OK;
 }
 
+static nsresult
+NegotiatedDetailsToVideoCodecConfigs(const JsepTrackNegotiatedDetails& aDetails,
+                                     PtrVector<VideoCodecConfig>* aConfigs)
+{
+  std::vector<JsepCodecDescription*> codecs(GetCodecs(aDetails));
+  for (const JsepCodecDescription* codec : codecs) {
+    VideoCodecConfig* config;
+    if (NS_FAILED(JsepCodecDescToCodecConfig(*codec, &config))) {
+      return NS_ERROR_INVALID_ARG;
+    }
+
+    for (size_t i = 0; i < aDetails.GetEncodingCount(); ++i) {
+      const JsepTrackEncoding& jsepEncoding(aDetails.GetEncoding(i));
+      if (jsepEncoding.HasFormat(codec->mDefaultPt)) {
+        VideoCodecConfig::SimulcastEncoding encoding;
+        encoding.rid = jsepEncoding.mRid;
+        encoding.constraints = jsepEncoding.mConstraints;
+        config->mSimulcastEncodings.push_back(encoding);
+      }
+    }
+    aConfigs->values.push_back(config);
+  }
+
+  return NS_OK;
+}
+
 // Accessing the PCMedia should be safe here because we shouldn't
 // have enqueued this function unless it was still active and
 // the ICE data is destroyed on the STS.
@@ -138,8 +193,9 @@ FinalizeTransportFlow_s(RefPtr<PeerConnectionMedia> aPCMedia,
 {
   TransportLayerIce* ice =
       static_cast<TransportLayerIce*>(aLayerList->values.front());
-  ice->SetParameters(
-      aPCMedia->ice_ctx(), aPCMedia->ice_media_stream(aLevel), aIsRtcp ? 2 : 1);
+  ice->SetParameters(aPCMedia->ice_ctx(),
+                     aPCMedia->ice_media_stream(aLevel),
+                     aIsRtcp ? 2 : 1);
   nsAutoPtr<std::queue<TransportLayer*> > layerQueue(
       new std::queue<TransportLayer*>);
   for (auto i = aLayerList->values.begin(); i != aLayerList->values.end();
@@ -150,6 +206,19 @@ FinalizeTransportFlow_s(RefPtr<PeerConnectionMedia> aPCMedia,
   (void)aFlow->PushLayers(layerQueue); // TODO(bug 854518): Process errors.
 }
 
+static void
+AddNewIceStreamForRestart_s(RefPtr<PeerConnectionMedia> aPCMedia,
+                            RefPtr<TransportFlow> aFlow,
+                            size_t aLevel,
+                            bool aIsRtcp)
+{
+  TransportLayerIce* ice =
+      static_cast<TransportLayerIce*>(aFlow->GetLayer("ice"));
+  ice->SetParameters(aPCMedia->ice_ctx(),
+                     aPCMedia->ice_media_stream(aLevel),
+                     aIsRtcp ? 2 : 1);
+}
+
 nsresult
 MediaPipelineFactory::CreateOrGetTransportFlow(
     size_t aLevel,
@@ -162,6 +231,21 @@ MediaPipelineFactory::CreateOrGetTransportFlow(
 
   flow = mPCMedia->GetTransportFlow(aLevel, aIsRtcp);
   if (flow) {
+    if (mPCMedia->IsIceRestarting()) {
+      MOZ_MTLOG(ML_INFO, "Flow[" << flow->id() << "]: "
+                                 << "detected ICE restart - level: "
+                                 << aLevel << " rtcp: " << aIsRtcp);
+
+      rv = mPCMedia->GetSTSThread()->Dispatch(
+          WrapRunnableNM(AddNewIceStreamForRestart_s,
+                         mPCMedia, flow, aLevel, aIsRtcp),
+          NS_DISPATCH_NORMAL);
+      if (NS_FAILED(rv)) {
+        MOZ_MTLOG(ML_ERROR, "Failed to dispatch AddNewIceStreamForRestart_s");
+        return rv;
+      }
+    }
+
     *aFlowOutparam = flow;
     return NS_OK;
   }
@@ -179,7 +263,7 @@ MediaPipelineFactory::CreateOrGetTransportFlow(
                     ? TransportLayerDtls::CLIENT
                     : TransportLayerDtls::SERVER);
 
-  RefPtr<DtlsIdentity> pcid = mPC->GetIdentity();
+  RefPtr<DtlsIdentity> pcid = mPC->Identity();
   if (!pcid) {
     MOZ_MTLOG(ML_ERROR, "Failed to get DTLS identity.");
     return NS_ERROR_FAILURE;
@@ -211,6 +295,22 @@ MediaPipelineFactory::CreateOrGetTransportFlow(
     return rv;
   }
 
+  // Always permits negotiation of the confidential mode.
+  // Only allow non-confidential (which is an allowed default),
+  // if we aren't confidential.
+  std::set<std::string> alpn;
+  std::string alpnDefault = "";
+  alpn.insert("c-webrtc");
+  if (!mPC->PrivacyRequested()) {
+    alpnDefault = "webrtc";
+    alpn.insert(alpnDefault);
+  }
+  rv = dtls->SetAlpn(alpn, alpnDefault);
+  if (NS_FAILED(rv)) {
+    MOZ_MTLOG(ML_ERROR, "Couldn't set ALPN");
+    return rv;
+  }
+
   nsAutoPtr<PtrVector<TransportLayer> > layers(new PtrVector<TransportLayer>);
   layers->values.push_back(ice.release());
   layers->values.push_back(dtls.release());
@@ -263,8 +363,7 @@ MediaPipelineFactory::GetTransportParameters(
   }
 
   if (aTrackPair.mBundleLevel.isSome()) {
-    bool receiving =
-        aTrack.GetDirection() == JsepTrack::Direction::kJsepTrackReceiving;
+    bool receiving = aTrack.GetDirection() == sdp::kRecv;
 
     *aFilterOut = new MediaPipelineFilter;
 
@@ -283,13 +382,6 @@ MediaPipelineFactory::GetTransportParameters(
       for (auto i = uniquePts.begin(); i != uniquePts.end(); ++i) {
         (*aFilterOut)->AddUniquePT(*i);
       }
-    } else {
-      // Add local SSRCs so we can distinguish which RTCP packets actually
-      // belong to this pipeline.
-      for (auto i = aTrack.GetSsrcs().begin();
-           i != aTrack.GetSsrcs().end(); ++i) {
-        (*aFilterOut)->AddLocalSSRC(*i);
-      }
     }
   }
 
@@ -301,10 +393,17 @@ MediaPipelineFactory::CreateOrUpdateMediaPipeline(
     const JsepTrackPair& aTrackPair,
     const JsepTrack& aTrack)
 {
+#if !defined(MOZILLA_EXTERNAL_LINKAGE)
+  // The GMP code is all the way on the other side of webrtc.org, and it is not
+  // feasible to plumb this information all the way through. So, we set it (for
+  // the duration of this call) in a global variable. This allows the GMP code
+  // to report errors to the PC.
+  WebrtcGmpPCHandleSetter setter(mPC->GetHandle());
+#endif
+
   MOZ_ASSERT(aTrackPair.mRtpTransport);
 
-  bool receiving =
-      aTrack.GetDirection() == JsepTrack::Direction::kJsepTrackReceiving;
+  bool receiving = aTrack.GetDirection() == sdp::kRecv;
 
   size_t level;
   RefPtr<TransportFlow> rtpFlow;
@@ -351,32 +450,6 @@ MediaPipelineFactory::CreateOrUpdateMediaPipeline(
     return NS_ERROR_FAILURE;
   }
 
-  RefPtr<MediaPipeline> pipeline =
-    stream->GetPipelineByTrackId_m(aTrack.GetTrackId());
-
-  if (pipeline && pipeline->level() != static_cast<int>(level)) {
-    MOZ_MTLOG(ML_WARNING, "Track " << aTrack.GetTrackId() <<
-                          " has moved from level " << pipeline->level() <<
-                          " to level " << level <<
-                          ". This requires re-creating the MediaPipeline.");
-    // Since we do not support changing the conduit on a pre-existing
-    // MediaPipeline
-    pipeline = nullptr;
-    stream->RemoveTrack(aTrack.GetTrackId());
-    stream->AddTrack(aTrack.GetTrackId());
-  }
-
-  if (pipeline) {
-    pipeline->UpdateTransport_m(level, rtpFlow, rtcpFlow, filter);
-    return NS_OK;
-  }
-
-  MOZ_MTLOG(ML_DEBUG,
-            "Creating media pipeline"
-                << " m-line index=" << aTrackPair.mLevel
-                << " type=" << aTrack.GetMediaType()
-                << " direction=" << aTrack.GetDirection());
-
   RefPtr<MediaSessionConduit> conduit;
   if (aTrack.GetMediaType() == SdpMediaSection::kAudio) {
     rv = GetOrCreateAudioConduit(aTrackPair, aTrack, &conduit);
@@ -391,6 +464,66 @@ MediaPipelineFactory::CreateOrUpdateMediaPipeline(
     return NS_OK;
   }
 
+  if (aTrack.GetActive()) {
+    if (receiving) {
+      auto error = conduit->StartReceiving();
+      if (error) {
+        MOZ_MTLOG(ML_ERROR, "StartReceiving failed: " << error);
+        return NS_ERROR_FAILURE;
+      }
+    } else {
+      auto error = conduit->StartTransmitting();
+      if (error) {
+        MOZ_MTLOG(ML_ERROR, "StartTransmitting failed: " << error);
+        return NS_ERROR_FAILURE;
+      }
+    }
+  } else {
+    if (receiving) {
+      auto error = conduit->StopReceiving();
+      if (error) {
+        MOZ_MTLOG(ML_ERROR, "StopReceiving failed: " << error);
+        return NS_ERROR_FAILURE;
+      }
+    } else {
+      auto error = conduit->StopTransmitting();
+      if (error) {
+        MOZ_MTLOG(ML_ERROR, "StopTransmitting failed: " << error);
+        return NS_ERROR_FAILURE;
+      }
+    }
+  }
+
+  RefPtr<MediaPipeline> pipeline =
+    stream->GetPipelineByTrackId_m(aTrack.GetTrackId());
+
+  if (pipeline && pipeline->level() != static_cast<int>(level)) {
+    MOZ_MTLOG(ML_WARNING, "Track " << aTrack.GetTrackId() <<
+                          " has moved from level " << pipeline->level() <<
+                          " to level " << level <<
+                          ". This requires re-creating the MediaPipeline.");
+    RefPtr<dom::MediaStreamTrack> domTrack =
+      stream->GetTrackById(aTrack.GetTrackId());
+    MOZ_ASSERT(domTrack, "MediaPipeline existed for a track, but no MediaStreamTrack");
+
+    // Since we do not support changing the conduit on a pre-existing
+    // MediaPipeline
+    pipeline = nullptr;
+    stream->RemoveTrack(aTrack.GetTrackId());
+    stream->AddTrack(aTrack.GetTrackId(), domTrack);
+  }
+
+  if (pipeline) {
+    pipeline->UpdateTransport_m(level, rtpFlow, rtcpFlow, filter);
+    return NS_OK;
+  }
+
+  MOZ_MTLOG(ML_DEBUG,
+            "Creating media pipeline"
+                << " m-line index=" << aTrackPair.mLevel
+                << " type=" << aTrack.GetMediaType()
+                << " direction=" << aTrack.GetDirection());
+
   if (receiving) {
     rv = CreateMediaPipelineReceiving(aTrackPair, aTrack,
                                       level, rtpFlow, rtcpFlow, filter,
@@ -419,15 +552,13 @@ MediaPipelineFactory::CreateMediaPipelineReceiving(
     const RefPtr<MediaSessionConduit>& aConduit)
 {
   // We will error out earlier if this isn't here.
-  nsRefPtr<RemoteSourceStreamInfo> stream =
+  RefPtr<RemoteSourceStreamInfo> stream =
       mPCMedia->GetRemoteStreamById(aTrack.GetStreamId());
 
   RefPtr<MediaPipelineReceive> pipeline;
 
   TrackID numericTrackId = stream->GetNumericTrackId(aTrack.GetTrackId());
-  MOZ_ASSERT(numericTrackId != TRACK_INVALID);
-
-  bool queue_track = stream->ShouldQueueTracks();
+  MOZ_ASSERT(IsTrackIDExplicit(numericTrackId));
 
   MOZ_MTLOG(ML_DEBUG, __FUNCTION__ << ": Creating pipeline for "
             << numericTrackId << " -> " << aTrack.GetTrackId());
@@ -437,29 +568,27 @@ MediaPipelineFactory::CreateMediaPipelineReceiving(
         mPC->GetHandle(),
         mPC->GetMainThread().get(),
         mPC->GetSTSThread(),
-        stream->GetMediaStream()->GetStream(),
+        stream->GetMediaStream()->GetInputStream()->AsSourceStream(),
         aTrack.GetTrackId(),
         numericTrackId,
         aLevel,
         static_cast<AudioSessionConduit*>(aConduit.get()), // Ugly downcast.
         aRtpFlow,
         aRtcpFlow,
-        aFilter,
-        queue_track);
+        aFilter);
   } else if (aTrack.GetMediaType() == SdpMediaSection::kVideo) {
     pipeline = new MediaPipelineReceiveVideo(
         mPC->GetHandle(),
         mPC->GetMainThread().get(),
         mPC->GetSTSThread(),
-        stream->GetMediaStream()->GetStream(),
+        stream->GetMediaStream()->GetInputStream()->AsSourceStream(),
         aTrack.GetTrackId(),
         numericTrackId,
         aLevel,
         static_cast<VideoSessionConduit*>(aConduit.get()), // Ugly downcast.
         aRtpFlow,
         aRtcpFlow,
-        aFilter,
-        queue_track);
+        aFilter);
   } else {
     MOZ_ASSERT(false);
     MOZ_MTLOG(ML_ERROR, "Invalid media type in CreateMediaPipelineReceiving");
@@ -498,28 +627,32 @@ MediaPipelineFactory::CreateMediaPipelineSending(
   nsresult rv;
 
   // This is checked earlier
-  nsRefPtr<LocalSourceStreamInfo> stream =
+  RefPtr<LocalSourceStreamInfo> stream =
       mPCMedia->GetLocalStreamById(aTrack.GetStreamId());
 
+  dom::MediaStreamTrack* track =
+    stream->GetTrackById(aTrack.GetTrackId());
+  MOZ_ASSERT(track);
+
   // Now we have all the pieces, create the pipeline
   RefPtr<MediaPipelineTransmit> pipeline = new MediaPipelineTransmit(
       mPC->GetHandle(),
       mPC->GetMainThread().get(),
       mPC->GetSTSThread(),
-      stream->GetMediaStream(),
+      track,
       aTrack.GetTrackId(),
       aLevel,
-      aTrack.GetMediaType() == SdpMediaSection::kVideo,
       aConduit,
       aRtpFlow,
       aRtcpFlow,
       aFilter);
 
-#ifdef MOZILLA_INTERNAL_API
+#if !defined(MOZILLA_EXTERNAL_LINKAGE)
   // implement checking for peerIdentity (where failure == black/silence)
   nsIDocument* doc = mPC->GetWindow()->GetExtantDoc();
   if (doc) {
-    pipeline->UpdateSinkIdentity_m(doc->NodePrincipal(),
+    pipeline->UpdateSinkIdentity_m(track,
+                                   doc->NodePrincipal(),
                                    mPC->GetPeerIdentity());
   } else {
     MOZ_MTLOG(ML_ERROR, "Cannot initialize pipeline without attached doc");
@@ -556,8 +689,7 @@ MediaPipelineFactory::GetOrCreateAudioConduit(
     return NS_ERROR_INVALID_ARG;
   }
 
-  bool receiving =
-      aTrack.GetDirection() == JsepTrack::Direction::kJsepTrackReceiving;
+  bool receiving = aTrack.GetDirection() == sdp::kRecv;
 
   RefPtr<AudioSessionConduit> conduit =
     mPCMedia->GetAudioConduit(aTrackPair.mLevel);
@@ -572,39 +704,37 @@ MediaPipelineFactory::GetOrCreateAudioConduit(
     mPCMedia->AddAudioConduit(aTrackPair.mLevel, conduit);
   }
 
-  size_t numCodecs = aTrack.GetNegotiatedDetails()->GetCodecCount();
-  if (numCodecs == 0) {
+  PtrVector<AudioCodecConfig> configs;
+  nsresult rv = NegotiatedDetailsToAudioCodecConfigs(
+      *aTrack.GetNegotiatedDetails(), &configs);
+
+  if (NS_FAILED(rv)) {
+    MOZ_MTLOG(ML_ERROR, "Failed to convert JsepCodecDescriptions to "
+                        "AudioCodecConfigs.");
+    return rv;
+  }
+
+  if (configs.values.empty()) {
     MOZ_MTLOG(ML_ERROR, "Can't set up a conduit with 0 codecs");
     return NS_ERROR_FAILURE;
   }
 
   if (receiving) {
-    PtrVector<AudioCodecConfig> configs;
-
-    for (size_t i = 0; i < numCodecs; i++) {
-      const JsepCodecDescription* cdesc;
-      nsresult rv = aTrack.GetNegotiatedDetails()->GetCodec(i, &cdesc);
-      MOZ_ASSERT(NS_SUCCEEDED(rv));
-      if (NS_FAILED(rv)) {
-        MOZ_MTLOG(ML_ERROR, "Failed to get codec from jsep track, rv="
-                                << static_cast<uint32_t>(rv));
-        return rv;
-      }
-
-      AudioCodecConfig* configRaw;
-      rv = JsepCodecDescToCodecConfig(*cdesc, &configRaw);
-      if (NS_FAILED(rv))
-        return rv;
-
-      configs.values.push_back(configRaw);
-    }
-
     auto error = conduit->ConfigureRecvMediaCodecs(configs.values);
 
     if (error) {
       MOZ_MTLOG(ML_ERROR, "ConfigureRecvMediaCodecs failed: " << error);
       return NS_ERROR_FAILURE;
     }
+
+    if (!aTrackPair.mSending) {
+      // No send track, but we still need to configure an SSRC for receiver
+      // reports.
+      if (!conduit->SetLocalSSRC(aTrackPair.mRecvonlySsrc)) {
+        MOZ_MTLOG(ML_ERROR, "SetLocalSSRC failed");
+        return NS_ERROR_FAILURE;
+      }
+    }
   } else {
     // For now we only expect to have one ssrc per local track.
     auto ssrcs = aTrack.GetSsrcs();
@@ -617,23 +747,14 @@ MediaPipelineFactory::GetOrCreateAudioConduit(
 
     conduit->SetLocalCNAME(aTrack.GetCNAME().c_str());
 
-    const JsepCodecDescription* cdesc;
-    // Best codec.
-    nsresult rv = aTrack.GetNegotiatedDetails()->GetCodec(0, &cdesc);
-    MOZ_ASSERT(NS_SUCCEEDED(rv));
-    if (NS_FAILED(rv)) {
-      MOZ_MTLOG(ML_ERROR, "Failed to get codec from jsep track, rv="
-                              << static_cast<uint32_t>(rv));
-      return rv;
+    if (configs.values.size() > 1
+        && configs.values.back()->mName == "telephone-event") {
+      // we have a telephone event codec, so we need to make sure
+      // the dynamic pt is set properly
+      conduit->SetDtmfPayloadType(configs.values.back()->mType);
     }
 
-    AudioCodecConfig* configRaw;
-    rv = JsepCodecDescToCodecConfig(*cdesc, &configRaw);
-    if (NS_FAILED(rv))
-      return rv;
-
-    ScopedDeletePtr<AudioCodecConfig> config(configRaw);
-    auto error = conduit->ConfigureSendMediaCodec(config.get());
+    auto error = conduit->ConfigureSendMediaCodec(configs.values[0]);
     if (error) {
       MOZ_MTLOG(ML_ERROR, "ConfigureSendMediaCodec failed: " << error);
       return NS_ERROR_FAILURE;
@@ -671,8 +792,7 @@ MediaPipelineFactory::GetOrCreateVideoConduit(
     return NS_ERROR_INVALID_ARG;
   }
 
-  bool receiving =
-      aTrack.GetDirection() == JsepTrack::Direction::kJsepTrackReceiving;
+  bool receiving = aTrack.GetDirection() == sdp::kRecv;
 
   RefPtr<VideoSessionConduit> conduit =
     mPCMedia->GetVideoConduit(aTrackPair.mLevel);
@@ -680,44 +800,60 @@ MediaPipelineFactory::GetOrCreateVideoConduit(
   if (!conduit) {
     conduit = VideoSessionConduit::Create();
     if (!conduit) {
-      MOZ_MTLOG(ML_ERROR, "Could not create audio conduit");
+      MOZ_MTLOG(ML_ERROR, "Could not create video conduit");
       return NS_ERROR_FAILURE;
     }
 
     mPCMedia->AddVideoConduit(aTrackPair.mLevel, conduit);
   }
 
-  size_t numCodecs = aTrack.GetNegotiatedDetails()->GetCodecCount();
-  if (numCodecs == 0) {
+  PtrVector<VideoCodecConfig> configs;
+  nsresult rv = NegotiatedDetailsToVideoCodecConfigs(
+      *aTrack.GetNegotiatedDetails(), &configs);
+
+  if (NS_FAILED(rv)) {
+    MOZ_MTLOG(ML_ERROR, "Failed to convert JsepCodecDescriptions to "
+                        "VideoCodecConfigs.");
+    return rv;
+  }
+
+  if (configs.values.empty()) {
     MOZ_MTLOG(ML_ERROR, "Can't set up a conduit with 0 codecs");
     return NS_ERROR_FAILURE;
   }
 
   if (receiving) {
-    PtrVector<VideoCodecConfig> configs;
-
-    for (size_t i = 0; i < numCodecs; i++) {
-      const JsepCodecDescription* cdesc;
-
-      nsresult rv = aTrack.GetNegotiatedDetails()->GetCodec(i, &cdesc);
-      MOZ_ASSERT(NS_SUCCEEDED(rv));
-      if (NS_FAILED(rv)) {
-        MOZ_MTLOG(ML_ERROR, "Failed to get codec from jsep track, rv="
-                                << static_cast<uint32_t>(rv));
-        return rv;
+    if (!aTrackPair.mSending) {
+      // No send track, but we still need to configure an SSRC for receiver
+      // reports.
+      if (!conduit->SetLocalSSRC(aTrackPair.mRecvonlySsrc)) {
+        MOZ_MTLOG(ML_ERROR, "SetLocalSSRC failed");
+        return NS_ERROR_FAILURE;
       }
+    }
 
-      VideoCodecConfig* configRaw;
-      rv = JsepCodecDescToCodecConfig(*cdesc, &configRaw);
-      if (NS_FAILED(rv))
-        return rv;
-
-      UniquePtr<VideoCodecConfig> config(configRaw);
-      if (EnsureExternalCodec(*conduit, config.get(), false)) {
+    // Prune out stuff we cannot actually do. We should work to eliminate the
+    // need for this.
+    bool configuredH264 = false;
+    for (size_t i = 0; i < configs.values.size();) {
+      // TODO(bug 1200768): We can only handle configuring one recv H264 codec
+      if (configuredH264 && (configs.values[i]->mName == "H264")) {
+        delete configs.values[i];
+        configs.values.erase(configs.values.begin() + i);
         continue;
       }
 
-      configs.values.push_back(config.release());
+      // TODO(bug 1018791): This really should be checked sooner
+      if (EnsureExternalCodec(*conduit, configs.values[i], false)) {
+        delete configs.values[i];
+        configs.values.erase(configs.values.begin() + i);
+        continue;
+      }
+
+      if (configs.values[i]->mName == "H264") {
+        configuredH264 = true;
+      }
+      ++i;
     }
 
     auto error = conduit->ConfigureRecvMediaCodecs(configs.values);
@@ -738,30 +874,33 @@ MediaPipelineFactory::GetOrCreateVideoConduit(
 
     conduit->SetLocalCNAME(aTrack.GetCNAME().c_str());
 
-    const JsepCodecDescription* cdesc;
-    // Best codec.
-    nsresult rv = aTrack.GetNegotiatedDetails()->GetCodec(0, &cdesc);
-    MOZ_ASSERT(NS_SUCCEEDED(rv));
+    rv = ConfigureVideoCodecMode(aTrack,*conduit);
     if (NS_FAILED(rv)) {
-      MOZ_MTLOG(ML_ERROR, "Failed to get codec from jsep track, rv="
-                              << static_cast<uint32_t>(rv));
       return rv;
     }
 
-    VideoCodecConfig* configRaw;
-    rv = JsepCodecDescToCodecConfig(*cdesc, &configRaw);
-    if (NS_FAILED(rv))
-      return rv;
-
-    // Take possession of this pointer
-    ScopedDeletePtr<VideoCodecConfig> config(configRaw);
-
-    if (EnsureExternalCodec(*conduit, config, true)) {
+    // TODO(bug 1018791): This really should be checked sooner
+    if (EnsureExternalCodec(*conduit, configs.values[0], true)) {
       MOZ_MTLOG(ML_ERROR, "External codec not available");
       return NS_ERROR_FAILURE;
     }
 
-    auto error = conduit->ConfigureSendMediaCodec(config);
+
+    auto error = conduit->ConfigureSendMediaCodec(configs.values[0]);
+
+    const SdpExtmapAttributeList::Extmap* rtpStreamIdExt =
+        aTrack.GetNegotiatedDetails()->GetExt(
+            "urn:ietf:params:rtp-hdrext:sdes:rtp-stream-id");
+
+    if (rtpStreamIdExt) {
+      MOZ_MTLOG(ML_DEBUG, "Calling EnableRTPSenderIdExtension");
+      error = conduit->EnableRTPStreamIdExtension(true, rtpStreamIdExt->entry);
+
+      if (error) {
+        MOZ_MTLOG(ML_ERROR, "EnableRTPSenderIdExtension failed: " << error);
+        return NS_ERROR_FAILURE;
+      }
+    }
 
     if (error) {
       MOZ_MTLOG(ML_ERROR, "ConfigureSendMediaCodec failed: " << error);
@@ -774,6 +913,52 @@ MediaPipelineFactory::GetOrCreateVideoConduit(
   return NS_OK;
 }
 
+nsresult
+MediaPipelineFactory::ConfigureVideoCodecMode(const JsepTrack& aTrack,
+                                              VideoSessionConduit& aConduit)
+{
+#if !defined(MOZILLA_EXTERNAL_LINKAGE)
+  RefPtr<LocalSourceStreamInfo> stream =
+    mPCMedia->GetLocalStreamByTrackId(aTrack.GetTrackId());
+
+  //get video track
+  RefPtr<mozilla::dom::MediaStreamTrack> track =
+    stream->GetTrackById(aTrack.GetTrackId());
+
+  RefPtr<mozilla::dom::VideoStreamTrack> videotrack =
+    track->AsVideoStreamTrack();
+
+  if (!videotrack) {
+    MOZ_MTLOG(ML_ERROR, "video track not available");
+    return NS_ERROR_FAILURE;
+  }
+
+  dom::MediaSourceEnum source = videotrack->GetSource().GetMediaSource();
+  webrtc::VideoCodecMode mode = webrtc::kRealtimeVideo;
+  switch (source) {
+    case dom::MediaSourceEnum::Browser:
+    case dom::MediaSourceEnum::Screen:
+    case dom::MediaSourceEnum::Application:
+    case dom::MediaSourceEnum::Window:
+      mode = webrtc::kScreensharing;
+      break;
+
+    case dom::MediaSourceEnum::Camera:
+    default:
+      mode = webrtc::kRealtimeVideo;
+      break;
+  }
+
+  auto error = aConduit.ConfigureCodecMode(mode);
+  if (error) {
+    MOZ_MTLOG(ML_ERROR, "ConfigureCodecMode failed: " << error);
+    return NS_ERROR_FAILURE;
+  }
+
+#endif
+  return NS_OK;
+}
+
 /*
  * Add external H.264 video codec.
  */
@@ -783,8 +968,65 @@ MediaPipelineFactory::EnsureExternalCodec(VideoSessionConduit& aConduit,
                                           bool aIsSend)
 {
   if (aConfig->mName == "VP8") {
+#ifdef MOZ_WEBRTC_MEDIACODEC
+     if (aIsSend) {
+#ifdef MOZILLA_INTERNAL_API
+       bool enabled = mozilla::Preferences::GetBool("media.navigator.hardware.vp8_encode.acceleration_enabled", false);
+#else
+       bool enabled = false;
+#endif
+       if (enabled) {
+         nsCOMPtr<nsIGfxInfo> gfxInfo = do_GetService("@mozilla.org/gfx/info;1");
+         if (gfxInfo) {
+           int32_t status;
+           nsCString discardFailureId;
+           if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_WEBRTC_HW_ACCELERATION_ENCODE, discardFailureId, &status))) {
+             if (status != nsIGfxInfo::FEATURE_STATUS_OK) {
+               NS_WARNING("VP8 encoder hardware is not whitelisted: disabling.\n");
+             } else {
+               VideoEncoder* encoder = nullptr;
+               encoder = MediaCodecVideoCodec::CreateEncoder(MediaCodecVideoCodec::CodecType::CODEC_VP8);
+               if (encoder) {
+                 return aConduit.SetExternalSendCodec(aConfig, encoder);
+               }
+               return kMediaConduitNoError;
+             }
+           }
+         }
+       }
+     } else {
+#ifdef MOZILLA_INTERNAL_API
+       bool enabled = mozilla::Preferences::GetBool("media.navigator.hardware.vp8_decode.acceleration_enabled", false);
+#else
+       bool enabled = false;
+#endif
+       if (enabled) {
+         nsCOMPtr<nsIGfxInfo> gfxInfo = do_GetService("@mozilla.org/gfx/info;1");
+         if (gfxInfo) {
+           int32_t status;
+           nsCString discardFailureId;
+           if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_WEBRTC_HW_ACCELERATION_DECODE, discardFailureId, &status))) {
+             if (status != nsIGfxInfo::FEATURE_STATUS_OK) {
+               NS_WARNING("VP8 decoder hardware is not whitelisted: disabling.\n");
+             } else {
+               VideoDecoder* decoder;
+               decoder = MediaCodecVideoCodec::CreateDecoder(MediaCodecVideoCodec::CodecType::CODEC_VP8);
+               if (decoder) {
+                 return aConduit.SetExternalRecvCodec(aConfig, decoder);
+               }
+               return kMediaConduitNoError;
+             }
+           }
+         }
+       }
+     }
+#endif
+     return kMediaConduitNoError;
+  }
+  if (aConfig->mName == "VP9") {
     return kMediaConduitNoError;
-  } else if (aConfig->mName == "H264") {
+  }
+  if (aConfig->mName == "H264") {
     if (aConduit.CodecPluginID() != 0) {
       return kMediaConduitNoError;
     }
@@ -799,32 +1041,25 @@ MediaPipelineFactory::EnsureExternalCodec(VideoSessionConduit& aConduit,
 #endif
       if (encoder) {
         return aConduit.SetExternalSendCodec(aConfig, encoder);
-      } else {
-        return kMediaConduitInvalidSendCodec;
-      }
-    } else {
-      VideoDecoder* decoder;
-#ifdef MOZ_WEBRTC_OMX
-      decoder =
-          OMXVideoCodec::CreateDecoder(OMXVideoCodec::CodecType::CODEC_H264);
-#else
-      decoder = GmpVideoCodec::CreateDecoder();
-#endif
-      if (decoder) {
-        return aConduit.SetExternalRecvCodec(aConfig, decoder);
-      } else {
-        return kMediaConduitInvalidReceiveCodec;
       }
+      return kMediaConduitInvalidSendCodec;
     }
-    NS_NOTREACHED("Shouldn't get here!");
-  } else {
-    MOZ_MTLOG(ML_ERROR,
-              "Invalid video codec configured: " << aConfig->mName.c_str());
-    return aIsSend ? kMediaConduitInvalidSendCodec
-                   : kMediaConduitInvalidReceiveCodec;
+    VideoDecoder* decoder = nullptr;
+#ifdef MOZ_WEBRTC_OMX
+    decoder =
+      OMXVideoCodec::CreateDecoder(OMXVideoCodec::CodecType::CODEC_H264);
+#else
+    decoder = GmpVideoCodec::CreateDecoder();
+#endif
+    if (decoder) {
+      return aConduit.SetExternalRecvCodec(aConfig, decoder);
+    }
+    return kMediaConduitInvalidReceiveCodec;
   }
-
-  NS_NOTREACHED("Shouldn't get here!");
+  MOZ_MTLOG(ML_ERROR,
+            "Invalid video codec configured: " << aConfig->mName.c_str());
+  return aIsSend ? kMediaConduitInvalidSendCodec
+                 : kMediaConduitInvalidReceiveCodec;
 }
 
 } // namespace mozilla
