@@ -17,4 +17,8 @@ const char * ProxyToString(ProxyType proxy) {
   return PROXY_NAMES[proxy];
 }
 
+ProxyInfo::ProxyInfo() : type(PROXY_NONE), autodetect(false) {
+}
+ProxyInfo::~ProxyInfo() = default;
+
 } // namespace rtc
