@@ -12,15 +12,16 @@
 class nsFileProtocolHandler : public nsIFileProtocolHandler
                             , public nsSupportsWeakReference
 {
+    virtual ~nsFileProtocolHandler() {}
+
 public:
     NS_DECL_THREADSAFE_ISUPPORTS
     NS_DECL_NSIPROTOCOLHANDLER
     NS_DECL_NSIFILEPROTOCOLHANDLER
 
     nsFileProtocolHandler();
-    virtual ~nsFileProtocolHandler() {}
 
-    nsresult Init();
+    MOZ_MUST_USE nsresult Init();
 };
 
 #endif // !nsFileProtocolHandler_h__
