@@ -2027,6 +2027,11 @@ nsHttpTransaction::CheckForStickyAuthScheme()
   MOZ_ASSERT(mResponseHead);
   MOZ_ASSERT(PR_GetCurrentThread() == gSocketThread);
 
+  if (mClosed) {
+      LOG(("  closed, not checking"));
+      return;
+  }
+
   CheckForStickyAuthSchemeAt(nsHttp::WWW_Authenticate);
   CheckForStickyAuthSchemeAt(nsHttp::Proxy_Authenticate);
 }
