@@ -9,15 +9,17 @@
 
 #include "prio.h"
 #include "nscore.h"
+#include "nsIProxyInfo.h"
 
 nsresult nsSOCKSIOLayerAddToSocket(int32_t       family,
                                    const char   *host, 
                                    int32_t       port,
-                                   const char   *proxyHost,
-                                   int32_t       proxyPort,
+                                   nsIProxyInfo *proxyInfo,
                                    int32_t       socksVersion,
                                    uint32_t      flags,
                                    PRFileDesc   *fd, 
                                    nsISupports **info);
 
+bool IsHostLocalTarget(const nsACString& aHost);
+
 #endif /* nsSOCKSIOLayer_h__ */
