@@ -14,7 +14,7 @@
 #include "nsComponentManagerUtils.h"
 #include "nsStringAPI.h"
 #include "nsCOMPtr.h"
-#include "mozilla/Logging.h"
+#include "NetwerkTestLogging.h"
 
 //
 // set NSPR_LOG_MODULES=Test:5
@@ -25,7 +25,7 @@ static PRLogModuleInfo *gTestLog = nullptr;
 class MySocketListener : public nsIServerSocketListener
 {
 protected:
-    virtual ~MySocketListener() {}
+    virtual ~MySocketListener() = default;
 
 public:
     NS_DECL_THREADSAFE_ISUPPORTS
