@@ -81,8 +81,6 @@ if (params.testRoot == "browser") {
   params.testPrefix = "chrome://mochitests/content/chrome/";
 } else if (params.testRoot == "a11y") {
   params.testPrefix = "chrome://mochitests/content/a11y/";
-} else if (params.testRoot == "webapprtContent") {
-  params.testPrefix = "/webapprtContent/";
 } else {
   params.testPrefix = "/tests/";
 }
