@@ -8,9 +8,10 @@ PACKAGE_VERSION = '0.1'
 
 setup(name='mozdebug',
       version=PACKAGE_VERSION,
-      description="Utilities for running applications under native code debuggers intended for use in GNU testing",
+      description="Utilities for running applications under native code debuggers "
+      "intended for use in GNU testing",
       long_description="see http://mozbase.readthedocs.org/",
-      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
+      classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
       keywords='mozilla',
       author='Mozilla Automation and Testing Team',
       author_email='tools@lists.mozilla.org',
@@ -24,4 +25,3 @@ setup(name='mozdebug',
       # -*- Entry points: -*-
       """,
       )
-
