@@ -5,23 +5,24 @@
 import os
 import posixpath
 import re
+import shutil
 import subprocess
 import tempfile
 import time
 import traceback
 
 from abc import ABCMeta, abstractmethod
+from distutils import dir_util
 
 
 class ADBProcess(object):
     """ADBProcess encapsulates the data related to executing the adb process."""
+
     def __init__(self, args):
         #: command argument argument list.
         self.args = args
         #: Temporary file handle to be used for stdout.
         self.stdout_file = tempfile.TemporaryFile()
-        #: Temporary file handle to be used for stderr.
-        self.stderr_file = tempfile.TemporaryFile()
         #: boolean indicating if the command timed out.
         self.timedout = None
         #: exitcode of the process.
@@ -29,7 +30,7 @@ class ADBProcess(object):
         #: subprocess Process object used to execute the command.
         self.proc = subprocess.Popen(args,
                                      stdout=self.stdout_file,
-                                     stderr=self.stderr_file)
+                                     stderr=subprocess.STDOUT)
 
     @property
     def stdout(self):
@@ -41,24 +42,15 @@ class ADBProcess(object):
             content = self.stdout_file.read().rstrip()
         return content
 
-    @property
-    def stderr(self):
-        """Return the contents of stderr."""
-        if not self.stderr_file or self.stderr_file.closed:
-            content = ""
-        else:
-            self.stderr_file.seek(0, os.SEEK_SET)
-            content = self.stderr_file.read().rstrip()
-        return content
-
     def __str__(self):
-        return ('args: %s, exitcode: %s, stdout: %s, stderr: %s' % (
-            ' '.join(self.args), self.exitcode, self.stdout, self.stderr))
+        return ('args: %s, exitcode: %s, stdout: %s' % (
+            ' '.join(self.args), self.exitcode, self.stdout))
 
 # ADBError, ADBRootError, and ADBTimeoutError are treated
 # differently in order that unhandled ADBRootErrors and
 # ADBTimeoutErrors can be handled distinctly from ADBErrors.
 
+
 class ADBError(Exception):
     """ADBError is raised in situations where a command executed on a
     device either exited with a non-zero exitcode or when an
@@ -67,6 +59,19 @@ class ADBError(Exception):
     """
     pass
 
+
+class ADBListDevicesError(ADBError):
+    """ADBListDevicesError is raised when errors are found listing the
+    devices, typically not any permissions.
+
+    The devices information is stocked with the *devices* member.
+    """
+
+    def __init__(self, msg, devices):
+        ADBError.__init__(self, msg)
+        self.devices = devices
+
+
 class ADBRootError(Exception):
     """ADBRootError is raised when a shell command is to be executed as
     root but the device does not support it. This error is fatal since
@@ -75,6 +80,7 @@ class ADBRootError(Exception):
     """
     pass
 
+
 class ADBTimeoutError(Exception):
     """ADBTimeoutError is raised when either a host command or shell
     command takes longer than the specified timeout to execute. The
@@ -145,16 +151,20 @@ class ADBCommand(object):
         self._adb_port = adb_port
         self._timeout = timeout
         self._polling_interval = 0.1
+        self._adb_version = ''
 
         self._logger.debug("%s: %s" % (self.__class__.__name__,
                                        self.__dict__))
 
         # catch early a missing or non executable adb command
+        # and get the adb version while we are at it.
         try:
-            subprocess.Popen([adb, 'help'],
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE).communicate()
-        except Exception, exc:
+            output = subprocess.Popen([adb, 'version'],
+                                      stdout=subprocess.PIPE,
+                                      stderr=subprocess.PIPE).communicate()
+            re_version = re.compile(r'Android Debug Bridge version (.*)')
+            self._adb_version = re_version.match(output[0]).group(1)
+        except Exception as exc:
             raise ADBError('%s: %s is not executable.' % (exc, adb))
 
     def _get_logger(self, logger_name):
@@ -193,7 +203,7 @@ class ADBCommand(object):
         commands on the host via adb.
 
         command() executes on the host in such a fashion that stdout
-        and stderr of the adb process are file handles on the host and
+        of the adb process is a file handle on the host and
         the exit code is available as the exit code of the adb
         process.
 
@@ -201,11 +211,11 @@ class ADBCommand(object):
         timeout period in seconds.
 
         A subprocess is spawned to execute adb with stdout and stderr
-        directed to temporary files. If the process takes longer than
+        directed to a temporary file. If the process takes longer than
         the specified timeout, the process is terminated.
 
         It is the caller's responsibilty to clean up by closing
-        the stdout and stderr temporary files.
+        the stdout temporary file.
         """
         args = [self._adb_path]
         if self._adb_host:
@@ -224,16 +234,15 @@ class ADBCommand(object):
         start_time = time.time()
         adb_process.exitcode = adb_process.proc.poll()
         while ((time.time() - start_time) <= timeout and
-               adb_process.exitcode == None):
+               adb_process.exitcode is None):
             time.sleep(self._polling_interval)
             adb_process.exitcode = adb_process.proc.poll()
-        if adb_process.exitcode == None:
+        if adb_process.exitcode is None:
             adb_process.proc.kill()
             adb_process.timedout = True
             adb_process.exitcode = adb_process.proc.poll()
 
         adb_process.stdout_file.seek(0, os.SEEK_SET)
-        adb_process.stderr_file.seek(0, os.SEEK_SET)
 
         return adb_process
 
@@ -286,7 +295,6 @@ class ADBCommand(object):
         finally:
             if adb_process and isinstance(adb_process.stdout_file, file):
                 adb_process.stdout_file.close()
-                adb_process.stderr_file.close()
 
 
 class ADBHost(ADBCommand):
@@ -300,6 +308,7 @@ class ADBHost(ADBCommand):
        adbhost = ADBHost()
        adbhost.start_server()
     """
+
     def __init__(self,
                  adb='adb',
                  adb_host=None,
@@ -340,7 +349,7 @@ class ADBHost(ADBCommand):
         commands on the host via adb.
 
         command() executes on the host in such a fashion that stdout
-        and stderr of the adb process are file handles on the host and
+        of the adb process is a file handle on the host and
         the exit code is available as the exit code of the adb
         process.
 
@@ -348,11 +357,11 @@ class ADBHost(ADBCommand):
         timeout period in seconds.
 
         A subprocess is spawned to execute adb with stdout and stderr
-        directed to temporary files. If the process takes longer than
+        directed to a temporary file. If the process takes longer than
         the specified timeout, the process is terminated.
 
         It is the caller's responsibilty to clean up by closing
-        the stdout and stderr temporary files.
+        the stdout temporary file.
         """
         return ADBCommand.command(self, cmds, timeout=timeout)
 
@@ -432,6 +441,7 @@ class ADBHost(ADBCommand):
         :type timeout: integer or None
         :returns: an object contain
         :raises: * ADBTimeoutError
+                 * ADBListDevicesError
                  * ADBError
 
         The output of adb devices -l ::
@@ -447,7 +457,9 @@ class ADBHost(ADBCommand):
         """
         # b313b945               device usb:1-7 product:d2vzw model:SCH_I535 device:d2vzw
         # from Android system/core/adb/transport.c statename()
-        re_device_info = re.compile(r'([^\s]+)\s+(offline|bootloader|device|host|recovery|sideload|no permissions|unauthorized|unknown)')
+        re_device_info = re.compile(
+            r"([^\s]+)\s+(offline|bootloader|device|host|recovery|sideload|"
+            "no permissions|unauthorized|unknown)")
         devices = []
         lines = self.command_output(["devices", "-l"], timeout=timeout).split('\n')
         for line in lines:
@@ -468,6 +480,14 @@ class ADBHost(ADBCommand):
                         self._logger.warning('devices: Unable to parse '
                                              'remainder for device %s' % line)
                 devices.append(device)
+        for device in devices:
+            if device['state'] == 'no permissions':
+                raise ADBListDevicesError(
+                    "No permissions to detect devices. You should restart the"
+                    " adb server as root:\n"
+                    "\n# adb kill-server\n# adb start-server\n"
+                    "\nor maybe configure your udev rules.",
+                    devices)
         return devices
 
 
@@ -574,20 +594,38 @@ class ADBDevice(ADBCommand):
         # there is /sbin/ls which embeds ansi escape codes to colorize
         # the output.  Detect if we are using busybox ls. We want each
         # entry on a single line and we don't want . or ..
-        if self.shell_bool("/system/bin/ls /", timeout=timeout):
+        if self.shell_bool("/system/bin/ls /data/local/tmp", timeout=timeout):
             self._ls = "/system/bin/ls"
-        elif self.shell_bool("/system/xbin/ls /", timeout=timeout):
+        elif self.shell_bool("/system/xbin/ls /data/local/tmp", timeout=timeout):
             self._ls = "/system/xbin/ls"
         else:
             raise ADBError("ADBDevice.__init__: ls not found")
         try:
-            self.shell_output("%s -1A /" % self._ls, timeout=timeout)
+            self.shell_output("%s -1A /data/local/tmp" % self._ls, timeout=timeout)
             self._ls += " -1A"
         except ADBError:
             self._ls += " -a"
 
+        self._logger.info("%s supported" % self._ls)
+
         # Do we have cp?
         self._have_cp = self.shell_bool("type cp", timeout=timeout)
+        self._logger.info("Native cp support: %s" % self._have_cp)
+
+        # Do we have chmod -R?
+        try:
+            self._chmod_R = False
+            re_recurse = re.compile(r'[-]R')
+            chmod_output = self.shell_output("chmod --help", timeout=timeout)
+            match = re_recurse.search(chmod_output)
+            if match:
+                self._chmod_R = True
+        except (ADBError, ADBTimeoutError) as e:
+            self._logger.debug('Check chmod -R: %s' % e)
+            match = re_recurse.search(e.message)
+            if match:
+                self._chmod_R = True
+        self._logger.info("Native chmod -R support: %s" % self._chmod_R)
 
         self._logger.debug("ADBDevice: %s" % self.__dict__)
 
@@ -637,8 +675,7 @@ class ADBDevice(ADBCommand):
 
         # Do we need to run adb root to get a root shell?
         try:
-            if (not self._have_root_shell and
-                self.command_output(
+            if (not self._have_root_shell and self.command_output(
                     ["root"],
                     timeout=timeout).find("cannot run as root") == -1):
                 self._have_root_shell = True
@@ -646,7 +683,6 @@ class ADBDevice(ADBCommand):
         except ADBError:
             self._logger.debug("Check for root adbd failed")
 
-
     @staticmethod
     def _escape_command_line(cmd):
         """Utility function to return escaped and quoted version of command
@@ -655,7 +691,7 @@ class ADBDevice(ADBCommand):
         quoted_cmd = []
 
         for arg in cmd:
-            arg.replace('&', '\&')
+            arg.replace('&', r'\&')
 
             needs_quoting = False
             for char in [' ', '(', ')', '"', '&']:
@@ -796,7 +832,7 @@ class ADBDevice(ADBCommand):
         commands for a specific device on the host via adb.
 
         command() executes on the host in such a fashion that stdout
-        and stderr of the adb process are file handles on the host and
+        of the adb process are file handles on the host and
         the exit code is available as the exit code of the adb
         process.
 
@@ -805,12 +841,12 @@ class ADBDevice(ADBCommand):
         commands, as well as a timeout period in seconds.
 
         A subprocess is spawned to execute adb for the device with
-        stdout and stderr directed to temporary files. If the process
+        stdout and stderr directed to a temporary file. If the process
         takes longer than the specified timeout, the process is
         terminated.
 
         It is the caller's responsibilty to clean up by closing
-        the stdout and stderr temporary files.
+        the stdout temporary file.
         """
 
         return ADBCommand.command(self, cmds,
@@ -953,12 +989,11 @@ class ADBDevice(ADBCommand):
         on the device via adb shell.
 
         shell() executes on the host in such as fashion that stdout
-        contains the stdout of the host abd process combined with the
-        combined stdout/stderr of the shell command on the device
-        while stderr is still the stderr of the adb process on the
-        host. The exit code of shell() is the exit code of
+        contains the stdout and stderr of the host abd process
+        combined with the stdout and stderr of the shell command
+        on the device. The exit code of shell() is the exit code of
         the adb command if it was non-zero or the extracted exit code
-        from the stdout/stderr of the shell command executed on the
+        from the output of the shell command executed on the
         device.
 
         The caller provides a flag indicating if the command is to be
@@ -976,13 +1011,14 @@ class ADBDevice(ADBCommand):
         exit code.
 
         A subprocess is spawned to execute adb shell for the device
-        with stdout and stderr directed to temporary files. If the
+        with stdout and stderr directed to a temporary file. If the
         process takes longer than the specified timeout, the process
         is terminated. The return code is extracted from the stdout
         and is then removed from the file.
 
         It is the caller's responsibilty to clean up by closing
-        the stdout and stderr temporary files.
+        the stdout temporary files.
+
         """
         if root and not self._have_root_shell:
             # If root was requested and we do not already have a root
@@ -1020,10 +1056,10 @@ class ADBDevice(ADBCommand):
 
         start_time = time.time()
         exitcode = adb_process.proc.poll()
-        while ((time.time() - start_time) <= timeout) and exitcode == None:
+        while ((time.time() - start_time) <= timeout) and exitcode is None:
             time.sleep(self._polling_interval)
             exitcode = adb_process.proc.poll()
-        if exitcode == None:
+        if exitcode is None:
             adb_process.proc.kill()
             adb_process.timedout = True
             adb_process.exitcode = adb_process.proc.poll()
@@ -1033,7 +1069,6 @@ class ADBDevice(ADBCommand):
             adb_process.exitcode = exitcode
 
         adb_process.stdout_file.seek(0, os.SEEK_SET)
-        adb_process.stderr_file.seek(0, os.SEEK_SET)
 
         return adb_process
 
@@ -1071,7 +1106,6 @@ class ADBDevice(ADBCommand):
         finally:
             if adb_process:
                 adb_process.stdout_file.close()
-                adb_process.stderr_file.close()
 
     def shell_output(self, cmd, env=None, cwd=None, timeout=None, root=False):
         """Executes an adb shell on the device returning stdout.
@@ -1111,18 +1145,17 @@ class ADBDevice(ADBCommand):
                                    'timedout: %s, '
                                    'exitcode: %s, '
                                    'output: %s' %
-                                (' '.join(adb_process.args),
-                                 timeout,
+                                   (' '.join(adb_process.args),
+                                    timeout,
                                     root,
-                                 adb_process.timedout,
-                                 adb_process.exitcode,
-                                 output))
+                                    adb_process.timedout,
+                                    adb_process.exitcode,
+                                    output))
 
             return output
         finally:
             if adb_process and isinstance(adb_process.stdout_file, file):
                 adb_process.stdout_file.close()
-                adb_process.stderr_file.close()
 
     # Informational methods
 
@@ -1251,20 +1284,104 @@ class ADBDevice(ADBCommand):
         :raises: * ADBTimeoutError
                  * ADBError
         """
-        ip_regexp = re.compile(r'(\w+)\s+UP\s+([1-9]\d{0,2}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
-        data = self.shell_output('netcfg')
-        for line in data.split("\n"):
-            match = ip_regexp.search(line)
+        if not interfaces:
+            interfaces = ["wlan0", "eth0"]
+            wifi_interface = self.shell_output('getprop wifi.interface', timeout=timeout)
+            self._logger.debug('get_ip_address: wifi_interface: %s' % wifi_interface)
+            if wifi_interface and wifi_interface not in interfaces:
+                interfaces = interfaces.append(wifi_interface)
+
+        # ifconfig interface
+        # can return two different formats:
+        # eth0: ip 192.168.1.139 mask 255.255.255.0 flags [up broadcast running multicast]
+        # or
+        # wlan0     Link encap:Ethernet  HWaddr 00:9A:CD:B8:39:65
+        # inet addr:192.168.1.38  Bcast:192.168.1.255  Mask:255.255.255.0
+        # inet6 addr: fe80::29a:cdff:feb8:3965/64 Scope: Link
+        # UP BROADCAST RUNNING MULTICAST  MTU:1500  Metric:1
+        # RX packets:180 errors:0 dropped:0 overruns:0 frame:0
+        # TX packets:218 errors:0 dropped:0 overruns:0 carrier:0
+        # collisions:0 txqueuelen:1000
+        # RX bytes:84577 TX bytes:31202
+
+        re1_ip = re.compile(r'(\w+): ip ([0-9.]+) mask.*')
+        # re1_ip will match output of the first format
+        # with group 1 returning the interface and group 2 returing the ip address.
+
+        # re2_interface will match the interface line in the second format
+        # while re2_ip will match the inet addr line of the second format.
+        re2_interface = re.compile(r'(\w+)\s+Link')
+        re2_ip = re.compile(r'\s+inet addr:([0-9.]+)')
+
+        matched_interface = None
+        matched_ip = None
+        re_bad_addr = re.compile(r'127.0.0.1|0.0.0.0')
+
+        self._logger.debug('get_ip_address: ifconfig')
+        for interface in interfaces:
+            try:
+                output = self.shell_output('ifconfig %s' % interface,
+                                           timeout=timeout)
+            except ADBError:
+                output = ''
+
+            for line in output.split("\n"):
+                if not matched_interface:
+                    match = re1_ip.match(line)
+                    if match:
+                        matched_interface, matched_ip = match.groups()
+                    else:
+                        match = re2_interface.match(line)
+                        if match:
+                            matched_interface = match.group(1)
+                else:
+                    match = re2_ip.match(line)
+                    if match:
+                        matched_ip = match.group(1)
+
+                if matched_ip:
+                    if not re_bad_addr.match(matched_ip):
+                        self._logger.debug('get_ip_address: found: %s %s' %
+                                           (matched_interface, matched_ip))
+                        return matched_ip
+                    matched_interface = None
+                    matched_ip = None
+
+        self._logger.debug('get_ip_address: netcfg')
+        # Fall back on netcfg if ifconfig does not work.
+        # $ adb shell netcfg
+        # lo       UP   127.0.0.1/8       0x00000049 00:00:00:00:00:00
+        # dummy0   DOWN   0.0.0.0/0       0x00000082 8e:cd:67:48:b7:c2
+        # rmnet0   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet1   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet2   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet3   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet4   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet5   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet6   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # rmnet7   DOWN   0.0.0.0/0       0x00000000 00:00:00:00:00:00
+        # sit0     DOWN   0.0.0.0/0       0x00000080 00:00:00:00:00:00
+        # vip0     DOWN   0.0.0.0/0       0x00001012 00:01:00:00:00:01
+        # wlan0    UP   192.168.1.157/24  0x00001043 38:aa:3c:1c:f6:94
+
+        re3_netcfg = re.compile(r'(\w+)\s+UP\s+([1-9]\d{0,2}\.\d{1,3}\.\d{1,3}\.\d{1,3})')
+        try:
+            output = self.shell_output('netcfg', timeout=timeout)
+        except ADBError:
+            output = ''
+        for line in output.split("\n"):
+            match = re3_netcfg.search(line)
             if match:
-                interface, ip = match.groups()
-
-                if interface == "lo" or ip == "127.0.0.1":
-                    continue
-
-                if interfaces is None or interface in interfaces:
-                    return ip
-
-        return None
+                matched_interface, matched_ip = match.groups()
+                if matched_interface == "lo" or re_bad_addr.match(matched_ip):
+                    matched_interface = None
+                    matched_ip = None
+                elif matched_ip and matched_interface in interfaces:
+                    self._logger.debug('get_ip_address: found: %s %s' %
+                                       (matched_interface, matched_ip))
+                    return matched_ip
+        self._logger.debug('get_ip_address: not found')
+        return matched_ip
 
     # File management methods
 
@@ -1306,34 +1423,56 @@ class ADBDevice(ADBCommand):
                  * ADBRootError
                  * ADBError
         """
+        # Note that on some tests such as webappstartup, an error
+        # occurs during recursive calls to chmod where a "No such file
+        # or directory" error will occur for the
+        # /data/data/org.mozilla.icecatmobile/files/mozilla/*.webapp0/lock
+        # which is a symbolic link to a socket: lock ->
+        # 127.0.0.1:+<port>.  On Linux, chmod -R ignores symbolic
+        # links but it appear Android's version does not. We ignore
+        # this type of error, but pass on any other errors that are
+        # detected.
         path = posixpath.normpath(path.strip())
         self._logger.debug('chmod: path=%s, recursive=%s, mask=%s, root=%s' %
                            (path, recursive, mask, root))
-        self.shell_output("chmod %s %s" % (mask, path),
-                          timeout=timeout, root=root)
-        if recursive and self.is_dir(path, timeout=timeout, root=root):
-            files = self.list_files(path, timeout=timeout, root=root)
-            for f in files:
-                entry = path + "/" + f
-                self._logger.debug('chmod: entry=%s' % entry)
-                if self.is_dir(entry, timeout=timeout, root=root):
-                    self._logger.debug('chmod: recursion entry=%s' % entry)
-                    self.chmod(entry, recursive=recursive, mask=mask,
-                               timeout=timeout, root=root)
-                elif self.is_file(entry, timeout=timeout, root=root):
-                    try:
-                        self.shell_output("chmod %s %s" % (mask, entry),
-                                          timeout=timeout, root=root)
-                        self._logger.debug('chmod: file entry=%s' % entry)
-                    except ADBError, e:
-                        if e.message.find('No such file or directory'):
-                            # some kind of race condition is causing files
-                            # to disappear. Catch and report the error here.
-                            self._logger.warning('chmod: File %s vanished!: %s' %
-                                                 (entry, e))
-                else:
-                    self._logger.warning('chmod: entry %s does not exist' %
-                                         entry)
+        if not recursive:
+            self.shell_output("chmod %s %s" % (mask, path),
+                              timeout=timeout, root=root)
+            return
+
+        if self._chmod_R:
+            try:
+                self.shell_output("chmod -R %s %s" % (mask, path),
+                                  timeout=timeout, root=root)
+            except ADBError as e:
+                if e.message.find('No such file or directory') == -1:
+                    raise
+                self._logger.warning('chmod -R %s %s: Ignoring Error: %s' %
+                                     (mask, path, e.message))
+            return
+        # Obtain a list of the directories and files which match path
+        # and construct a shell script which explictly calls chmod on
+        # each of them.
+        entries = self.ls(path, recursive=recursive, timeout=timeout,
+                          root=root)
+        tmpf = None
+        chmodsh = None
+        try:
+            tmpf = tempfile.NamedTemporaryFile(delete=False)
+            for entry in entries:
+                tmpf.write('chmod %s %s\n' % (mask, entry))
+            tmpf.close()
+            chmodsh = '/data/local/tmp/%s' % os.path.basename(tmpf.name)
+            self.push(tmpf.name, chmodsh)
+            self.shell_output('chmod 777 %s' % chmodsh, timeout=timeout,
+                              root=root)
+            self.shell_output('sh -c %s' % chmodsh, timeout=timeout,
+                              root=root)
+        finally:
+            if tmpf:
+                os.unlink(tmpf.name)
+            if chmodsh:
+                self.rm(chmodsh, timeout=timeout, root=root)
 
     def exists(self, path, timeout=None, root=False):
         """Returns True if the path exists on the device.
@@ -1428,11 +1567,94 @@ class ADBDevice(ADBCommand):
             except ADBError:
                 self._logger.error('Ignoring exception in ADBDevice.list_files\n%s' %
                                    traceback.format_exc())
-                pass
         data[:] = [item for item in data if item]
         self._logger.debug('list_files: %s' % data)
         return data
 
+    def ls(self, path, recursive=False, timeout=None, root=False):
+        """Return a list of matching files/directories on the device.
+
+        The ls method emulates the behavior of the ls shell command.
+        It differs from the list_files method by supporting wild cards
+        and returning matches even if the path is not a directory and
+        by allowing a recursive listing.
+
+        ls /sdcard always returns /sdcard and not the contents of the
+        sdcard path. The ls method makes the behavior consistent with
+        others paths by adjusting /sdcard to /sdcard/. Note this is
+        also the case of other sdcard related paths such as
+        /storage/emulated/legacy but no adjustment is made in those
+        cases.
+
+        The ls method works around a Nexus 4 bug which prevents
+        recursive listing of directories on the sdcard unless the path
+        ends with "/*" by adjusting sdcard paths ending in "/" to end
+        with "/*". This adjustment is only made on official Nexus 4
+        builds with property ro.product.model "Nexus 4". Note that
+        this will fail to return any "hidden" files or directories
+        which begin with ".".
+
+        :param str path: The directory name on the device.
+        :param bool recursive: Flag specifying if a recursive listing
+            is to be returned. If recursive is False, the returned
+            matches will be relative to the path. If recursive is True,
+            the returned matches will be absolute paths.
+        :param timeout: The maximum time in
+            seconds for any spawned adb process to complete before
+            throwing an ADBTimeoutError.
+            This timeout is per adb call. The total time spent
+            may exceed this value. If it is not specified, the value
+            set in the ADBDevice constructor is used.
+        :type timeout: integer or None
+        :param bool root: Flag specifying if the command should
+            be executed as root.
+        :returns: list of files/directories contained in the directory.
+        :raises: * ADBTimeoutError
+                 * ADBRootError
+        """
+        path = posixpath.normpath(path.strip())
+        parent = ''
+        entries = {}
+
+        if path == '/sdcard':
+            path += '/'
+
+        # Android 2.3 and later all appear to support ls -R however
+        # Nexus 4 does not perform a recursive search on the sdcard
+        # unless the path is a directory with * wild card.
+        if not recursive:
+            recursive_flag = ''
+        else:
+            recursive_flag = '-R'
+            if path.startswith('/sdcard') and path.endswith('/'):
+                model = self.shell_output('getprop ro.product.model',
+                                          timeout=timeout,
+                                          root=root)
+                if model == 'Nexus 4':
+                    path += '*'
+        lines = self.shell_output('%s %s %s' % (self._ls, recursive_flag, path),
+                                  timeout=timeout,
+                                  root=root).split('\r\n')
+        for line in lines:
+            line = line.strip()
+            if not line:
+                parent = ''
+                continue
+            if line.endswith(':'):  # This is a directory
+                parent = line.replace(':', '/')
+                entry = parent
+                # Remove earlier entry which is marked as a file.
+                if parent[:-1] in entries:
+                    del entries[parent[:-1]]
+            elif parent:
+                entry = "%s%s" % (parent, line)
+            else:
+                entry = line
+            entries[entry] = 1
+        entry_list = entries.keys()
+        entry_list.sort()
+        return entry_list
+
     def mkdir(self, path, parents=False, timeout=None, root=False):
         """Create a directory on the device.
 
@@ -1502,8 +1724,42 @@ class ADBDevice(ADBCommand):
         :raises: * ADBTimeoutError
                  * ADBError
         """
-        self.command_output(["push", os.path.realpath(local), remote],
-                            timeout=timeout)
+        # remove trailing /
+        local = os.path.normpath(local)
+        remote = os.path.normpath(remote)
+        copy_required = False
+        if self._adb_version >= '1.0.36' and \
+           os.path.isdir(local) and self.is_dir(remote):
+            # See do_sync_push in
+            # https://android.googlesource.com/platform/system/core/+/master/adb/file_sync_client.cpp
+            # Work around change in behavior in adb 1.0.36 where if
+            # the remote destination directory exists, adb push will
+            # copy the source directory *into* the destination
+            # directory otherwise it will copy the source directory
+            # *onto* the destination directory.
+            #
+            # If the destination directory does exist, push to its
+            # parent directory.  If the source and destination leaf
+            # directory names are different, copy the source directory
+            # to a temporary directory with the same leaf name as the
+            # destination so that when we push to the parent, the
+            # source is copied onto the destination directory.
+            local_name = os.path.basename(local)
+            remote_name = os.path.basename(remote)
+            if local_name != remote_name:
+                copy_required = True
+                temp_parent = tempfile.mkdtemp()
+                new_local = os.path.join(temp_parent, remote_name)
+                dir_util.copy_tree(local, new_local)
+                local = new_local
+            remote = '/'.join(remote.rstrip('/').split('/')[:-1])
+        try:
+            self.command_output(["push", local, remote], timeout=timeout)
+        except:
+            raise
+        finally:
+            if copy_required:
+                shutil.rmtree(temp_parent)
 
     def pull(self, remote, local, timeout=None):
         """Pulls a file or directory from the device.
@@ -1522,8 +1778,42 @@ class ADBDevice(ADBCommand):
         :raises: * ADBTimeoutError
                  * ADBError
         """
-        self.command_output(["pull", remote, os.path.realpath(local)],
-                            timeout=timeout)
+        # remove trailing /
+        local = os.path.normpath(local)
+        remote = os.path.normpath(remote)
+        copy_required = False
+        original_local = local
+        if self._adb_version >= '1.0.36' and \
+           os.path.isdir(local) and self.is_dir(remote):
+            # See do_sync_pull in
+            # https://android.googlesource.com/platform/system/core/+/master/adb/file_sync_client.cpp
+            # Work around change in behavior in adb 1.0.36 where if
+            # the local destination directory exists, adb pull will
+            # copy the source directory *into* the destination
+            # directory otherwise it will copy the source directory
+            # *onto* the destination directory.
+            #
+            # If the destination directory does exist, pull to its
+            # parent directory. If the source and destination leaf
+            # directory names are different, pull the source directory
+            # into a temporary directory and then copy the temporary
+            # directory onto the destination.
+            local_name = os.path.basename(local)
+            remote_name = os.path.basename(remote)
+            if local_name != remote_name:
+                copy_required = True
+                temp_parent = tempfile.mkdtemp()
+                local = os.path.join(temp_parent, remote_name)
+            else:
+                local = '/'.join(local.rstrip('/').split('/')[:-1])
+        try:
+            self.command_output(["pull", remote, local], timeout=timeout)
+        except:
+            raise
+        finally:
+            if copy_required:
+                dir_util.copy_tree(local, original_local)
+                shutil.rmtree(temp_parent)
 
     def rm(self, path, recursive=False, force=False, timeout=None, root=False):
         """Delete files or directories on the device.
@@ -1554,7 +1844,7 @@ class ADBDevice(ADBCommand):
             self.shell_output("%s %s" % (cmd, path), timeout=timeout, root=root)
             if self.is_file(path, timeout=timeout, root=root):
                 raise ADBError('rm("%s") failed to remove file.' % path)
-        except ADBError, e:
+        except ADBError as e:
             if not force and 'No such file or directory' in e.message:
                 raise
 
@@ -1636,9 +1926,8 @@ class ADBDevice(ADBCommand):
         finally:
             if adb_process and isinstance(adb_process.stdout_file, file):
                 adb_process.stdout_file.close()
-                adb_process.stderr_file.close()
 
-    def kill(self, pids, sig=None,  attempts=3, wait=5,
+    def kill(self, pids, sig=None, attempts=3, wait=5,
              timeout=None, root=False):
         """Kills processes on the device given a list of process ids.
 
@@ -1669,7 +1958,7 @@ class ADBDevice(ADBCommand):
             args.extend(pid_list)
             try:
                 self.shell_output(' '.join(args), timeout=timeout, root=root)
-            except ADBError, e:
+            except ADBError as e:
                 if 'No such process' not in e.message:
                     raise
             pid_set = set(pid_list)
@@ -1679,7 +1968,7 @@ class ADBDevice(ADBCommand):
             if not pid_list:
                 break
             self._logger.debug("Attempt %d of %d to kill processes %s failed" %
-                               (attempt+1, attempts, pid_list))
+                               (attempt + 1, attempts, pid_list))
             time.sleep(wait)
 
         if pid_list:
@@ -1721,7 +2010,7 @@ class ADBDevice(ADBCommand):
         try:
             self.kill(pids, sig, attempts=attempts, wait=wait,
                       timeout=timeout, root=root)
-        except ADBError, e:
+        except ADBError as e:
             if self.process_exist(appname, timeout=timeout):
                 raise e
 
@@ -1951,7 +2240,7 @@ class ADBDevice(ADBCommand):
         directives = ['battery', 'disk', 'id', 'os', 'process', 'systime',
                       'uptime']
 
-        if (directive in directives):
+        if directive in directives:
             directives = [directive]
 
         info = {}
@@ -1973,7 +2262,7 @@ class ADBDevice(ADBCommand):
         if 'uptime' in directives:
             uptime = self.shell_output('uptime', timeout=timeout)
             if uptime:
-                m = re.match('up time: ((\d+) days, )*(\d{2}):(\d{2}):(\d{2})',
+                m = re.match(r'up time: ((\d+) days, )*(\d{2}):(\d{2}):(\d{2})',
                              uptime)
                 if m:
                     uptime = '%d days %d hours %d minutes %d seconds' % tuple(
