@@ -15,7 +15,7 @@ config = {
         # 'generate-build-stats',
         'update',  # decided by query_is_nightly()
     ],
-    'stage_platform': 'linux64-cc',
+    'stage_platform': 'linux64-ccov',
     'platform_supports_post_upload_to_latest': False,
     'enable_signing': False,
     'enable_talos_sendchange': False,
