@@ -1,18 +1,20 @@
+import os
+
+ABS_WORK_DIR = os.path.join(os.getcwd(), "build")
+
 config = {
     "log_name": "aurora_to_beta",
     "version_files": [
-        "browser/config/version.txt",
-        "browser/config/version_display.txt",
-        "config/milestone.txt",
-        "b2g/confvars.sh",
+        {"file": "browser/config/version.txt", "suffix": ""},
+        {"file": "browser/config/version_display.txt", "suffix": "b1"},
+        {"file": "config/milestone.txt", "suffix": ""},
     ],
     "replacements": [
         # File, from, to
         ("{}/{}".format(d, f),
         "ac_add_options --with-branding=mobile/android/branding/aurora",
         "ac_add_options --with-branding=mobile/android/branding/beta")
-        for d in ["mobile/android/config/mozconfigs/android-api-11/",
-                  "mobile/android/config/mozconfigs/android-api-9-10-constrained/",
+        for d in ["mobile/android/config/mozconfigs/android-api-15/",
                   "mobile/android/config/mozconfigs/android-x86/"]
         for f in ["debug", "nightly", "l10n-nightly"]
     ] + [
@@ -49,13 +51,19 @@ config = {
         ("browser/config/mozconfigs/whitelist",
          "ac_add_options --with-branding=browser/branding/aurora",
          "ac_add_options --with-branding=browser/branding/nightly"),
+    ] + [
+        ("build/mozconfig.common",
+         "MOZ_REQUIRE_SIGNING=${MOZ_REQUIRE_SIGNING-0}",
+         "MOZ_REQUIRE_SIGNING=${MOZ_REQUIRE_SIGNING-1}"),
+        ("build/mozconfig.common",
+         "# Disable enforcing that add-ons are signed by the trusted root",
+         "# Enable enforcing that add-ons are signed by the trusted root")
     ],
 
-    # Disallow sharing, since we want pristine .hg directories.
-    # "vcs_share_base": None,
+    "vcs_share_base": os.path.join(ABS_WORK_DIR, 'hg-shared'),
     # "hg_share_base": None,
     "tools_repo_url": "https://hg.mozilla.org/build/tools",
-    "tools_repo_revision": "default",
+    "tools_repo_branch": "default",
     "from_repo_url": "ssh://hg.mozilla.org/releases/mozilla-aurora",
     "to_repo_url": "ssh://hg.mozilla.org/releases/mozilla-beta",
 
@@ -68,9 +76,7 @@ config = {
         "requests==2.8.1",
     ],
 
-    "post_merge_builders": [
-        "mozilla-beta hg bundle",
-    ],
+    "post_merge_builders": [],
     "post_merge_nightly_branches": [
         # No nightlies on mozilla-beta
     ],
