@@ -4,18 +4,16 @@
 
 Components.utils.import("resource://gre/modules/AppConstants.jsm");
 
-function AppPicker() {};
+function AppPicker() {}
 
-var g_dialog = null;
-
-AppPicker.prototype = 
+AppPicker.prototype =
 {
     // Class members
     _incomingParams:null,
 
-    /** 
+    /**
     * Init the dialog and populate the application list
-    */ 
+    */
     appPickerLoad: function appPickerLoad() {
         const nsILocalHandlerApp = Components.interfaces.nsILocalHandlerApp;
 
@@ -24,9 +22,9 @@ AppPicker.prototype =
 
         document.title = this._incomingParams.title;
 
-        // Header creation - at the very least, we must have 
+        // Header creation - at the very least, we must have
         // a mime type:
-        //        
+        //
         // (icon) Zip File
         // (icon) filename
         //
@@ -46,9 +44,9 @@ AppPicker.prototype =
           description = filename;
           filename = "";
         }
-        
+
         // Setup the dialog header information
-        document.getElementById("content-description").setAttribute("value", 
+        document.getElementById("content-description").setAttribute("value",
           description);
         document.getElementById("suggested-filename").setAttribute("value",
           filename);
@@ -62,13 +60,13 @@ AppPicker.prototype =
         var list = document.getElementById("app-picker-listbox");
 
         var primaryCount = 0;
-        
+
         if (!fileList || fileList.length == 0) {
           // display a message saying nothing is configured
           document.getElementById("app-picker-notfound").removeAttribute("hidden");
           return;
         }
-        
+
         for (var idx = 0; idx < fileList.length; idx++) {
           var file = fileList.queryElementAt(idx, nsILocalHandlerApp);
           try {
@@ -94,9 +92,9 @@ AppPicker.prototype =
         }
     },
 
-    /** 
+    /**
     * Retrieve the moz-icon for the app
-    */ 
+    */
     getFileIconURL: function getFileIconURL(file) {
       var ios = Components.classes["@mozilla.org/network/io-service;1"].
                 getService(Components.interfaces.nsIIOService);
@@ -113,9 +111,9 @@ AppPicker.prototype =
       return "moz-icon://" + urlSpec + "?size=32";
     },
 
-    /** 
+    /**
     * Retrieve the pretty description from the file
-    */ 
+    */
     getFileDisplayName: function getFileDisplayName(file) {
       if (AppConstants.platform == "win") {
         if (file instanceof Components.interfaces.nsILocalFileWin) {
@@ -181,7 +179,7 @@ AppPicker.prototype =
 
       fp.init(window, this._incomingParams.title, nsIFilePicker.modeOpen);
       fp.appendFilters(nsIFilePicker.filterApps);
-      
+
       var fileLoc = Components.classes["@mozilla.org/file/directory_service;1"]
                             .getService(Components.interfaces.nsIProperties);
       var startLocation;
@@ -192,11 +190,11 @@ AppPicker.prototype =
       } else {
         startLocation = "Home";
       }
-      fp.displayDirectory = 
+      fp.displayDirectory =
         fileLoc.get(startLocation, Components.interfaces.nsILocalFile);
-      
+
       if (fp.show() == nsIFilePicker.returnOK && fp.file) {
-          var localHandlerApp = 
+          var localHandlerApp =
             Components.classes["@mozilla.org/uriloader/local-handler-app;1"].
             createInstance(Components.interfaces.nsILocalHandlerApp);
           localHandlerApp.executable = fp.file;
