@@ -8,9 +8,7 @@
 #include "mozIDownloadPlatform.h"
 
 #include "nsCOMPtr.h"
-
 class nsIURI;
-class nsIFile;
 
 class DownloadPlatform : public mozIDownloadPlatform
 {
@@ -28,6 +26,9 @@ public:
   static DownloadPlatform *gDownloadPlatformService;
 
   static DownloadPlatform* GetDownloadPlatform();
+
+private:
+  static bool IsURLPossiblyFromWeb(nsIURI* aURI);
 };
 
 #endif
