@@ -7,8 +7,7 @@
 
 "use strict";
 
-////////////////////////////////////////////////////////////////////////////////
-//// Globals
+// Globals
 
 XPCOMUtils.defineLazyModuleGetter(this, "Sqlite",
                                   "resource://gre/modules/Sqlite.jsm");
@@ -49,8 +48,8 @@ const TEST_DATA_PARTIAL_LENGTH = TEST_DATA_REPLACEMENT.length;
 // is seen when expected.
 const MAXBYTES_IN_DB = TEST_DATA_LENGTH - 10;
 
-let gDownloadsRowToImport;
-let gDownloadsRowNonImportable;
+var gDownloadsRowToImport;
+var gDownloadsRowNonImportable;
 
 /**
  * Creates a database with an empty moz_downloads table and leaves an
@@ -66,7 +65,7 @@ let gDownloadsRowNonImportable;
  * @rejects If an error occurred during the database creation.
  */
 function promiseEmptyDatabaseConnection({aPath, aSchemaVersion}) {
-  return Task.spawn(function () {
+  return Task.spawn(function* () {
     let connection = yield Sqlite.openConnection({ path: aPath });
 
     yield connection.execute("CREATE TABLE moz_downloads ("
@@ -90,7 +89,7 @@ function promiseEmptyDatabaseConnection({aPath, aSchemaVersion}) {
 
     yield connection.setSchemaVersion(aSchemaVersion);
 
-    throw new Task.Result(connection);
+    return connection;
   });
 }
 
@@ -126,10 +125,10 @@ function promiseInsertRow(aConnection, aDownloadRow) {
                             + "maxBytes, mimeType, preferredApplication,"
                             + "preferredAction, autoResume, guid)"
                             + "VALUES ("
-                            + "'', ?, ?, ?, ?, " //name,
-                            + "0, ?, ?, ?, 0, "  //endTime, currBytes
+                            + "'', ?, ?, ?, ?, " // name,
+                            + "0, ?, ?, ?, 0, "  // endTime, currBytes
                             + " ?, ?, ?, "       //
-                            + " ?, ?, '')",      //and guid are not imported
+                            + " ?, ?, '')",      // and guid are not imported
                             values);
 }
 
@@ -164,16 +163,12 @@ function promiseTableCount(aConnection) {
 function promiseEntityID(aUrl) {
   let deferred = Promise.defer();
   let entityID = "";
-  let channel = NetUtil.newChannel2(NetUtil.newURI(aUrl),
-                                    null,
-                                    null,
-                                    null,      // aLoadingNode
-                                    Services.scriptSecurityManager.getSystemPrincipal(),
-                                    null,      // aTriggeringPrincipal
-                                    Ci.nsILoadInfo.SEC_NORMAL,
-                                    Ci.nsIContentPolicy.TYPE_OTHER);
+  let channel = NetUtil.newChannel({
+    uri: NetUtil.newURI(aUrl),
+    loadUsingSystemPrincipal: true
+  });
 
-  channel.asyncOpen({
+  channel.asyncOpen2({
     onStartRequest: function (aRequest) {
       if (aRequest instanceof Ci.nsIResumableChannel) {
         entityID = aRequest.entityID;
@@ -190,7 +185,7 @@ function promiseEntityID(aUrl) {
     },
 
     onDataAvailable: function () {}
-  }, null);
+  });
 
   return deferred.promise;
 }
@@ -269,7 +264,7 @@ function getStartTime(aOffset) {
  * @rejects Never
  */
 function checkDownload(aDownload, aDownloadRow) {
-  return Task.spawn(function() {
+  return Task.spawn(function*() {
     do_check_eq(aDownload.source.url, aDownloadRow.source);
     do_check_eq(aDownload.source.referrer, aDownloadRow.referrer);
 
@@ -326,14 +321,13 @@ function checkDownload(aDownload, aDownloadRow) {
   });
 }
 
-////////////////////////////////////////////////////////////////////////////////
-//// Preparation tasks
+// Preparation tasks
 
 /**
  * Prepares the list of downloads to be added to the database that should
  * be imported by the import procedure.
  */
-add_task(function prepareDownloadsToImport() {
+add_task(function* prepareDownloadsToImport() {
 
   let sourceUrl = httpUrl("source.txt");
   let sourceEntityId = yield promiseEntityID(sourceUrl);
@@ -539,7 +533,7 @@ add_task(function prepareDownloadsToImport() {
  * Prepares the list of downloads to be added to the database that should
  * *not* be imported by the import procedure.
  */
-add_task(function prepareNonImportableDownloads()
+add_task(function* prepareNonImportableDownloads()
 {
   gDownloadsRowNonImportable = [
     // Download with no source (should never happen in normal circumstances).
@@ -656,15 +650,14 @@ add_task(function prepareNonImportableDownloads()
   ];
 });
 
-////////////////////////////////////////////////////////////////////////////////
-//// Test
+// Test
 
 /**
  * Creates a temporary Sqlite database with download data and perform an
  * import of that data to the new Downloads API to verify that the import
  * worked correctly.
  */
-add_task(function test_downloadImport()
+add_task(function* test_downloadImport()
 {
   let connection = null;
   let downloadsSqlite = getTempFile("downloads.sqlite").path;
