@@ -1,16 +1,11 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set sw=2 ts=2 et lcs=trail\:.,tab\:>~ : */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/**
+/*
  * nsILoginManagerStorage implementation for the JSON back-end.
  */
 
-////////////////////////////////////////////////////////////////////////////////
-//// Globals
-
 "use strict";
 
 const { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;
@@ -32,16 +27,13 @@ XPCOMUtils.defineLazyServiceGetter(this, "gUUIDGenerator",
                                    "@mozilla.org/uuid-generator;1",
                                    "nsIUUIDGenerator");
 
-////////////////////////////////////////////////////////////////////////////////
-//// LoginManagerStorage_json
-
-this.LoginManagerStorage_json = function () {}
+this.LoginManagerStorage_json = function () {};
 
 this.LoginManagerStorage_json.prototype = {
-  classID : Components.ID("{c00c432d-a0c9-46d7-bef6-9c45b4d07341}"),
-  QueryInterface : XPCOMUtils.generateQI([Ci.nsILoginManagerStorage]),
+  classID: Components.ID("{c00c432d-a0c9-46d7-bef6-9c45b4d07341}"),
+  QueryInterface: XPCOMUtils.generateQI([Ci.nsILoginManagerStorage]),
 
-  __crypto : null,  // nsILoginManagerCrypto service
+  __crypto: null,  // nsILoginManagerCrypto service
   get _crypto() {
     if (!this.__crypto)
       this.__crypto = Cc["@mozilla.org/login-manager/crypto/SDR;1"].
@@ -49,27 +41,7 @@ this.LoginManagerStorage_json.prototype = {
     return this.__crypto;
   },
 
-  /*
-   * log
-   *
-   * Internal function for logging debug messages to the Error Console.
-   */
-  log : function (message) {
-    if (!this._debug)
-      return;
-    dump("PwMgr json: " + message + "\n");
-    Services.console.logStringMessage("PwMgr json: " + message);
-  },
-  _debug : false,
-
-
-  /*
-   * initialize
-   *
-   */
-  initialize : function () {
-    this._debug = Services.prefs.getBoolPref("signon.debug");
-
+  initialize() {
     try {
       // Force initialization of the crypto module.
       // See bug 717490 comment 17.
@@ -80,9 +52,9 @@ this.LoginManagerStorage_json.prototype = {
                                   "logins.json");
       this._store = new LoginStore(jsonPath);
 
-      return Task.spawn(function () {
+      return Task.spawn(function* () {
         // Load the data asynchronously.
-        this.log("Opening database at " + this._store.path);
+        this.log("Opening database at", this._store.path);
         yield this._store.load();
 
         // The import from previous versions operates the first time
@@ -114,29 +86,21 @@ this.LoginManagerStorage_json.prototype = {
         Services.prefs.setBoolPref("signon.importedFromSqlite", true);
       }.bind(this)).catch(Cu.reportError);
     } catch (e) {
-      this.log("Initialization failed: " + e);
-      throw "Initialization failed";
+      this.log("Initialization failed:", e);
+      throw new Error("Initialization failed");
     }
   },
 
-
-  /*
-   * terminate
-   *
+  /**
    * Internal method used by regression tests only.  It is called before
    * replacing this storage module with a new instance.
    */
-  terminate : function () {
+  terminate() {
     this._store._saver.disarm();
-    return this._store.save();
+    return this._store._save();
   },
 
-
-  /*
-   * addLogin
-   *
-   */
-  addLogin : function (login) {
+  addLogin(login) {
     this._store.ensureDataReady();
 
     // Throws if there are bogus values.
@@ -151,7 +115,7 @@ this.LoginManagerStorage_json.prototype = {
     loginClone.QueryInterface(Ci.nsILoginMetaInfo);
     if (loginClone.guid) {
       if (!this._isGuidUnique(loginClone.guid))
-        throw "specified GUID already exists";
+        throw new Error("specified GUID already exists");
     } else {
       loginClone.guid = gUUIDGenerator.generateUUID().toString();
     }
@@ -186,20 +150,16 @@ this.LoginManagerStorage_json.prototype = {
     this._store.saveSoon();
 
     // Send a notification that a login was added.
-    this._sendNotification("addLogin", loginClone);
+    LoginHelper.notifyStorageChanged("addLogin", loginClone);
+    return loginClone;
   },
 
-
-  /*
-   * removeLogin
-   *
-   */
-  removeLogin : function (login) {
+  removeLogin(login) {
     this._store.ensureDataReady();
 
     let [idToDelete, storedLogin] = this._getIdForLogin(login);
     if (!idToDelete)
-      throw "No matching logins";
+      throw new Error("No matching logins");
 
     let foundIndex = this._store.data.logins.findIndex(l => l.id == idToDelete);
     if (foundIndex != -1) {
@@ -207,28 +167,22 @@ this.LoginManagerStorage_json.prototype = {
       this._store.saveSoon();
     }
 
-    this._sendNotification("removeLogin", storedLogin);
+    LoginHelper.notifyStorageChanged("removeLogin", storedLogin);
   },
 
-
-  /*
-   * modifyLogin
-   *
-   */
-  modifyLogin : function (oldLogin, newLoginData) {
+  modifyLogin(oldLogin, newLoginData) {
     this._store.ensureDataReady();
 
     let [idToModify, oldStoredLogin] = this._getIdForLogin(oldLogin);
     if (!idToModify)
-      throw "No matching logins";
+      throw new Error("No matching logins");
 
     let newLogin = LoginHelper.buildModifiedLogin(oldStoredLogin, newLoginData);
 
     // Check if the new GUID is duplicate.
     if (newLogin.guid != oldStoredLogin.guid &&
-        !this._isGuidUnique(newLogin.guid))
-    {
-      throw "specified GUID already exists";
+        !this._isGuidUnique(newLogin.guid)) {
+      throw new Error("specified GUID already exists");
     }
 
     // Look for an existing entry in case key properties changed.
@@ -238,7 +192,7 @@ this.LoginManagerStorage_json.prototype = {
                                    newLogin.httpRealm);
 
       if (logins.some(login => newLogin.matches(login, true)))
-        throw "This login already exists.";
+        throw new Error("This login already exists.");
     }
 
     // Get the encrypted value of the username and password.
@@ -264,46 +218,51 @@ this.LoginManagerStorage_json.prototype = {
       }
     }
 
-    this._sendNotification("modifyLogin", [oldStoredLogin, newLogin]);
+    LoginHelper.notifyStorageChanged("modifyLogin", [oldStoredLogin, newLogin]);
   },
 
-
-  /*
-   * getAllLogins
-   *
-   * Returns an array of nsILoginInfo.
+  /**
+   * @return {nsILoginInfo[]}
    */
-  getAllLogins : function (count) {
+  getAllLogins(count) {
     let [logins, ids] = this._searchLogins({});
 
     // decrypt entries for caller.
     logins = this._decryptLogins(logins);
 
-    this.log("_getAllLogins: returning " + logins.length + " logins.");
+    this.log("_getAllLogins: returning", logins.length, "logins.");
     if (count)
       count.value = logins.length; // needed for XPCOM
     return logins;
   },
 
-
-  /*
-   * searchLogins
-   *
+  /**
    * Public wrapper around _searchLogins to convert the nsIPropertyBag to a
    * JavaScript object and decrypt the results.
    *
-   * Returns an array of decrypted nsILoginInfo.
+   * @return {nsILoginInfo[]} which are decrypted.
    */
-  searchLogins : function(count, matchData) {
+  searchLogins(count, matchData) {
     let realMatchData = {};
+    let options = {};
     // Convert nsIPropertyBag to normal JS object
     let propEnum = matchData.enumerator;
     while (propEnum.hasMoreElements()) {
       let prop = propEnum.getNext().QueryInterface(Ci.nsIProperty);
-      realMatchData[prop.name] = prop.value;
+      switch (prop.name) {
+        // Some property names aren't field names but are special options to affect the search.
+        case "schemeUpgrades": {
+          options[prop.name] = prop.value;
+          break;
+        }
+        default: {
+          realMatchData[prop.name] = prop.value;
+          break;
+        }
+      }
     }
 
-    let [logins, ids] = this._searchLogins(realMatchData);
+    let [logins, ids] = this._searchLogins(realMatchData, options);
 
     // Decrypt entries found for the caller.
     logins = this._decryptLogins(logins);
@@ -312,10 +271,7 @@ this.LoginManagerStorage_json.prototype = {
     return logins;
   },
 
-
-  /*
-   * _searchLogins
-   *
+  /**
    * Private method to perform arbitrary searches on any field. Decryption is
    * left to the caller.
    *
@@ -323,25 +279,36 @@ this.LoginManagerStorage_json.prototype = {
    * is an array of encrypted nsLoginInfo and ids is an array of associated
    * ids in the database.
    */
-  _searchLogins : function (matchData) {
+  _searchLogins(matchData, aOptions = {
+    schemeUpgrades: false,
+  }) {
     this._store.ensureDataReady();
 
-    let conditions = [];
-
     function match(aLogin) {
       for (let field in matchData) {
-        let value = matchData[field];
+        let wantedValue = matchData[field];
         switch (field) {
-          // Historical compatibility requires this special case
           case "formSubmitURL":
-            if (value != null) {
-              if (aLogin.formSubmitURL != "" && aLogin.formSubmitURL != value) {
+            if (wantedValue != null) {
+              // Historical compatibility requires this special case
+              if (aLogin.formSubmitURL == "") {
+                break;
+              }
+              if (!LoginHelper.isOriginMatching(aLogin[field], wantedValue, aOptions)) {
                 return false;
               }
               break;
             }
-          // Normal cases.
+            // fall through
           case "hostname":
+            if (wantedValue != null) { // needed for formSubmitURL fall through
+              if (!LoginHelper.isOriginMatching(aLogin[field], wantedValue, aOptions)) {
+                return false;
+              }
+              break;
+            }
+            // fall through
+          // Normal cases.
           case "httpRealm":
           case "id":
           case "usernameField":
@@ -354,15 +321,15 @@ this.LoginManagerStorage_json.prototype = {
           case "timeLastUsed":
           case "timePasswordChanged":
           case "timesUsed":
-            if (value == null && aLogin[field]) {
+            if (wantedValue == null && aLogin[field]) {
               return false;
-            } else if (aLogin[field] != value) {
+            } else if (aLogin[field] != wantedValue) {
               return false;
             }
             break;
           // Fail if caller requests an unknown property.
           default:
-            throw "Unexpected field: " + field;
+            throw new Error("Unexpected field: " + field);
         }
       }
       return true;
@@ -390,96 +357,32 @@ this.LoginManagerStorage_json.prototype = {
       }
     }
 
-    this.log("_searchLogins: returning " + foundLogins.length + " logins");
+    this.log("_searchLogins: returning", foundLogins.length, "logins for", matchData,
+             "with options", aOptions);
     return [foundLogins, foundIds];
   },
 
-  /*
-   * removeAllLogins
-   *
+  /**
    * Removes all logins from storage.
-   *
-   * Disabled hosts are kept, as one presumably doesn't want to erase those.
    */
-  removeAllLogins : function () {
+  removeAllLogins() {
     this._store.ensureDataReady();
 
     this.log("Removing all logins");
     this._store.data.logins = [];
     this._store.saveSoon();
 
-    this._sendNotification("removeAllLogins", null);
+    LoginHelper.notifyStorageChanged("removeAllLogins", null);
   },
 
-
-  /*
-   * getAllDisabledHosts
-   *
-   */
-  getAllDisabledHosts : function (count) {
-    this._store.ensureDataReady();
-
-    let disabledHosts = this._store.data.disabledHosts.slice(0);
-
-    this.log("_getAllDisabledHosts: returning " + disabledHosts.length + " disabled hosts.");
-    if (count)
-      count.value = disabledHosts.length; // needed for XPCOM
-    return disabledHosts;
-  },
-
-
-  /*
-   * getLoginSavingEnabled
-   *
-   */
-  getLoginSavingEnabled : function (hostname) {
-    this._store.ensureDataReady();
-
-    this.log("Getting login saving is enabled for " + hostname);
-    return this._store.data.disabledHosts.indexOf(hostname) == -1;
-  },
-
-
-  /*
-   * setLoginSavingEnabled
-   *
-   */
-  setLoginSavingEnabled : function (hostname, enabled) {
-    this._store.ensureDataReady();
-
-    // Throws if there are bogus values.
-    LoginHelper.checkHostnameValue(hostname);
-
-    this.log("Setting login saving enabled for " + hostname + " to " + enabled);
-    let foundIndex = this._store.data.disabledHosts.indexOf(hostname);
-    if (enabled) {
-      if (foundIndex != -1) {
-        this._store.data.disabledHosts.splice(foundIndex, 1);
-        this._store.saveSoon();
-      }
-    } else {
-      if (foundIndex == -1) {
-        this._store.data.disabledHosts.push(hostname);
-        this._store.saveSoon();
-      }
-    }
-
-    this._sendNotification(enabled ? "hostSavingEnabled" : "hostSavingDisabled", hostname);
-  },
-
-
-  /*
-   * findLogins
-   *
-   */
-  findLogins : function (count, hostname, formSubmitURL, httpRealm) {
+  findLogins(count, hostname, formSubmitURL, httpRealm) {
     let loginData = {
       hostname: hostname,
       formSubmitURL: formSubmitURL,
       httpRealm: httpRealm
     };
     let matchData = { };
-    for each (let field in ["hostname", "formSubmitURL", "httpRealm"])
+    for (let field of ["hostname", "formSubmitURL", "httpRealm"])
       if (loginData[field] != '')
         matchData[field] = loginData[field];
     let [logins, ids] = this._searchLogins(matchData);
@@ -487,82 +390,43 @@ this.LoginManagerStorage_json.prototype = {
     // Decrypt entries found for the caller.
     logins = this._decryptLogins(logins);
 
-    this.log("_findLogins: returning " + logins.length + " logins");
+    this.log("_findLogins: returning", logins.length, "logins");
     count.value = logins.length; // needed for XPCOM
     return logins;
   },
 
-
-  /*
-   * countLogins
-   *
-   */
-  countLogins : function (hostname, formSubmitURL, httpRealm) {
-    let count = {};
+  countLogins(hostname, formSubmitURL, httpRealm) {
     let loginData = {
       hostname: hostname,
       formSubmitURL: formSubmitURL,
       httpRealm: httpRealm
     };
     let matchData = { };
-    for each (let field in ["hostname", "formSubmitURL", "httpRealm"])
+    for (let field of ["hostname", "formSubmitURL", "httpRealm"])
       if (loginData[field] != '')
         matchData[field] = loginData[field];
     let [logins, ids] = this._searchLogins(matchData);
 
-    this.log("_countLogins: counted logins: " + logins.length);
+    this.log("_countLogins: counted logins:", logins.length);
     return logins.length;
   },
 
-
-  /*
-   * uiBusy
-   */
   get uiBusy() {
     return this._crypto.uiBusy;
   },
 
-
-  /*
-   * isLoggedIn
-   */
   get isLoggedIn() {
     return this._crypto.isLoggedIn;
   },
 
-
-  /*
-   * _sendNotification
-   *
-   * Send a notification when stored data is changed.
-   */
-  _sendNotification : function (changeType, data) {
-    let dataObject = data;
-    // Can't pass a raw JS string or array though notifyObservers(). :-(
-    if (data instanceof Array) {
-      dataObject = Cc["@mozilla.org/array;1"].
-                   createInstance(Ci.nsIMutableArray);
-      for (let i = 0; i < data.length; i++)
-        dataObject.appendElement(data[i], false);
-    } else if (typeof(data) == "string") {
-      dataObject = Cc["@mozilla.org/supports-string;1"].
-                   createInstance(Ci.nsISupportsString);
-      dataObject.data = data;
-    }
-    Services.obs.notifyObservers(dataObject, "passwordmgr-storage-changed", changeType);
-  },
-
-
-  /*
-   * _getIdForLogin
-   *
+  /**
    * Returns an array with two items: [id, login]. If the login was not
    * found, both items will be null. The returned login contains the actual
    * stored login (useful for looking at the actual nsILoginMetaInfo values).
    */
-  _getIdForLogin : function (login) {
+  _getIdForLogin(login) {
     let matchData = { };
-    for each (let field in ["hostname", "formSubmitURL", "httpRealm"])
+    for (let field of ["hostname", "formSubmitURL", "httpRealm"])
       if (login[field] != '')
         matchData[field] = login[field];
     let [logins, ids] = this._searchLogins(matchData);
@@ -589,26 +453,20 @@ this.LoginManagerStorage_json.prototype = {
     return [id, foundLogin];
   },
 
-
-  /*
-   * _isGuidUnique
-   *
+  /**
    * Checks to see if the specified GUID already exists.
    */
-  _isGuidUnique : function (guid) {
+  _isGuidUnique(guid) {
     this._store.ensureDataReady();
 
     return this._store.data.logins.every(l => l.guid != guid);
   },
 
-
-  /*
-   * _encryptLogin
-   *
+  /**
    * Returns the encrypted username, password, and encrypton type for the specified
    * login. Can throw if the user cancels a master password entry.
    */
-  _encryptLogin : function (login) {
+  _encryptLogin(login) {
     let encUsername = this._crypto.encrypt(login.username);
     let encPassword = this._crypto.encrypt(login.password);
     let encType     = this._crypto.defaultEncType;
@@ -616,10 +474,7 @@ this.LoginManagerStorage_json.prototype = {
     return [encUsername, encPassword, encType];
   },
 
-
-  /*
-   * _decryptLogins
-   *
+  /**
    * Decrypts username and password fields in the provided array of
    * logins.
    *
@@ -630,10 +485,10 @@ this.LoginManagerStorage_json.prototype = {
    * to lose unencrypted entries (eg, because the user clicked Cancel
    * instead of entering their master password)
    */
-  _decryptLogins : function (logins) {
+  _decryptLogins(logins) {
     let result = [];
 
-    for each (let login in logins) {
+    for (let login of logins) {
       try {
         login.username = this._crypto.decrypt(login.username);
         login.password = this._crypto.decrypt(login.password);
@@ -651,4 +506,9 @@ this.LoginManagerStorage_json.prototype = {
   },
 };
 
+XPCOMUtils.defineLazyGetter(this.LoginManagerStorage_json.prototype, "log", () => {
+  let logger = LoginHelper.createLogger("Login storage");
+  return logger.log.bind(logger);
+});
+
 this.NSGetFactory = XPCOMUtils.generateNSGetFactory([LoginManagerStorage_json]);
