@@ -1,19 +1,14 @@
-/* Any copyright is dedicated to the Public Domain.
-   http://creativecommons.org/publicdomain/zero/1.0/ */
-
 /**
  * This file tests the async history API exposed by mozIAsyncHistory.
  */
 
-////////////////////////////////////////////////////////////////////////////////
-//// Globals
+// Globals
 
 const TEST_DOMAIN = "http://mozilla.org/";
 const URI_VISIT_SAVED = "uri-visit-saved";
 const RECENT_EVENT_THRESHOLD = 15 * 60 * 1000000;
 
-////////////////////////////////////////////////////////////////////////////////
-//// Helpers
+// Helpers
 /**
  * Object that represents a mozIVisitInfo object.
  *
@@ -108,7 +103,7 @@ VisitObserver.prototype = {
   {
     do_print("onVisit(" + aURI.spec + ", " + aVisitId + ", " + aTime +
              ", " + aSessionId + ", " + aReferringId + ", " +
-             aTransitionType + ", " + aGUID + ")"); 
+             aTransitionType + ", " + aGUID + ")");
     if (!this.uri.equals(aURI) || this.guid != aGUID) {
       return;
     }
@@ -131,7 +126,7 @@ function do_check_title_for_uri(aURI,
   let stmt = DBConn().createStatement(
     `SELECT title
      FROM moz_places
-     WHERE url = :url`
+     WHERE url_hash = hash(:url) AND url = :url`
   );
   stmt.params.url = aURI.spec;
   do_check_true(stmt.executeStep(), stack);
@@ -139,8 +134,7 @@ function do_check_title_for_uri(aURI,
   stmt.finalize();
 }
 
-////////////////////////////////////////////////////////////////////////////////
-//// Test Functions
+// Test Functions
 
 add_task(function* test_interface_exists() {
   let history = Cc["@mozilla.org/browser/history;1"].getService(Ci.nsISupports);
@@ -245,7 +239,6 @@ add_task(function* test_no_visits_throws() {
   const TEST_URI =
     NetUtil.newURI(TEST_DOMAIN + "test_no_id_or_guid_no_visits_throws");
   const TEST_GUID = "_RANDOMGUID_";
-  const TEST_PLACEID = 2;
 
   let log_test_conditions = function(aPlace) {
     let str = "Testing place with " +
@@ -333,7 +326,7 @@ add_task(function* test_add_visit_invalid_transitionType_throws() {
   }
 
   // Now, test something that has a transition type greater than the last one.
-  place.visits[0] = new VisitInfo(TRANSITION_FRAMED_LINK + 1);
+  place.visits[0] = new VisitInfo(TRANSITION_RELOAD + 1);
   try {
     yield promiseUpdatePlaces(place);
     do_throw("Should have thrown!");
@@ -371,7 +364,10 @@ add_task(function* test_non_addable_uri_errors() {
       };
       places.push(place);
     }
-    catch (e if e.result === Cr.NS_ERROR_FAILURE) {
+    catch (e) {
+      if (e.result != Cr.NS_ERROR_FAILURE) {
+        throw e;
+      }
       // NetUtil.newURI() can throw if e.g. our app knows about imap://
       // but the account is not set up and so the URL is invalid for us.
       // Note this in the log but ignore as it's not the subject of this test.
@@ -545,7 +541,8 @@ add_task(function* test_old_referrer_ignored() {
   let stmt = DBConn().createStatement(
     `SELECT COUNT(1) AS count
      FROM moz_historyvisits
-     WHERE place_id = (SELECT id FROM moz_places WHERE url = :page_url)
+     JOIN moz_places h ON h.id = place_id
+     WHERE url_hash = hash(:page_url) AND url = :page_url
      AND from_visit = 0`
   );
   stmt.params.page_url = place.uri.spec;
@@ -619,21 +616,12 @@ add_task(function* test_handleCompletion_called_when_complete() {
 
   const EXPECTED_COUNT_SUCCESS = 2;
   const EXPECTED_COUNT_FAILURE = 1;
-  let callbackCountSuccess = 0;
-  let callbackCountFailure = 0;
 
-  let placesResult = yield promiseUpdatePlaces(places);
-  for (let place of placesResult.results) {
-    let checker = PlacesUtils.history.canAddURI(place.uri) ?
-      do_check_true : do_check_false;
-    callbackCountSuccess++;
-  }
-  for (let error of placesResult.errors) {
-    callbackCountFailure++;
-  }
+  let {results, errors} = yield promiseUpdatePlaces(places);
+
+  do_check_eq(results.length, EXPECTED_COUNT_SUCCESS);
+  do_check_eq(errors.length, EXPECTED_COUNT_FAILURE);
 
-  do_check_eq(callbackCountSuccess, EXPECTED_COUNT_SUCCESS);
-  do_check_eq(callbackCountFailure, EXPECTED_COUNT_FAILURE);
   yield PlacesTestUtils.promiseAsyncUpdates();
 });
 
@@ -644,9 +632,8 @@ add_task(function* test_add_visit() {
     title: "test_add_visit title",
     visits: [],
   };
-  for (let transitionType = TRANSITION_LINK;
-       transitionType <= TRANSITION_FRAMED_LINK;
-       transitionType++) {
+  for (let t in PlacesUtils.history.TRANSITIONS) {
+    let transitionType = PlacesUtils.history.TRANSITIONS[t];
     place.visits.push(new VisitInfo(transitionType, VISIT_TIME));
   }
   do_check_false(yield promiseIsURIVisited(place.uri));
@@ -669,8 +656,7 @@ add_task(function* test_add_visit() {
     do_check_eq(visits.length, 1);
     let visit = visits[0];
     do_check_eq(visit.visitDate, VISIT_TIME);
-    do_check_true(visit.transitionType >= TRANSITION_LINK &&
-                    visit.transitionType <= TRANSITION_FRAMED_LINK);
+    do_check_true(Object.values(PlacesUtils.history.TRANSITIONS).includes(visit.transitionType));
     do_check_true(visit.referrerURI === null);
 
     // For TRANSITION_EMBED visits, many properties will always be zero or
@@ -703,9 +689,8 @@ add_task(function* test_add_visit() {
 add_task(function* test_properties_saved() {
   // Check each transition type to make sure it is saved properly.
   let places = [];
-  for (let transitionType = TRANSITION_LINK;
-       transitionType <= TRANSITION_FRAMED_LINK;
-       transitionType++) {
+  for (let t in PlacesUtils.history.TRANSITIONS) {
+    let transitionType = PlacesUtils.history.TRANSITIONS[t];
     let place = {
       uri: NetUtil.newURI(TEST_DOMAIN + "test_properties_saved/" +
                           transitionType),
@@ -739,7 +724,7 @@ add_task(function* test_properties_saved() {
        FROM moz_places h
        JOIN moz_historyvisits v
        ON h.id = v.place_id
-       WHERE h.url = :page_url
+       WHERE h.url_hash = hash(:page_url) AND h.url = :page_url
        AND v.visit_date = :visit_date`
     );
     stmt.params.page_url = uri.spec;
@@ -754,7 +739,7 @@ add_task(function* test_properties_saved() {
        FROM moz_places h
        JOIN moz_historyvisits v
        ON h.id = v.place_id
-       WHERE h.url = :page_url
+       WHERE h.url_hash = hash(:page_url) AND h.url = :page_url
        AND v.visit_type = :transition_type`
     );
     stmt.params.page_url = uri.spec;
@@ -767,7 +752,7 @@ add_task(function* test_properties_saved() {
     stmt = DBConn().createStatement(
       `SELECT COUNT(1) AS count
        FROM moz_places h
-       WHERE h.url = :page_url
+       WHERE h.url_hash = hash(:page_url) AND h.url = :page_url
        AND h.title = :title`
     );
     stmt.params.page_url = uri.spec;
@@ -840,11 +825,13 @@ add_task(function* test_referrer_saved() {
       let stmt = DBConn().createStatement(
         `SELECT COUNT(1) AS count
          FROM moz_historyvisits
-         WHERE place_id = (SELECT id FROM moz_places WHERE url = :page_url)
+         JOIN moz_places h ON h.id = place_id
+         WHERE url_hash = hash(:page_url) AND url = :page_url
          AND from_visit = (
-           SELECT id
-           FROM moz_historyvisits
-           WHERE place_id = (SELECT id FROM moz_places WHERE url = :referrer)
+           SELECT v.id
+           FROM moz_historyvisits v
+           JOIN moz_places h ON h.id = place_id
+           WHERE url_hash = hash(:referrer) AND url = :referrer
          )`
       );
       stmt.params.page_url = uri.spec;
@@ -1002,7 +989,7 @@ add_task(function* test_title_change_notifies() {
             PlacesUtils.history.removeObserver(observer);
             resolve();
             break;
-        };
+        }
       });
 
       PlacesUtils.history.addObserver(observer, false);
@@ -1080,7 +1067,10 @@ add_task(function* test_callbacks_not_supplied() {
       };
       places.push(place);
     }
-    catch (e if e.result === Cr.NS_ERROR_FAILURE) {
+    catch (e) {
+      if (e.result != Cr.NS_ERROR_FAILURE) {
+        throw e;
+      }
       // NetUtil.newURI() can throw if e.g. our app knows about imap://
       // but the account is not set up and so the URL is invalid for us.
       // Note this in the log but ignore as it's not the subject of this test.
@@ -1113,8 +1103,9 @@ add_task(function* test_typed_hidden_not_overwritten() {
   yield promiseUpdatePlaces(places);
 
   let db = yield PlacesUtils.promiseDBConnection();
-  let rows = yield db.execute("SELECT hidden, typed FROM moz_places WHERE url = :url",
-                              { url: "http://mozilla.org/" });
+  let rows = yield db.execute(
+    "SELECT hidden, typed FROM moz_places WHERE url_hash = hash(:url) AND url = :url",
+    { url: "http://mozilla.org/" });
   Assert.equal(rows[0].getResultByName("typed"), 1,
                "The page should be marked as typed");
   Assert.equal(rows[0].getResultByName("hidden"), 0,
