@@ -16,18 +16,18 @@ Components.utils.import("resource://gre/modules/PlacesDBUtils.jsm");
 const FINISHED_MAINTENANCE_NOTIFICATION_TOPIC = "places-maintenance-finished";
 
 // Get services and database connection
-let hs = PlacesUtils.history;
-let bs = PlacesUtils.bookmarks;
-let ts = PlacesUtils.tagging;
-let as = PlacesUtils.annotations;
-let fs = PlacesUtils.favicons;
+var hs = PlacesUtils.history;
+var bs = PlacesUtils.bookmarks;
+var ts = PlacesUtils.tagging;
+var as = PlacesUtils.annotations;
+var fs = PlacesUtils.favicons;
 
-let mDBConn = hs.QueryInterface(Ci.nsPIPlacesDatabase).DBConnection;
+var mDBConn = hs.QueryInterface(Ci.nsPIPlacesDatabase).DBConnection;
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 // Helpers
 
-let defaultBookmarksMaxId = 0;
+var defaultBookmarksMaxId = 0;
 function cleanDatabase() {
   mDBConn.executeSimpleSQL("DELETE FROM moz_places");
   mDBConn.executeSimpleSQL("DELETE FROM moz_historyvisits");
@@ -42,7 +42,7 @@ function cleanDatabase() {
 
 function addPlace(aUrl, aFavicon) {
   let stmt = mDBConn.createStatement(
-    "INSERT INTO moz_places (url, favicon_id) VALUES (:url, :favicon)");
+    "INSERT INTO moz_places (url, url_hash, favicon_id) VALUES (:url, hash(:url), :favicon)");
   stmt.params["url"] = aUrl || "http://www.mozilla.org";
   stmt.params["favicon"] = aFavicon || null;
   stmt.execute();
@@ -67,12 +67,12 @@ function addBookmark(aPlaceId, aType, aParent, aKeywordId, aFolderType, aTitle)
   return mDBConn.lastInsertRowID;
 }
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 // Tests
 
-let tests = [];
+var tests = [];
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "A.1",
@@ -222,7 +222,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "B.1",
@@ -269,7 +269,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "B.2",
@@ -316,7 +316,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 tests.push({
   name: "C.1",
   desc: "fix missing Places root",
@@ -346,7 +346,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 tests.push({
   name: "C.2",
   desc: "Fix roots titles",
@@ -369,13 +369,13 @@ tests.push({
     do_check_eq(bs.getItemTitle(bs.tagsFolder),
                 PlacesUtils.getString("TagsFolderTitle"));
     do_check_eq(bs.getItemTitle(bs.unfiledBookmarksFolder),
-                PlacesUtils.getString("UnsortedBookmarksFolderTitle"));
+                PlacesUtils.getString("OtherBookmarksFolderTitle"));
     do_check_eq(bs.getItemTitle(bs.toolbarFolder),
                 PlacesUtils.getString("BookmarksToolbarFolderTitle"));
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.1",
@@ -407,7 +407,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.2",
@@ -452,7 +452,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.3",
@@ -495,7 +495,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.4",
@@ -542,54 +542,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
-
-tests.push({
-  name: "D.5",
-  desc: "Fix wrong keywords",
-
-  _validKeywordItemId: null,
-  _invalidKeywordItemId: null,
-  _validKeywordId: 1,
-  _invalidKeywordId: 8888,
-  _placeId: null,
-
-  setup: function() {
-    // Insert a keyword
-    let stmt = mDBConn.createStatement("INSERT INTO moz_keywords (id, keyword) VALUES(:id, :keyword)");
-    stmt.params["id"] = this._validKeywordId;
-    stmt.params["keyword"] = "used";
-    stmt.execute();
-    stmt.finalize();
-    // Add a place to ensure place_id = 1 is valid
-    this._placeId = addPlace();
-    // Add a bookmark using the keyword
-    this._validKeywordItemId = addBookmark(this._placeId, bs.TYPE_BOOKMARK, bs.unfiledBookmarksFolder, this._validKeywordId);
-    // Add a bookmark using a nonexistent keyword
-    this._invalidKeywordItemId = addBookmark(this._placeId, bs.TYPE_BOOKMARK, bs.unfiledBookmarksFolder, this._invalidKeywordId);
-  },
-
-  check: function() {
-    // Check that item with valid keyword is there
-    let stmt = mDBConn.createStatement("SELECT id FROM moz_bookmarks WHERE id = :item_id AND keyword_id = :keyword");
-    stmt.params["item_id"] = this._validKeywordItemId;
-    stmt.params["keyword"] = this._validKeywordId;
-    do_check_true(stmt.executeStep());
-    stmt.reset();
-    // Check that item with invalid keyword has been corrected
-    stmt.params["item_id"] = this._invalidKeywordItemId;
-    stmt.params["keyword"] = this._invalidKeywordId;
-    do_check_false(stmt.executeStep());
-    stmt.finalize();
-    // Check that item with invalid keyword has not been removed
-    stmt = mDBConn.createStatement("SELECT id FROM moz_bookmarks WHERE id = :item_id");
-    stmt.params["item_id"] = this._invalidKeywordItemId;
-    do_check_true(stmt.executeStep());
-    stmt.finalize();
-  }
-});
-
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.6",
@@ -622,7 +575,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.7",
@@ -656,7 +609,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.9",
@@ -694,7 +647,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.10",
@@ -785,7 +738,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "D.12",
@@ -832,7 +785,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "E.1",
@@ -868,7 +821,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "F.1",
@@ -903,7 +856,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "G.1",
@@ -940,7 +893,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "H.1",
@@ -990,7 +943,7 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "H.2",
@@ -1042,7 +995,7 @@ tests.push({
 });
 
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "I.1",
@@ -1053,27 +1006,17 @@ tests.push({
 
   setup: function() {
     // Insert 2 keywords
-    let stmt = mDBConn.createStatement("INSERT INTO moz_keywords (id, keyword) VALUES(:id, :keyword)");
+    let stmt = mDBConn.createStatement("INSERT INTO moz_keywords (id, keyword, place_id) VALUES(:id, :keyword, :place_id)");
     stmt.params["id"] = 1;
-    stmt.params["keyword"] = "used";
-    stmt.execute();
-    stmt.reset();
-    stmt.params["id"] = 2;
     stmt.params["keyword"] = "unused";
+    stmt.params["place_id"] = 100;
     stmt.execute();
     stmt.finalize();
-    // Add a place to ensure place_id = 1 is valid
-    this._placeId = addPlace();
-    // Insert a bookmark using the "used" keyword
-    this._bookmarkId = addBookmark(this._placeId, bs.TYPE_BOOKMARK, bs.unfiledBookmarksFolder, 1);
   },
 
   check: function() {
     // Check that "used" keyword is still there
     let stmt = mDBConn.createStatement("SELECT id FROM moz_keywords WHERE keyword = :keyword");
-    stmt.params["keyword"] = "used";
-    do_check_true(stmt.executeStep());
-    stmt.reset();
     // Check that "unused" keyword has gone
     stmt.params["keyword"] = "unused";
     do_check_false(stmt.executeStep());
@@ -1082,7 +1025,7 @@ tests.push({
 });
 
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "L.1",
@@ -1125,16 +1068,17 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "L.2",
   desc: "Recalculate visit_count and last_visit_date",
 
-  setup: function() {
+  setup: function* () {
     function setVisitCount(aURL, aValue) {
       let stmt = mDBConn.createStatement(
-        "UPDATE moz_places SET visit_count = :count WHERE url = :url"
+        `UPDATE moz_places SET visit_count = :count WHERE url_hash = hash(:url)
+                                                      AND url = :url`
       );
       stmt.params.count = aValue;
       stmt.params.url = aURL;
@@ -1143,7 +1087,8 @@ tests.push({
     }
     function setLastVisitDate(aURL, aValue) {
       let stmt = mDBConn.createStatement(
-        "UPDATE moz_places SET last_visit_date = :date WHERE url = :url"
+        `UPDATE moz_places SET last_visit_date = :date WHERE url_hash = hash(:url)
+                                                         AND url = :url`
       );
       stmt.params.date = aValue;
       stmt.params.url = aURL;
@@ -1190,7 +1135,7 @@ tests.push({
   check: function() {
     let stmt = mDBConn.createStatement(
       `SELECT h.id FROM moz_places h
-       JOIN moz_historyvisits v ON v.place_id = h.id AND visit_type NOT IN (0,4,7,8)
+       JOIN moz_historyvisits v ON v.place_id = h.id AND visit_type NOT IN (0,4,7,8,9)
        GROUP BY h.id HAVING h.visit_count <> count(*)
        UNION ALL
        SELECT h.id FROM moz_places h
@@ -1202,14 +1147,14 @@ tests.push({
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "L.3",
   desc: "recalculate hidden for redirects.",
 
-  setup: function() {
-    PlacesTestUtils.addVisits([
+  *setup() {
+    yield PlacesTestUtils.addVisits([
       { uri: NetUtil.newURI("http://l3.moz.org/"),
         transition: TRANSITION_TYPED },
       { uri: NetUtil.newURI("http://l3.moz.org/redirecting/"),
@@ -1223,34 +1168,91 @@ tests.push({
     ]);
   },
 
-  asyncCheck: function(aCallback) {
-    let stmt = mDBConn.createAsyncStatement(
-      "SELECT h.url FROM moz_places h WHERE h.hidden = 1"
-    );
-    stmt.executeAsync({
-      _count: 0,
-      handleResult: function(aResultSet) {
-        for (let row; (row = aResultSet.getNextRow());) {
-          let url = row.getResultByIndex(0);
-          do_check_true(/redirecting/.test(url));
-          this._count++;
+  check: function () {
+    return new Promise(resolve => {
+      let stmt = mDBConn.createAsyncStatement(
+        "SELECT h.url FROM moz_places h WHERE h.hidden = 1"
+      );
+      stmt.executeAsync({
+        _count: 0,
+        handleResult: function(aResultSet) {
+          for (let row; (row = aResultSet.getNextRow());) {
+            let url = row.getResultByIndex(0);
+            do_check_true(/redirecting/.test(url));
+            this._count++;
+          }
+        },
+        handleError: function(aError) {
+        },
+        handleCompletion: function(aReason) {
+          dump_table("moz_places");
+          dump_table("moz_historyvisits");
+          do_check_eq(aReason, Ci.mozIStorageStatementCallback.REASON_FINISHED);
+          do_check_eq(this._count, 2);
+          resolve();
         }
-      },
-      handleError: function(aError) {
-      },
-      handleCompletion: function(aReason) {
-        dump_table("moz_places");
-        dump_table("moz_historyvisits");
-        do_check_eq(aReason, Ci.mozIStorageStatementCallback.REASON_FINISHED);
-        do_check_eq(this._count, 2);
-        aCallback();
-      }
+      });
+      stmt.finalize();
     });
-    stmt.finalize();
   }
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
+
+tests.push({
+  name: "L.4",
+  desc: "recalculate foreign_count.",
+
+  *setup() {
+    this._pageGuid = (yield PlacesUtils.history.insert({ url: "http://l4.moz.org/",
+                                                         visits: [{ date: new Date() }] })).guid;
+    yield PlacesUtils.bookmarks.insert({ url: "http://l4.moz.org/",
+                                         parentGuid: PlacesUtils.bookmarks.unfiledGuid});
+    yield PlacesUtils.keywords.insert({ url: "http://l4.moz.org/", keyword: "kw" });
+    Assert.equal((yield this._getForeignCount()), 2);
+  },
+
+  *_getForeignCount() {
+    let db = yield PlacesUtils.promiseDBConnection();
+    let rows = yield db.execute(`SELECT foreign_count FROM moz_places
+                                 WHERE guid = :guid`, { guid: this._pageGuid });
+    return rows[0].getResultByName("foreign_count");
+  },
+
+  *check() {
+    Assert.equal((yield this._getForeignCount()), 2);
+  }
+});
+
+// ------------------------------------------------------------------------------
+
+tests.push({
+  name: "L.5",
+  desc: "recalculate hashes when missing.",
+
+  *setup() {
+    this._pageGuid = (yield PlacesUtils.history.insert({ url: "http://l5.moz.org/",
+                                                         visits: [{ date: new Date() }] })).guid;
+    Assert.ok((yield this._getHash()) > 0);
+    yield PlacesUtils.withConnectionWrapper("change url hash", Task.async(function* (db) {
+      yield db.execute(`UPDATE moz_places SET url_hash = 0`);
+    }));
+    Assert.equal((yield this._getHash()), 0);
+  },
+
+  *_getHash() {
+    let db = yield PlacesUtils.promiseDBConnection();
+    let rows = yield db.execute(`SELECT url_hash FROM moz_places
+                                 WHERE guid = :guid`, { guid: this._pageGuid });
+    return rows[0].getResultByName("url_hash");
+  },
+
+  *check() {
+    Assert.ok((yield this._getHash()) > 0);
+  }
+});
+
+// ------------------------------------------------------------------------------
 
 tests.push({
   name: "Z",
@@ -1262,7 +1264,7 @@ tests.push({
   _bookmarkId: null,
   _separatorId: null,
 
-  setup: function() {
+  setup: function* () {
     // use valid api calls to create a bunch of items
     yield PlacesTestUtils.addVisits([
       { uri: this._uri1 },
@@ -1280,54 +1282,44 @@ tests.push({
     do_check_true(this._separatorId > 0);
     ts.tagURI(this._uri1, ["testtag"]);
     fs.setAndFetchFaviconForPage(this._uri2, SMALLPNG_DATA_URI, false,
-                                 PlacesUtils.favicons.FAVICON_LOAD_NON_PRIVATE);
-    bs.setKeywordForBookmark(this._bookmarkId, "testkeyword");
+                                 PlacesUtils.favicons.FAVICON_LOAD_NON_PRIVATE,
+                                 null,
+                                 Services.scriptSecurityManager.getSystemPrincipal());
+    yield PlacesUtils.keywords.insert({ url: this._uri1.spec, keyword: "testkeyword" });
     as.setPageAnnotation(this._uri2, "anno", "anno", 0, as.EXPIRE_NEVER);
     as.setItemAnnotation(this._bookmarkId, "anno", "anno", 0, as.EXPIRE_NEVER);
   },
 
-  asyncCheck: function (aCallback) {
+  check: Task.async(function* () {
     // Check that all items are correct
-    PlacesUtils.asyncHistory.isURIVisited(this._uri1, function(aURI, aIsVisited) {
-      do_check_true(aIsVisited);
-      PlacesUtils.asyncHistory.isURIVisited(this._uri2, function(aURI, aIsVisited) {
-        do_check_true(aIsVisited);
+    let isVisited = yield promiseIsURIVisited(this._uri1);
+    do_check_true(isVisited);
+    isVisited = yield promiseIsURIVisited(this._uri2);
+    do_check_true(isVisited);
 
-        do_check_eq(bs.getBookmarkURI(this._bookmarkId).spec, this._uri1.spec);
-        do_check_eq(bs.getItemIndex(this._folderId), 0);
+    do_check_eq(bs.getBookmarkURI(this._bookmarkId).spec, this._uri1.spec);
+    do_check_eq(bs.getItemIndex(this._folderId), 0);
+    do_check_eq(bs.getItemType(this._folderId), bs.TYPE_FOLDER);
+    do_check_eq(bs.getItemType(this._separatorId), bs.TYPE_SEPARATOR);
 
-        do_check_eq(bs.getItemType(this._folderId), bs.TYPE_FOLDER);
-        do_check_eq(bs.getItemType(this._separatorId), bs.TYPE_SEPARATOR);
+    do_check_eq(ts.getTagsForURI(this._uri1).length, 1);
+    do_check_eq((yield PlacesUtils.keywords.fetch({ url: this._uri1.spec })).keyword, "testkeyword");
+    do_check_eq(as.getPageAnnotation(this._uri2, "anno"), "anno");
+    do_check_eq(as.getItemAnnotation(this._bookmarkId, "anno"), "anno");
 
-        do_check_eq(ts.getTagsForURI(this._uri1).length, 1);
-        do_check_eq(bs.getKeywordForBookmark(this._bookmarkId), "testkeyword");
-        do_check_eq(as.getPageAnnotation(this._uri2, "anno"), "anno");
-        do_check_eq(as.getItemAnnotation(this._bookmarkId, "anno"), "anno");
-
-        fs.getFaviconURLForPage(this._uri2, function (aFaviconURI) {
-          do_check_true(aFaviconURI.equals(SMALLPNG_DATA_URI));
-          aCallback();
-        });
-      }.bind(this));
-    }.bind(this));
-  }
+    yield new Promise(resolve => {
+      fs.getFaviconURLForPage(this._uri2, aFaviconURI => {
+        do_check_true(aFaviconURI.equals(SMALLPNG_DATA_URI));
+        resolve();
+      });
+    });
+  })
 });
 
-//------------------------------------------------------------------------------
+// ------------------------------------------------------------------------------
 
-// main
-function run_test()
+add_task(function* test_preventive_maintenance()
 {
-  run_next_test();
-}
-
-add_task(function test_preventive_maintenance()
-{
-  // Force initialization of the bookmarks hash. This test could cause
-  // it to go out of sync due to direct queries on the database.
-  yield PlacesTestUtils.addVisits(uri("http://force.bookmarks.hash"));
-  do_check_false(bs.isBookmarked(uri("http://force.bookmarks.hash")));
-
   // Get current bookmarks max ID for cleanup
   let stmt = mDBConn.createStatement("SELECT MAX(id) FROM moz_bookmarks");
   stmt.executeStep();
@@ -1335,25 +1327,22 @@ add_task(function test_preventive_maintenance()
   stmt.finalize();
   do_check_true(defaultBookmarksMaxId > 0);
 
-  for ([, test] in Iterator(tests)) {
+  for (let test of tests) {
     dump("\nExecuting test: " + test.name + "\n" + "*** " + test.desc + "\n");
     yield test.setup();
 
     let promiseMaintenanceFinished =
         promiseTopicObserved(FINISHED_MAINTENANCE_NOTIFICATION_TOPIC);
-    PlacesDBUtils.maintenanceOnIdle();
+    Services.prefs.clearUserPref("places.database.lastMaintenance");
+    let callbackInvoked = false;
+    PlacesDBUtils.maintenanceOnIdle(() => callbackInvoked = true);
     yield promiseMaintenanceFinished;
+    do_check_true(callbackInvoked);
 
     // Check the lastMaintenance time has been saved.
     do_check_neq(Services.prefs.getIntPref("places.database.lastMaintenance"), null);
 
-    if (test.asyncCheck) {
-      let deferred = Promise.defer();
-      test.asyncCheck(deferred.resolve);
-      yield deferred.promise;
-    } else {
-      test.check();
-    }
+    yield test.check();
 
     cleanDatabase();
   }
