@@ -18,8 +18,7 @@
 /**
  * Thread proxy from the main thread to the worker thread.
  */
-class UrlClassifierDBServiceWorkerProxy final :
-  public nsIUrlClassifierDBServiceWorker
+class UrlClassifierDBServiceWorkerProxy final : public nsIUrlClassifierDBService
 {
 public:
   explicit UrlClassifierDBServiceWorkerProxy(nsUrlClassifierDBServiceWorker* aTarget)
@@ -28,9 +27,8 @@ public:
 
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURLCLASSIFIERDBSERVICE
-  NS_DECL_NSIURLCLASSIFIERDBSERVICEWORKER
 
-  class LookupRunnable : public nsRunnable
+  class LookupRunnable : public mozilla::Runnable
   {
   public:
     LookupRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -52,7 +50,7 @@ public:
     nsCOMPtr<nsIUrlClassifierCallback> mCB;
   };
 
-  class GetTablesRunnable : public nsRunnable
+  class GetTablesRunnable : public mozilla::Runnable
   {
   public:
     GetTablesRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -68,7 +66,7 @@ public:
     nsCOMPtr<nsIUrlClassifierCallback> mCB;
   };
 
-  class BeginUpdateRunnable : public nsRunnable
+  class BeginUpdateRunnable : public mozilla::Runnable
   {
   public:
     BeginUpdateRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -87,7 +85,7 @@ public:
     nsCString mTables;
   };
 
-  class BeginStreamRunnable : public nsRunnable
+  class BeginStreamRunnable : public mozilla::Runnable
   {
   public:
     BeginStreamRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -103,7 +101,7 @@ public:
     nsCString mTable;
   };
 
-  class UpdateStreamRunnable : public nsRunnable
+  class UpdateStreamRunnable : public mozilla::Runnable
   {
   public:
     UpdateStreamRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -119,7 +117,7 @@ public:
     nsCString mUpdateChunk;
   };
 
-  class CacheCompletionsRunnable : public nsRunnable
+  class CacheCompletionsRunnable : public mozilla::Runnable
   {
   public:
     CacheCompletionsRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -135,7 +133,7 @@ public:
      mozilla::safebrowsing::CacheResultArray *mEntries;
   };
 
-  class CacheMissesRunnable : public nsRunnable
+  class CacheMissesRunnable : public mozilla::Runnable
   {
   public:
     CacheMissesRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -151,7 +149,7 @@ public:
     mozilla::safebrowsing::PrefixArray *mEntries;
   };
 
-  class DoLocalLookupRunnable : public nsRunnable
+  class DoLocalLookupRunnable : public mozilla::Runnable
   {
   public:
     DoLocalLookupRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -173,7 +171,7 @@ public:
     mozilla::safebrowsing::LookupResultArray* mResults;
   };
 
-  class SetLastUpdateTimeRunnable : public nsRunnable
+  class SetLastUpdateTimeRunnable : public mozilla::Runnable
   {
   public:
     SetLastUpdateTimeRunnable(nsUrlClassifierDBServiceWorker* aTarget,
@@ -191,11 +189,29 @@ public:
     uint64_t mUpdateTime;
   };
 
+  class ClearLastResultsRunnable : public mozilla::Runnable
+  {
+  public:
+    explicit ClearLastResultsRunnable(nsUrlClassifierDBServiceWorker* aTarget)
+      : mTarget(aTarget)
+    { }
+
+    NS_DECL_NSIRUNNABLE
+  private:
+    RefPtr<nsUrlClassifierDBServiceWorker> mTarget;
+  };
+
 public:
   nsresult DoLocalLookup(const nsACString& spec,
                          const nsACString& tables,
                          mozilla::safebrowsing::LookupResultArray* results);
 
+  nsresult OpenDb();
+  nsresult CloseDb();
+
+  nsresult CacheCompletions(mozilla::safebrowsing::CacheResultArray * aEntries);
+  nsresult CacheMisses(mozilla::safebrowsing::PrefixArray * aEntries);
+
 private:
   ~UrlClassifierDBServiceWorkerProxy() {}
 
@@ -215,7 +231,7 @@ public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURLCLASSIFIERLOOKUPCALLBACK
 
-  class LookupCompleteRunnable : public nsRunnable
+  class LookupCompleteRunnable : public mozilla::Runnable
   {
   public:
     LookupCompleteRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierLookupCallback>& aTarget,
@@ -247,7 +263,7 @@ public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURLCLASSIFIERCALLBACK
 
-  class HandleEventRunnable : public nsRunnable
+  class HandleEventRunnable : public mozilla::Runnable
   {
   public:
     HandleEventRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierCallback>& aTarget,
@@ -280,7 +296,7 @@ public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIURLCLASSIFIERUPDATEOBSERVER
 
-  class UpdateUrlRequestedRunnable : public nsRunnable
+  class UpdateUrlRequestedRunnable : public mozilla::Runnable
   {
   public:
     UpdateUrlRequestedRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierUpdateObserver>& aTarget,
@@ -298,7 +314,7 @@ public:
     nsCString mURL, mTable;
   };
 
-  class StreamFinishedRunnable : public nsRunnable
+  class StreamFinishedRunnable : public mozilla::Runnable
   {
   public:
     StreamFinishedRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierUpdateObserver>& aTarget,
@@ -316,7 +332,7 @@ public:
     uint32_t mDelay;
   };
 
-  class UpdateErrorRunnable : public nsRunnable
+  class UpdateErrorRunnable : public mozilla::Runnable
   {
   public:
     UpdateErrorRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierUpdateObserver>& aTarget,
@@ -332,7 +348,7 @@ public:
     nsresult mError;
   };
 
-  class UpdateSuccessRunnable : public nsRunnable
+  class UpdateSuccessRunnable : public mozilla::Runnable
   {
   public:
     UpdateSuccessRunnable(const nsMainThreadPtrHandle<nsIUrlClassifierUpdateObserver>& aTarget,
