@@ -29,7 +29,8 @@ prefBranch.setIntPref("urlclassifier.gethashnoise", 0);
 
 // Enable malware/phishing checking for tests
 prefBranch.setBoolPref("browser.safebrowsing.malware.enabled", true);
-prefBranch.setBoolPref("browser.safebrowsing.enabled", true);
+prefBranch.setBoolPref("browser.safebrowsing.blockedURIs.enabled", true);
+prefBranch.setBoolPref("browser.safebrowsing.phishing.enabled", true);
 
 // Enable all completions for tests
 prefBranch.setCharPref("urlclassifier.disallow_completions", "");
@@ -54,26 +55,24 @@ function cleanUp() {
   delFile("safebrowsing/test-phish-simple.sbstore");
   delFile("safebrowsing/test-malware-simple.sbstore");
   delFile("safebrowsing/test-unwanted-simple.sbstore");
-  delFile("safebrowsing/test-forbid-simple.sbstore");
+  delFile("safebrowsing/test-block-simple.sbstore");
   delFile("safebrowsing/test-track-simple.sbstore");
   delFile("safebrowsing/test-trackwhite-simple.sbstore");
-  delFile("safebrowsing/test-phish-simple.cache");
-  delFile("safebrowsing/test-malware-simple.cache");
-  delFile("safebrowsing/test-unwanted-simple.cache");
-  delFile("safebrowsing/test-forbid-simple.cache");
-  delFile("safebrowsing/test-track-simple.cache");
-  delFile("safebrowsing/test-trackwhite-simple.cache");
   delFile("safebrowsing/test-phish-simple.pset");
   delFile("safebrowsing/test-malware-simple.pset");
   delFile("safebrowsing/test-unwanted-simple.pset");
-  delFile("safebrowsing/test-forbid-simple.pset");
+  delFile("safebrowsing/test-block-simple.pset");
   delFile("safebrowsing/test-track-simple.pset");
   delFile("safebrowsing/test-trackwhite-simple.pset");
+  delFile("safebrowsing/moz-phish-simple.sbstore");
+  delFile("safebrowsing/moz-phish-simple.pset");
   delFile("testLarge.pset");
   delFile("testNoDelta.pset");
 }
 
-var allTables = "test-phish-simple,test-malware-simple,test-unwanted-simple,test-forbid-simple,test-track-simple,test-trackwhite-simple";
+// Update uses allTables by default
+var allTables = "test-phish-simple,test-malware-simple,test-unwanted-simple,test-track-simple,test-trackwhite-simple,test-block-simple";
+var mozTables = "moz-phish-simple";
 
 var dbservice = Cc["@mozilla.org/url-classifier/dbservice;1"].getService(Ci.nsIUrlClassifierDBService);
 var streamUpdater = Cc["@mozilla.org/url-classifier/streamupdater;1"]
@@ -130,8 +129,12 @@ function buildUnwantedUpdate(chunks, hashSize) {
   return buildUpdate({"test-unwanted-simple" : chunks}, hashSize);
 }
 
-function buildForbiddenUpdate(chunks, hashSize) {
-  return buildUpdate({"test-forbid-simple" : chunks}, hashSize);
+function buildBlockedUpdate(chunks, hashSize) {
+  return buildUpdate({"test-block-simple" : chunks}, hashSize);
+}
+
+function buildMozPhishingUpdate(chunks, hashSize) {
+  return buildUpdate({"moz-phish-simple" : chunks}, hashSize);
 }
 
 function buildBareUpdate(chunks, hashSize) {
@@ -199,7 +202,7 @@ function doStreamUpdate(updateText, success, failure, downloadFailure) {
     downloadFailure = failure;
   }
 
-  streamUpdater.downloadUpdates(allTables, "",
+  streamUpdater.downloadUpdates(allTables, "", true,
                                 dataUpdate, success, failure, downloadFailure);
 }
 
@@ -221,15 +224,16 @@ tableData : function(expectedTables, cb)
     });
 },
 
-checkUrls: function(urls, expected, cb)
+checkUrls: function(urls, expected, cb, useMoz = false)
 {
   // work with a copy of the list.
   urls = urls.slice(0);
   var doLookup = function() {
     if (urls.length > 0) {
+      var tables = useMoz ? mozTables : allTables;
       var fragment = urls.shift();
       var principal = secMan.createCodebasePrincipal(iosvc.newURI("http://" + fragment, null, null), {});
-      dbservice.lookup(principal, allTables,
+      dbservice.lookup(principal, tables,
                                 function(arg) {
                                   do_check_eq(expected, arg);
                                   doLookup();
@@ -241,6 +245,22 @@ checkUrls: function(urls, expected, cb)
   doLookup();
 },
 
+checkTables: function(url, expected, cb)
+{
+  var principal = secMan.createCodebasePrincipal(iosvc.newURI("http://" + url, null, null), {});
+  dbservice.lookup(principal, allTables, function(tables) {
+    // Rebuild tables in a predictable order.
+    var parts = tables.split(",");
+    while (parts[parts.length - 1] == '') {
+      parts.pop();
+    }
+    parts.sort();
+    tables = parts.join(",");
+    do_check_eq(tables, expected);
+    cb();
+  }, true);
+},
+
 urlsDontExist: function(urls, cb)
 {
   this.checkUrls(urls, '', cb);
@@ -261,9 +281,14 @@ unwantedUrlsExist: function(urls, cb)
   this.checkUrls(urls, 'test-unwanted-simple', cb);
 },
 
-forbiddenUrlsExist: function(urls, cb)
+blockedUrlsExist: function(urls, cb)
 {
-  this.checkUrls(urls, 'test-forbid-simple', cb);
+  this.checkUrls(urls, 'test-block-simple', cb);
+},
+
+mozPhishingUrlsExist: function(urls, cb)
+{
+  this.checkUrls(urls, 'moz-phish-simple', cb, true);
 },
 
 subsDontExist: function(urls, cb)
@@ -276,6 +301,11 @@ subsExist: function(urls, cb)
 {
   // XXX: there's no interface for checking items in the subs table
   cb();
+},
+
+urlExistInMultipleTables: function(data, cb)
+{
+  this.checkTables(data["url"], data["tables"], cb);
 }
 
 };
