@@ -196,6 +196,27 @@ static string GetDumpLocalID()
   return localId.substr(0, dot);
 }
 
+// This appends the StackTraces entry generated by the minidump analyzer to the
+// main crash event so that it can be picked up by IceCat once it restarts
+static void AppendStackTracesToEventFile(const string& aStackTraces)
+{
+  if (gEventsPath.empty()) {
+    // If there is no path for finding the crash event, skip this step.
+    return;
+  }
+
+  string localId = GetDumpLocalID();
+  string path = gEventsPath + UI_DIR_SEPARATOR + localId;
+  ofstream* f = UIOpenWrite(path.c_str(), true);
+
+  if (f->is_open()) {
+    *f << "StackTraces=" << aStackTraces;
+    f->close();
+  }
+
+  delete f;
+}
+
 static void WriteSubmissionEvent(SubmissionResult result,
                                  const string& remoteId)
 {
@@ -492,6 +513,21 @@ bool CheckEndOfLifed(string version)
   return UIFileExists(reportPath);
 }
 
+#ifndef RELEASE_OR_BETA
+
+static string
+GetMinidumpAnalyzerPath()
+{
+  string path = gArgv[0];
+  size_t pos = path.rfind(UI_CRASH_REPORTER_FILENAME BIN_SUFFIX);
+  path.erase(pos);
+  path.append(UI_MINIDUMP_ANALYZER_FILENAME BIN_SUFFIX);
+
+  return path;
+}
+
+#endif
+
 int main(int argc, char** argv)
 {
   gArgc = argc;
@@ -513,6 +549,13 @@ int main(int argc, char** argv)
     // no dump file specified, run the default UI
     UIShowDefaultUI();
   } else {
+#ifndef RELEASE_OR_BETA
+    // start by running minidump analyzer, this is currently enabled only in
+    // nightly and aurora
+    UIRunMinidumpAnalyzer(GetMinidumpAnalyzerPath(), gReporterDumpFile);
+#endif
+
+    // go ahead with the crash reporter
     gExtraFile = GetAdditionalFilename(gReporterDumpFile, kExtraDataExtension);
     if (gExtraFile.empty()) {
       UIError(gStrings[ST_ERROR_BADARGUMENTS]);
@@ -597,6 +640,12 @@ int main(int argc, char** argv)
       gEventsPath.clear();
     }
 
+    // Update the crash event with stacks if they are present
+    auto stackTracesItr = queryParameters.find("StackTraces");
+    if (stackTracesItr != queryParameters.end()) {
+      AppendStackTracesToEventFile(stackTracesItr->second);
+    }
+
     if (!UIFileExists(gReporterDumpFile)) {
       UIError(gStrings[ST_ERROR_DUMPFILEEXISTS]);
       return 0;
@@ -641,7 +690,7 @@ int main(int argc, char** argv)
       paramName.str("");
       paramName << "MOZ_CRASHREPORTER_RESTART_ARG_" << i++;
       param = getenv(paramName.str().c_str());
-    };
+    }
 
     // allow override of the server url via environment variable
     //XXX: remove this in the far future when our robot
