@@ -57,9 +57,9 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */
 
 
 #include "common/arm_ex_reader.h"
-#include "common/logging.h"
 
 #include <assert.h>
+#include <stdio.h>
 
 // This file, in conjunction with arm_ex_to_module.cc, translates
 // EXIDX unwind information into the same format that Breakpad uses
@@ -116,8 +116,7 @@ using arm_ex_to_module::ARM_EXIDX_VFP_FSTMD;
 using google_breakpad::MemoryRange;
 
 
-static void* Prel31ToAddr(const void* addr) 
-{
+static void* Prel31ToAddr(const void* addr) {
   uint32_t offset32 = *reinterpret_cast<const uint32_t*>(addr);
   // sign extend offset32[30:0] to 64 bits -- copy bit 30 to positions
   // 63:31 inclusive.
@@ -134,11 +133,10 @@ static void* Prel31ToAddr(const void* addr)
 // and return the number of bytes of |buf| written, along with a code
 // indicating the outcome.
 
-ExceptionTableInfo::ExExtractResult
-ExceptionTableInfo::ExtabEntryExtract(const struct exidx_entry* entry,
-                                      uint8_t* buf, size_t buf_size,
-                                      /*OUT*/size_t* buf_used)
-{
+ExceptionTableInfo::ExExtractResult ExceptionTableInfo::ExtabEntryExtract(
+    const struct exidx_entry* entry,
+    uint8_t* buf, size_t buf_size,
+    size_t* buf_used) {
   MemoryRange mr_out(buf, buf_size);
 
   *buf_used = 0;
@@ -258,8 +256,7 @@ ExceptionTableInfo::ExtabEntryExtract(const struct exidx_entry* entry,
 // This reads from |buf[0, +data_size)|.  It checks for overruns of
 // the input buffer and returns a negative value if that happens, or
 // for any other failure cases.  It returns zero in case of success.
-int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
-{
+int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size) {
   if (buf == NULL || buf_size == 0)
     return -1;
 
@@ -282,13 +279,11 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
       // vsp = vsp + (xxxxxx << 2) + 4
       edata.cmd = ARM_EXIDX_CMD_ADD_TO_VSP;
       edata.data = (((int)op & 0x3f) << 2) + 4;
-    }
-    else if ((op & 0xc0) == 0x40) {
+    } else if ((op & 0xc0) == 0x40) {
       // vsp = vsp - (xxxxxx << 2) - 4
       edata.cmd = ARM_EXIDX_CMD_SUB_FROM_VSP;
       edata.data = (((int)op & 0x3f) << 2) + 4;
-    }
-    else if ((op & 0xf0) == 0x80) {
+    } else if ((op & 0xf0) == 0x80) {
       uint8_t op2;
       GET_BUF_U8(op2);
       if (op == 0x80 && op2 == 0x00) {
@@ -300,8 +295,7 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
         edata.data = ((op & 0xf) << 8) | op2;
         edata.data = edata.data << 4;
       }
-    }
-    else if ((op & 0xf0) == 0x90) {
+    } else if ((op & 0xf0) == 0x90) {
       if (op == 0x9d || op == 0x9f) {
         // 9d: Reserved as prefix for ARM register to register moves
         // 9f: Reserved as perfix for Intel Wireless MMX reg to reg moves
@@ -311,8 +305,7 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
         edata.cmd = ARM_EXIDX_CMD_REG_TO_SP;
         edata.data = op & 0x0f;
       }
-    }
-    else if ((op & 0xf0) == 0xa0) {
+    } else if ((op & 0xf0) == 0xa0) {
       // Pop r4 to r[4+nnn],          or
       // Pop r4 to r[4+nnn] and r14   or
       unsigned end = (op & 0x07);
@@ -320,13 +313,11 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
       edata.data = edata.data << 4;
       if (op & 0x08) edata.data |= 1 << 14;
       edata.cmd = ARM_EXIDX_CMD_REG_POP;
-    }
-    else if (op == ARM_EXTBL_OP_FINISH) {
+    } else if (op == ARM_EXTBL_OP_FINISH) {
       // Finish
       edata.cmd = ARM_EXIDX_CMD_FINISH;
       buf = end;
-    }
-    else if (op == 0xb1) {
+    } else if (op == 0xb1) {
       uint8_t op2;
       GET_BUF_U8(op2);
       if (op2 == 0 || (op2 & 0xf0)) {
@@ -337,8 +328,7 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
         edata.cmd = ARM_EXIDX_CMD_REG_POP;
         edata.data = op2 & 0x0f;
       }
-    }
-    else if (op == 0xb2) {
+    } else if (op == 0xb2) {
       // vsp = vsp + 0x204 + (uleb128 << 2)
       uint64_t offset = 0;
       uint8_t byte, shift = 0;
@@ -349,8 +339,7 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
       } while ((byte & 0x80) && buf < end);
       edata.data = offset * 4 + 0x204;
       edata.cmd = ARM_EXIDX_CMD_ADD_TO_VSP;
-    }
-    else if (op == 0xb3 || op == 0xc8 || op == 0xc9) {
+    } else if (op == 0xb3 || op == 0xc8 || op == 0xc9) {
       // b3: Pop VFP regs D[ssss]    to D[ssss+cccc],    FSTMFDX-ishly
       // c8: Pop VFP regs D[16+ssss] to D[16+ssss+cccc], FSTMFDD-ishly
       // c9: Pop VFP regs D[ssss]    to D[ssss+cccc],    FSTMFDD-ishly
@@ -358,25 +347,21 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
       GET_BUF_U8(edata.data);
       if (op == 0xc8) edata.data |= ARM_EXIDX_VFP_SHIFT_16;
       if (op != 0xb3) edata.data |= ARM_EXIDX_VFP_FSTMD;
-    }
-    else if ((op & 0xf8) == 0xb8 || (op & 0xf8) == 0xd0) {
+    } else if ((op & 0xf8) == 0xb8 || (op & 0xf8) == 0xd0) {
       // b8: Pop VFP regs D[8] to D[8+nnn], FSTMFDX-ishly
       // d0: Pop VFP regs D[8] to D[8+nnn], FSTMFDD-ishly
       edata.cmd = ARM_EXIDX_CMD_VFP_POP;
       edata.data = 0x80 | (op & 0x07);
       if ((op & 0xf8) == 0xd0) edata.data |= ARM_EXIDX_VFP_FSTMD;
-    }
-    else if (op >= 0xc0 && op <= 0xc5) {
+    } else if (op >= 0xc0 && op <= 0xc5) {
       // Intel Wireless MMX pop wR[10]-wr[10+nnn], nnn != 6,7
       edata.cmd = ARM_EXIDX_CMD_WREG_POP;
       edata.data = 0xa0 | (op & 0x07);
-    }
-    else if (op == 0xc6) {
+    } else if (op == 0xc6) {
       // Intel Wireless MMX pop wR[ssss] to wR[ssss+cccc]
       edata.cmd = ARM_EXIDX_CMD_WREG_POP;
       GET_BUF_U8(edata.data);
-    }
-    else if (op == 0xc7) {
+    } else if (op == 0xc7) {
       uint8_t op2;
       GET_BUF_U8(op2);
       if (op2 == 0 || (op2 & 0xf0)) {
@@ -387,22 +372,21 @@ int ExceptionTableInfo::ExtabEntryDecode(const uint8_t* buf, size_t buf_size)
         edata.cmd = ARM_EXIDX_CMD_WCGR_POP;
         edata.data = op2 & 0x0f;
       }
-    }
-    else {
+    } else {
       // Spare
       edata.cmd = ARM_EXIDX_CMD_RESERVED;
     }
 
     int ret = handler_->ImproveStackFrame(&edata);
-    if (ret < 0) return ret;
+    if (ret < 0)
+      return ret;
   }
   return 0;
 
 # undef GET_BUF_U8
 }
 
-void ExceptionTableInfo::Start()
-{
+void ExceptionTableInfo::Start() {
   const struct exidx_entry* start
     = reinterpret_cast<const struct exidx_entry*>(mr_exidx_.data());
   const struct exidx_entry* end
@@ -412,16 +396,15 @@ void ExceptionTableInfo::Start()
   // Iterate over each of the EXIDX entries (pairs of 32-bit words).
   // These occupy the entire .exidx section.
   for (const struct exidx_entry* entry = start; entry < end; ++entry) {
-
     // Figure out the code address range that this table entry is
     // associated with.
     uint32_t addr = (reinterpret_cast<char*>(Prel31ToAddr(&entry->addr))
                      - mapping_addr_ + loading_addr_) & 0x7fffffff;
     uint32_t next_addr;
-    if (entry < end - 1)
+    if (entry < end - 1) {
       next_addr = (reinterpret_cast<char*>(Prel31ToAddr(&((entry + 1)->addr)))
                    - mapping_addr_ + loading_addr_) & 0x7fffffff;
-    else {
+    } else {
       // This is the last EXIDX entry in the sequence, so we don't
       // have an address for the start of the next function, to limit
       // this one.  Instead use the address of the last byte of the
@@ -445,10 +428,10 @@ void ExceptionTableInfo::Start()
           plausible = true;
         }
       }
-      if (!plausible)
-        BPLOG(INFO) << "ExceptionTableInfo: implausible EXIDX last entry size "
-                    << (int32_t)(text_last_svma_ - addr)
-                    << "; using 1 instead.";
+      if (!plausible) {
+        fprintf(stderr, "ExceptionTableInfo: implausible EXIDX last entry size "
+                "%d, using 1 instead.", (int32_t)(text_last_svma_ - addr));
+      }
     }
 
     // Extract the unwind info into |buf|.  This might fail for
@@ -462,22 +445,22 @@ void ExceptionTableInfo::Start()
       // Couldn't extract the unwind info, for some reason.  Move on.
       switch (res) {
         case ExInBufOverflow:
-          BPLOG(INFO) << "ExtabEntryExtract: .exidx/.extab section overrun";
+          fprintf(stderr, "ExtabEntryExtract: .exidx/.extab section overrun");
           break;
         case ExOutBufOverflow:
-          BPLOG(INFO) << "ExtabEntryExtract: bytecode buffer overflow";
+          fprintf(stderr, "ExtabEntryExtract: bytecode buffer overflow");
           break;
         case ExCantUnwind:
-          BPLOG(INFO) << "ExtabEntryExtract: function is marked CANT_UNWIND";
+          fprintf(stderr, "ExtabEntryExtract: function is marked CANT_UNWIND");
           break;
         case ExCantRepresent:
-          BPLOG(INFO) << "ExtabEntryExtract: bytecode can't be represented";
+          fprintf(stderr, "ExtabEntryExtract: bytecode can't be represented");
           break;
         case ExInvalid:
-          BPLOG(INFO) << "ExtabEntryExtract: index table entry is invalid";
+          fprintf(stderr, "ExtabEntryExtract: index table entry is invalid");
           break;
         default:
-          BPLOG(INFO) << "ExtabEntryExtract: unknown error: " << (int)res;
+          fprintf(stderr, "ExtabEntryExtract: unknown error: %d", (int)res);
           break;
       }
       continue;
@@ -492,7 +475,7 @@ void ExceptionTableInfo::Start()
       int ret = ExtabEntryDecode(buf, buf_used);
       if (ret < 0) {
 	handler_->DeleteStackFrame();
-	BPLOG(INFO) << "ExtabEntryDecode: failed with error code: " << ret;
+	fprintf(stderr, "ExtabEntryDecode: failed with error code: %d", ret);
 	continue;
       }
       handler_->SubmitStackFrame();
@@ -501,4 +484,4 @@ void ExceptionTableInfo::Start()
   } /* iterating over .exidx */
 }
 
-} // arm_ex_reader
+}  // namespace arm_ex_reader
