@@ -1,7 +1,7 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-const {classes: Cc, interfaces: Ci, results: Cr, utils: Cu, manager: Cm} = Components;
+var {classes: Cc, interfaces: Ci, results: Cr, utils: Cu, manager: Cm} = Components;
 
 Cu.import("resource://gre/modules/Preferences.jsm");
 
@@ -92,7 +92,7 @@ add_test(function test_set_null_pref() {
     // We expect this to throw, so the test is designed to fail if it doesn't.
     do_check_true(false);
   }
-  catch(ex) {}
+  catch (ex) {}
 
   run_next_test();
 });
@@ -103,7 +103,7 @@ add_test(function test_set_undefined_pref() {
     // We expect this to throw, so the test is designed to fail if it doesn't.
     do_check_true(false);
   }
-  catch(ex) {}
+  catch (ex) {}
 
   run_next_test();
 });
@@ -114,7 +114,7 @@ add_test(function test_set_unsupported_pref() {
     // We expect this to throw, so the test is designed to fail if it doesn't.
     do_check_true(false);
   }
-  catch(ex) {}
+  catch (ex) {}
 
   run_next_test();
 });
@@ -167,7 +167,7 @@ add_test(function test_set_get_number_pref() {
     // We expect this to throw, so the test is designed to fail if it doesn't.
     do_check_true(false);
   }
-  catch(ex) {}
+  catch (ex) {}
 
   // Clean up.
   Preferences.reset("test_set_get_number_pref");
