@@ -16,6 +16,7 @@
 #include <algorithm>
 #include <endian.h>
 #include <fcntl.h>
+#include <pthread.h>
 #include <string>
 #include <sys/mman.h>
 #include <sys/stat.h>
@@ -601,7 +602,7 @@ AnimationThread(void *)
                 name.length() >= 256)
                 continue;
 
-            part.frames.push_back();
+            part.frames.resize(part.frames.size() + 1);
             AnimationFrame &frame = part.frames.back();
             strcpy(frame.path, name.c_str());
             frame.file = reader.GetLocalEntry(entry);
