@@ -108,7 +108,7 @@ status_t GonkNativeWindow::setDefaultBufferFormat(uint32_t defaultFormat) {
     return mBufferQueue->setDefaultBufferFormat(defaultFormat);
 }
 
-TemporaryRef<TextureClient>
+already_AddRefed<TextureClient>
 GonkNativeWindow::getCurrentBuffer() {
     Mutex::Autolock _l(mMutex);
     GonkBufferQueue::BufferItem item;
@@ -125,8 +125,8 @@ GonkNativeWindow::getCurrentBuffer() {
     if (!textureClient) {
         return NULL;
     }
-  textureClient->SetRecycleCallback(GonkNativeWindow::RecycleCallback, this);
-  return textureClient;
+    textureClient->SetRecycleCallback(GonkNativeWindow::RecycleCallback, this);
+    return textureClient.forget();
 }
 
 /* static */ void
@@ -134,6 +134,7 @@ GonkNativeWindow::RecycleCallback(TextureClient* client, void* closure) {
   GonkNativeWindow* nativeWindow =
     static_cast<GonkNativeWindow*>(closure);
 
+  MOZ_ASSERT(client && !client->IsDead());
   client->ClearRecycleCallback();
   nativeWindow->returnBuffer(client);
 }
@@ -146,17 +147,16 @@ void GonkNativeWindow::returnBuffer(TextureClient* client) {
     if (index < 0) {
     }
 
-    sp<Fence> fence = client->GetReleaseFenceHandle().mFence;
-    if (!fence.get()) {
-      fence = Fence::NO_FENCE;
-    }
+    FenceHandle handle = client->GetAndResetReleaseFenceHandle();
+    RefPtr<FenceHandle::FdObj> fdObj = handle.GetAndResetFdObj();
+    sp<Fence> fence = new Fence(fdObj->GetAndResetFd());
 
     addReleaseFenceLocked(index, fence);
 
     releaseBufferLocked(index);
 }
 
-TemporaryRef<TextureClient>
+already_AddRefed<TextureClient>
 GonkNativeWindow::getTextureClientFromBuffer(ANativeWindowBuffer* buffer) {
     Mutex::Autolock lock(mMutex);
     return mBufferQueue->getTextureClientFromBuffer(buffer);
