@@ -13,6 +13,8 @@
  * limitations under the License.
  */
 
+#include "nsWindow.h"
+
 #include "mozilla/DebugOnly.h"
 
 #include <fcntl.h>
@@ -20,20 +22,18 @@
 #include "android/log.h"
 #include "mozilla/dom/TabParent.h"
 #include "mozilla/Preferences.h"
+#include "mozilla/RefPtr.h"
 #include "mozilla/Services.h"
 #include "mozilla/FileUtils.h"
 #include "mozilla/ClearOnShutdown.h"
 #include "gfxContext.h"
 #include "gfxPlatform.h"
-#include "gfxUtils.h"
 #include "GLContextProvider.h"
 #include "GLContext.h"
 #include "GLContextEGL.h"
-#include "nsAutoPtr.h"
 #include "nsAppShell.h"
 #include "nsScreenManagerGonk.h"
 #include "nsTArray.h"
-#include "nsWindow.h"
 #include "nsIWidgetListener.h"
 #include "ClientLayerManager.h"
 #include "BasicLayers.h"
@@ -43,7 +43,9 @@
 #include "mozilla/gfx/Logging.h"
 #include "mozilla/layers/APZCTreeManager.h"
 #include "mozilla/layers/APZThreadUtils.h"
-#include "mozilla/layers/CompositorParent.h"
+#include "mozilla/layers/CompositorBridgeParent.h"
+#include "mozilla/layers/CompositorThread.h"
+#include "mozilla/layers/CompositorSession.h"
 #include "mozilla/TouchEvents.h"
 #include "HwcComposer2D.h"
 
@@ -66,8 +68,6 @@ static nsWindow *gFocusedWindow = nullptr;
 NS_IMPL_ISUPPORTS_INHERITED0(nsWindow, nsBaseWidget)
 
 nsWindow::nsWindow()
-    : mFramebuffer(nullptr)
-    , mMappedBuffer(nullptr)
 {
     RefPtr<nsScreenManagerGonk> screenManager = nsScreenManagerGonk::GetInstance();
     screenManager->Initialize();
@@ -87,7 +87,7 @@ nsWindow::nsWindow()
 nsWindow::~nsWindow()
 {
     if (mScreen->IsPrimaryScreen()) {
-        mComposer2D->SetCompositorParent(nullptr);
+        mComposer2D->SetCompositorBridgeParent(nullptr);
     }
 }
 
@@ -133,7 +133,7 @@ nsWindow::DoDraw(void)
 void
 nsWindow::ConfigureAPZControllerThread()
 {
-    APZThreadUtils::SetControllerThread(CompositorParent::CompositorLoop());
+    APZThreadUtils::SetControllerThread(CompositorThreadHolder::Loop());
 }
 
 /*static*/ nsEventStatus
@@ -146,7 +146,7 @@ nsWindow::DispatchKeyInput(WidgetKeyboardEvent& aEvent)
     gFocusedWindow->UserActivity();
 
     nsEventStatus status;
-    aEvent.widget = gFocusedWindow;
+    aEvent.mWidget = gFocusedWindow;
     gFocusedWindow->DispatchEvent(&aEvent, status);
     return status;
 }
@@ -163,7 +163,7 @@ nsWindow::DispatchTouchInput(MultiTouchInput& aInput)
     gFocusedWindow->DispatchTouchInputViaAPZ(aInput);
 }
 
-class DispatchTouchInputOnMainThread : public nsRunnable
+class DispatchTouchInputOnMainThread : public mozilla::Runnable
 {
 public:
     DispatchTouchInputOnMainThread(const MultiTouchInput& aInput,
@@ -176,7 +176,7 @@ public:
       , mApzResponse(aApzResponse)
     {}
 
-    NS_IMETHOD Run() {
+    NS_IMETHOD Run() override {
         if (gFocusedWindow) {
             gFocusedWindow->DispatchTouchEventForAPZ(mInput, mGuid, mInputBlockId, mApzResponse);
         }
@@ -237,18 +237,18 @@ nsWindow::DispatchTouchEventForAPZ(const MultiTouchInput& aInput,
     ProcessUntransformedAPZEvent(&event, aGuid, aInputBlockId, aApzResponse);
 }
 
-class DispatchTouchInputOnControllerThread : public Task
+class DispatchTouchInputOnControllerThread : public Runnable
 {
 public:
     DispatchTouchInputOnControllerThread(const MultiTouchInput& aInput)
-      : Task()
-      , mInput(aInput)
+      : mInput(aInput)
     {}
 
-    virtual void Run() override {
+    NS_IMETHOD Run() override {
         if (gFocusedWindow) {
             gFocusedWindow->DispatchTouchInputViaAPZ(mInput);
         }
+        return NS_OK;
     }
 
 private:
@@ -258,7 +258,7 @@ private:
 nsresult
 nsWindow::SynthesizeNativeTouchPoint(uint32_t aPointerId,
                                      TouchPointerState aPointerState,
-                                     ScreenIntPoint aPointerScreenPoint,
+                                     LayoutDeviceIntPoint aPoint,
                                      double aPointerPressure,
                                      uint32_t aPointerOrientation,
                                      nsIObserver* aObserver)
@@ -270,53 +270,17 @@ nsWindow::SynthesizeNativeTouchPoint(uint32_t aPointerId,
     }
 
     if (!mSynthesizedTouchInput) {
-        mSynthesizedTouchInput = new MultiTouchInput();
+        mSynthesizedTouchInput = MakeUnique<MultiTouchInput>();
     }
 
-    // We can't dispatch mSynthesizedTouchInput directly because (a) dispatching
-    // it might inadvertently modify it and (b) in the case of touchend or
-    // touchcancel events mSynthesizedTouchInput will hold the touches that are
-    // still down whereas the input dispatched needs to hold the removed
-    // touch(es). We use |inputToDispatch| for this purpose.
-    MultiTouchInput inputToDispatch;
-    inputToDispatch.mInputType = MULTITOUCH_INPUT;
+    // We should probably use a real timestamp here, but this is B2G and
+    // so this probably never even exercised any more.
+    uint32_t time = 0;
+    TimeStamp timestamp = TimeStamp::FromSystemTime(time);
 
-    int32_t index = mSynthesizedTouchInput->IndexOfTouch((int32_t)aPointerId);
-    if (aPointerState == TOUCH_CONTACT) {
-        if (index >= 0) {
-            // found an existing touch point, update it
-            SingleTouchData& point = mSynthesizedTouchInput->mTouches[index];
-            point.mScreenPoint = aPointerScreenPoint;
-            point.mRotationAngle = (float)aPointerOrientation;
-            point.mForce = (float)aPointerPressure;
-            inputToDispatch.mType = MultiTouchInput::MULTITOUCH_MOVE;
-        } else {
-            // new touch point, add it
-            mSynthesizedTouchInput->mTouches.AppendElement(SingleTouchData(
-                (int32_t)aPointerId,
-                aPointerScreenPoint,
-                ScreenSize(0, 0),
-                (float)aPointerOrientation,
-                (float)aPointerPressure));
-            inputToDispatch.mType = MultiTouchInput::MULTITOUCH_START;
-        }
-        inputToDispatch.mTouches = mSynthesizedTouchInput->mTouches;
-    } else {
-        MOZ_ASSERT(aPointerState == TOUCH_REMOVE || aPointerState == TOUCH_CANCEL);
-        // a touch point is being lifted, so remove it from the stored list
-        if (index >= 0) {
-            mSynthesizedTouchInput->mTouches.RemoveElementAt(index);
-        }
-        inputToDispatch.mType = (aPointerState == TOUCH_REMOVE
-            ? MultiTouchInput::MULTITOUCH_END
-            : MultiTouchInput::MULTITOUCH_CANCEL);
-        inputToDispatch.mTouches.AppendElement(SingleTouchData(
-            (int32_t)aPointerId,
-            aPointerScreenPoint,
-            ScreenSize(0, 0),
-            (float)aPointerOrientation,
-            (float)aPointerPressure));
-    }
+    MultiTouchInput inputToDispatch = UpdateSynthesizedTouchState(
+        mSynthesizedTouchInput.get(), time, timestamp, aPointerId, aPointerState,
+        aPoint, aPointerPressure, aPointerOrientation);
 
     // Can't use NewRunnableMethod here because that will pass a const-ref
     // argument to DispatchTouchInputViaAPZ whereas that function takes a
@@ -324,18 +288,19 @@ nsWindow::SynthesizeNativeTouchPoint(uint32_t aPointerId,
     // the function performs so this is fine. Also we can't pass |this| to the
     // task because nsWindow refcounting is not threadsafe. Instead we just use
     // the gFocusedWindow static ptr instead the task.
-    APZThreadUtils::RunOnControllerThread(new DispatchTouchInputOnControllerThread(inputToDispatch));
+    APZThreadUtils::RunOnControllerThread(
+      MakeAndAddRef<DispatchTouchInputOnControllerThread>(inputToDispatch));
 
     return NS_OK;
 }
 
-NS_IMETHODIMP
+nsresult
 nsWindow::Create(nsIWidget* aParent,
                  void* aNativeParent,
                  const LayoutDeviceIntRect& aRect,
                  nsWidgetInitData* aInitData)
 {
-    BaseCreate(aParent, aRect, aInitData);
+    BaseCreate(aParent, aInitData);
 
     nsCOMPtr<nsIScreen> screen;
 
@@ -347,13 +312,13 @@ nsWindow::Create(nsIWidget* aParent,
 
     mScreen = static_cast<nsScreenGonk*>(screen.get());
 
-    mBounds = aRect.ToUnknownRect();
+    mBounds = aRect;
 
     mParent = (nsWindow *)aParent;
     mVisible = false;
 
     if (!aParent) {
-        mBounds = mScreen->GetRect().ToUnknownRect();
+        mBounds = mScreen->GetRect();
     }
 
     mComposer2D = HwcComposer2D::GetInstance();
@@ -369,8 +334,8 @@ nsWindow::Create(nsIWidget* aParent,
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsWindow::Destroy(void)
+void
+nsWindow::Destroy()
 {
     mOnDestroyCalled = true;
     mScreen->UnregisterWindow(this);
@@ -378,7 +343,6 @@ nsWindow::Destroy(void)
         gFocusedWindow = nullptr;
     }
     nsBaseWidget::OnDestroy();
-    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -421,14 +385,6 @@ nsWindow::IsVisible() const
     return mVisible;
 }
 
-NS_IMETHODIMP
-nsWindow::ConstrainPosition(bool aAllowSlop,
-                            int32_t *aX,
-                            int32_t *aY)
-{
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsWindow::Move(double aX,
                double aY)
@@ -451,14 +407,14 @@ nsWindow::Resize(double aX,
                  double aHeight,
                  bool   aRepaint)
 {
-    mBounds = nsIntRect(NSToIntRound(aX), NSToIntRound(aY),
-                        NSToIntRound(aWidth), NSToIntRound(aHeight));
+    mBounds = LayoutDeviceIntRect(NSToIntRound(aX), NSToIntRound(aY),
+                                  NSToIntRound(aWidth), NSToIntRound(aHeight));
     if (mWidgetListener) {
         mWidgetListener->WindowResized(this, mBounds.width, mBounds.height);
     }
 
     if (aRepaint) {
-        Invalidate(LayoutDeviceIntRect::FromUnknownRect(mBounds));
+        Invalidate(mBounds);
     }
 
     return NS_OK;
@@ -539,10 +495,15 @@ nsWindow::GetNativeData(uint32_t aDataType)
         return mScreen->GetNativeWindow();
     case NS_NATIVE_OPENGL_CONTEXT:
         return mScreen->GetGLContext().take();
-    case NS_RAW_NATIVE_IME_CONTEXT:
+    case NS_RAW_NATIVE_IME_CONTEXT: {
+        void* pseudoIMEContext = GetPseudoIMEContext();
+        if (pseudoIMEContext) {
+            return pseudoIMEContext;
+        }
         // There is only one IME context on Gonk.
         return NS_ONLY_ONE_NATIVE_IME_CONTEXT;
     }
+    }
 
     return nullptr;
 }
@@ -588,19 +549,14 @@ nsWindow::GetInputContext()
     return mInputContext;
 }
 
-NS_IMETHODIMP
-nsWindow::ReparentNativeWidget(nsIWidget* aNewParent)
-{
-    return NS_OK;
-}
-
-NS_IMETHODIMP
+nsresult
 nsWindow::MakeFullScreen(bool aFullScreen, nsIScreen*)
 {
     if (mWindowType != eWindowType_toplevel) {
         // Ignore fullscreen request for non-toplevel windows.
         NS_WARNING("MakeFullScreen() on a dialog or child widget?");
-        return nsBaseWidget::MakeFullScreen(aFullScreen);
+        nsBaseWidget::InfallibleMakeFullScreen(aFullScreen);
+        return NS_OK;
     }
 
     if (aFullScreen) {
@@ -623,116 +579,17 @@ nsWindow::MakeFullScreen(bool aFullScreen, nsIScreen*)
     return NS_OK;
 }
 
-static gralloc_module_t const*
-gralloc_module()
-{
-    hw_module_t const *module;
-    if (hw_get_module(GRALLOC_HARDWARE_MODULE_ID, &module)) {
-        return nullptr;
-    }
-    return reinterpret_cast<gralloc_module_t const*>(module);
-}
-
-static SurfaceFormat
-HalFormatToSurfaceFormat(int aHalFormat)
-{
-    switch (aHalFormat) {
-    case HAL_PIXEL_FORMAT_RGBA_8888:
-        // Needs RB swap
-        return SurfaceFormat::B8G8R8A8;
-    case HAL_PIXEL_FORMAT_RGBX_8888:
-        // Needs RB swap
-        return SurfaceFormat::B8G8R8X8;
-    case HAL_PIXEL_FORMAT_BGRA_8888:
-        return SurfaceFormat::B8G8R8A8;
-    case HAL_PIXEL_FORMAT_RGB_565:
-        return SurfaceFormat::R5G6B5_UINT16;
-    default:
-        MOZ_CRASH("Unhandled HAL pixel format");
-        return SurfaceFormat::UNKNOWN; // not reached
-    }
-}
-
-static bool
-NeedsRBSwap(int aHalFormat)
-{
-    switch (aHalFormat) {
-    case HAL_PIXEL_FORMAT_RGBA_8888:
-        return true;
-    case HAL_PIXEL_FORMAT_RGBX_8888:
-        return true;
-    case HAL_PIXEL_FORMAT_BGRA_8888:
-        return false;
-    case HAL_PIXEL_FORMAT_RGB_565:
-        return false;
-    default:
-        MOZ_CRASH("Unhandled HAL pixel format");
-        return false; // not reached
-    }
-}
-
-
 already_AddRefed<DrawTarget>
 nsWindow::StartRemoteDrawing()
 {
-    mFramebuffer = mScreen->DequeueBuffer();
-    int width = mFramebuffer->width, height = mFramebuffer->height;
-    if (gralloc_module()->lock(gralloc_module(), mFramebuffer->handle,
-                               GRALLOC_USAGE_SW_READ_NEVER |
-                               GRALLOC_USAGE_SW_WRITE_OFTEN |
-                               GRALLOC_USAGE_HW_FB,
-                               0, 0, width, height,
-                               reinterpret_cast<void**>(&mMappedBuffer))) {
-        EndRemoteDrawing();
-        return nullptr;
-    }
-    SurfaceFormat format = HalFormatToSurfaceFormat(mScreen->GetSurfaceFormat());
-    mFramebufferTarget = Factory::CreateDrawTargetForData(
-        BackendType::CAIRO,
-        mMappedBuffer,
-        IntSize(width, height),
-        mFramebuffer->stride * gfx::BytesPerPixel(format),
-        format);
-    if (!mFramebufferTarget) {
-        MOZ_CRASH("nsWindow::StartRemoteDrawing failed in CreateDrawTargetForData");
-    }
-    if (!mBackBuffer ||
-        mBackBuffer->GetSize() != mFramebufferTarget->GetSize() ||
-        mBackBuffer->GetFormat() != mFramebufferTarget->GetFormat()) {
-        mBackBuffer = mFramebufferTarget->CreateSimilarDrawTarget(
-            mFramebufferTarget->GetSize(), mFramebufferTarget->GetFormat());
-    }
-    RefPtr<DrawTarget> buffer(mBackBuffer);
+    RefPtr<DrawTarget> buffer = mScreen->StartRemoteDrawing();
     return buffer.forget();
 }
 
 void
 nsWindow::EndRemoteDrawing()
 {
-    if (mFramebufferTarget && mFramebuffer) {
-        IntSize size = mFramebufferTarget->GetSize();
-        Rect rect(0, 0, size.width, size.height);
-        RefPtr<SourceSurface> source = mBackBuffer->Snapshot();
-        mFramebufferTarget->DrawSurface(source, rect, rect);
-
-        // Convert from BGR to RGB
-        // XXX this is a temporary solution. It consumes extra cpu cycles,
-        // it should not be used on product device.
-        if (NeedsRBSwap(mScreen->GetSurfaceFormat())) {
-            LOGE("Very slow composition path, it should not be used on product!!!");
-            SurfaceFormat format = HalFormatToSurfaceFormat(mScreen->GetSurfaceFormat());
-            gfxUtils::ConvertBGRAtoRGBA(
-                mMappedBuffer,
-                mFramebuffer->stride * mFramebuffer->height * gfx::BytesPerPixel(format));
-            mMappedBuffer = nullptr;
-            gralloc_module()->unlock(gralloc_module(), mFramebuffer->handle);
-        }
-    }
-    if (mFramebuffer) {
-        mScreen->QueueBuffer(mFramebuffer);
-    }
-    mFramebuffer = nullptr;
-    mFramebufferTarget = nullptr;
+    mScreen->EndRemoteDrawing();
 }
 
 float
@@ -761,18 +618,12 @@ nsWindow::GetDefaultScaleInternal()
 LayerManager *
 nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
                           LayersBackend aBackendHint,
-                          LayerManagerPersistence aPersistence,
-                          bool* aAllowRetaining)
+                          LayerManagerPersistence aPersistence)
 {
-    if (aAllowRetaining) {
-        *aAllowRetaining = true;
-    }
     if (mLayerManager) {
         // This layer manager might be used for painting outside of DoDraw(), so we need
         // to set the correct rotation on it.
-        if (mLayerManager->GetBackendType() == LayersBackend::LAYERS_CLIENT) {
-            ClientLayerManager* manager =
-                static_cast<ClientLayerManager*>(mLayerManager.get());
+        if (ClientLayerManager* manager = mLayerManager->AsClientLayerManager()) {
             uint32_t rotation = mScreen->EffectiveScreenRotation();
             manager->SetDefaultTargetConfiguration(mozilla::layers::BufferMode::BUFFER_NONE,
                                                    ScreenRotation(rotation));
@@ -789,8 +640,11 @@ nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
     }
 
     CreateCompositor();
-    if (mCompositorParent && mScreen->IsPrimaryScreen()) {
-        mComposer2D->SetCompositorParent(mCompositorParent);
+    if (RefPtr<CompositorBridgeParent> bridge = GetCompositorBridgeParent()) {
+        mScreen->SetCompositorBridgeParent(bridge);
+        if (mScreen->IsPrimaryScreen()) {
+            mComposer2D->SetCompositorBridgeParent(bridge);
+        }
     }
     MOZ_ASSERT(mLayerManager);
     return mLayerManager;
@@ -799,19 +653,16 @@ nsWindow::GetLayerManager(PLayerTransactionChild* aShadowManager,
 void
 nsWindow::DestroyCompositor()
 {
-    if (mCompositorParent && mScreen->IsPrimaryScreen()) {
-        // Unset CompositorParent
-        mComposer2D->SetCompositorParent(nullptr);
+    if (RefPtr<CompositorBridgeParent> bridge = GetCompositorBridgeParent()) {
+        mScreen->SetCompositorBridgeParent(nullptr);
+        if (mScreen->IsPrimaryScreen()) {
+            // Unset CompositorBridgeParent
+            mComposer2D->SetCompositorBridgeParent(nullptr);
+        }
     }
     nsBaseWidget::DestroyCompositor();
 }
 
-CompositorParent*
-nsWindow::NewCompositorParent(int aSurfaceWidth, int aSurfaceHeight)
-{
-    return new CompositorParent(this, true, aSurfaceWidth, aSurfaceHeight);
-}
-
 void
 nsWindow::BringToTop()
 {
@@ -828,7 +679,7 @@ nsWindow::BringToTop()
         mWidgetListener->WindowActivated();
     }
 
-    Invalidate(LayoutDeviceIntRect::FromUnknownRect(mBounds));
+    Invalidate(mBounds);
 }
 
 void
@@ -885,3 +736,9 @@ nsWindow::GetComposer2D()
 
     return mComposer2D;
 }
+
+CompositorBridgeParent*
+nsWindow::GetCompositorBridgeParent() const
+{
+    return mCompositorSession ? mCompositorSession->GetInProcessBridge() : nullptr;
+}
