@@ -45,7 +45,7 @@ class WinWakeLockListener final : public nsIDOMMozWakeLockListener
 public:
   NS_DECL_ISUPPORTS;
 
-  NS_IMETHODIMP Notify(nsITimer *timer) override {
+  NS_IMETHOD Notify(nsITimer *timer) override {
     WAKE_LOCK_LOG("WinWakeLock: periodic timer fired");
     ResetScreenSaverTimeout();
     return NS_OK;
@@ -246,9 +246,14 @@ nsAppShell::Init()
 NS_IMETHODIMP
 nsAppShell::Run(void)
 {
-  // Ignore failure; failing to start the application is not exactly an
-  // appropriate response to failing to start an audio session.
-  mozilla::widget::StartAudioSession();
+  // Content processes initialize audio later through PContent using audio
+  // tray id information pulled from the browser process AudioSession. This
+  // way the two share a single volume control.
+  // Note StopAudioSession() is called from nsAppRunner.cpp after xpcom is torn
+  // down to insure the browser shuts down after child processes.
+  if (XRE_IsParentProcess()) {
+    mozilla::widget::StartAudioSession();
+  }
 
   // Add an observer that disables the screen saver when requested by Gecko.
   // For example when we're playing video in the foreground tab.
@@ -258,8 +263,6 @@ nsAppShell::Run(void)
 
   RemoveScreenWakeLockListener();
 
-  mozilla::widget::StopAudioSession();
-
   return rv;
 }
 
