@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -47,14 +48,14 @@ public:
 class CategoryNode
 {
 public:
-  NS_METHOD GetLeaf(const char* aEntryName,
-                    char** aResult);
+  nsresult GetLeaf(const char* aEntryName,
+                   char** aResult);
 
-  NS_METHOD AddLeaf(const char* aEntryName,
-                    const char* aValue,
-                    bool aReplace,
-                    char** aResult,
-                    PLArenaPool* aArena);
+  nsresult AddLeaf(const char* aEntryName,
+                   const char* aValue,
+                   bool aReplace,
+                   char** aResult,
+                   PLArenaPool* aArena);
 
   void DeleteLeaf(const char* aEntryName);
 
@@ -71,7 +72,7 @@ public:
     return tCount;
   }
 
-  NS_METHOD Enumerate(nsISimpleEnumerator** aResult);
+  nsresult Enumerate(nsISimpleEnumerator** aResult);
 
   // CategoryNode is arena-allocated, with the strings
   static CategoryNode* Create(PLArenaPool* aArena);
@@ -109,7 +110,7 @@ public:
    * observer service. This is to be used by nsComponentManagerImpl
    * on startup while reading the stored category list.
    */
-  NS_METHOD SuppressNotifications(bool aSuppress);
+  nsresult SuppressNotifications(bool aSuppress);
 
   void AddCategoryEntry(const char* aCategory,
                         const char* aKey,
