@@ -24,18 +24,32 @@
  * release and debug builds, and the result is an expression which can be
  * used in subsequent expressions, such as:
  *
- * if (NS_WARN_IF(NS_FAILED(rv))
+ * if (NS_WARN_IF(NS_FAILED(rv)) {
  *   return rv;
+ * }
  *
  * This explicit warning and return is preferred to the NS_ENSURE_* macros
  * which hide the warning and the return control flow.
  *
+ * This macro can also be used outside of conditions just to issue a warning,
+ * like so:
+ *
+ *   Unused << NS_WARN_IF(NS_FAILED(FnWithSideEffects());
+ *
+ * (The |Unused <<| is necessary because of the MOZ_MUST_USE annotation.)
+ *
+ * However, note that the argument to this macro is evaluated in all builds. If
+ * you just want a warning assertion, it is better to use NS_WARNING_ASSERTION
+ * (which evaluates the condition only in debug builds) like so:
+ *
+ *   NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "operation failed");
+ *
  * @note This is C++-only
  */
 #ifdef __cplusplus
 #ifdef DEBUG
-inline bool NS_warn_if_impl(bool aCondition, const char* aExpr,
-                            const char* aFile, int32_t aLine)
+inline MOZ_MUST_USE bool NS_warn_if_impl(bool aCondition, const char* aExpr,
+                                         const char* aFile, int32_t aLine)
 {
   if (MOZ_UNLIKELY(aCondition)) {
     NS_DebugBreak(NS_DEBUG_WARNING, nullptr, aExpr, aFile, aLine);
@@ -50,7 +64,8 @@ inline bool NS_warn_if_impl(bool aCondition, const char* aExpr,
 #endif
 
 /**
- * Warn if a given condition is false.
+ * Test an assertion for truth. If the expression is not true then
+ * emit a warning.
  *
  * Program execution continues past the usage of this macro.
  *
@@ -58,17 +73,16 @@ inline bool NS_warn_if_impl(bool aCondition, const char* aExpr,
  * evaluate the message argument.
  */
 #ifdef DEBUG
-#define NS_WARN_IF_FALSE(_expr,_msg)                          \
+#define NS_WARNING_ASSERTION(_expr, _msg)                     \
   do {                                                        \
     if (!(_expr)) {                                           \
       NS_DebugBreak(NS_DEBUG_WARNING, _msg, #_expr, __FILE__, __LINE__); \
     }                                                         \
   } while(0)
 #else
-#define NS_WARN_IF_FALSE(_expr, _msg)  do { /* nothing */ } while(0)
+#define NS_WARNING_ASSERTION(_expr, _msg)  do { /* nothing */ } while(0)
 #endif
 
-
 /**
  * Test an assertion for truth. If the expression is not true then
  * trigger a program failure.
@@ -342,6 +356,10 @@ inline void MOZ_PretendNoReturn()
 
 /*****************************************************************************/
 
+#if (defined(DEBUG) || (defined(NIGHTLY_BUILD) && !defined(MOZ_PROFILING))) && !defined(XPCOM_GLUE_AVOID_NSPR)
+  #define MOZ_THREAD_SAFETY_OWNERSHIP_CHECKS_SUPPORTED  1
+#endif
+
 #ifdef XPCOM_GLUE
   #define NS_CheckThreadSafe(owningThread, msg)
 #else
@@ -408,14 +426,14 @@ void fprintf_stderr(FILE* aFile, const char* aFmt, ...) MOZ_FORMAT_PRINTF(2, 3);
 // advanced performance debugging and display/layers visualization.
 void set_stderr_callback(StderrCallback aCallback);
 
-#if defined(ANDROID) && !defined(RELEASE_BUILD)
+#if defined(ANDROID) && !defined(RELEASE_OR_BETA)
 // Call this if you want a copy of stderr logging sent to a file. This is
 // useful to get around logcat overflow problems on android devices, which use
 // a circular logcat buffer and can intermittently drop messages if there's too
 // much spew.
 //
 // This is intended for local debugging only, DO NOT USE IN PRODUCTION CODE.
-// (This is ifndef RELEASE_BUILD to catch uses of it that accidentally get
+// (This is ifndef RELEASE_OR_BETA to catch uses of it that accidentally get
 // checked in). Using this will also prevent the profiler from getting a copy of
 // the stderr messages which it uses for various visualization features.
 //
