@@ -1,4 +1,5 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -29,9 +30,9 @@ extern "C" {
  *      + greater than zero if the second element should be before the first
  *  5. extra data to pass to comparison function
  */
-NS_COM_GLUE void NS_QuickSort(void *, unsigned int, unsigned int,
-                              int (*)(const void *, const void *, void *), 
-                              void *);
+void NS_QuickSort(void*, unsigned int, unsigned int,
+                  int (*)(const void*, const void*, void*),
+                  void*);
 
 #ifdef __cplusplus
 }
