@@ -14,11 +14,17 @@
 #include "nsIThread.h"
 #include "nsIRunnable.h"
 #include "nsICancelableRunnable.h"
+#include "nsIIdlePeriod.h"
+#include "nsIIncrementalRunnable.h"
 #include "nsStringGlue.h"
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
 #include "mozilla/Atomics.h"
+#include "mozilla/IndexSequence.h"
 #include "mozilla/Likely.h"
+#include "mozilla/Move.h"
+#include "mozilla/TimeStamp.h"
+#include "mozilla/Tuple.h"
 #include "mozilla/TypeTraits.h"
 
 //-----------------------------------------------------------------------------
@@ -56,7 +62,7 @@ NS_SetThreadName(nsIThread* aThread, const char (&aName)[LEN])
  * @returns NS_ERROR_INVALID_ARG
  *   Indicates that the given name is not unique.
  */
-extern NS_METHOD
+extern nsresult
 NS_NewThread(nsIThread** aResult,
              nsIRunnable* aInitialEvent = nullptr,
              uint32_t aStackSize = nsIThreadManager::DEFAULT_STACK_SIZE);
@@ -65,7 +71,7 @@ NS_NewThread(nsIThread** aResult,
  * Creates a named thread, otherwise the same as NS_NewThread
  */
 template<size_t LEN>
-inline NS_METHOD
+inline nsresult
 NS_NewNamedThread(const char (&aName)[LEN],
                   nsIThread** aResult,
                   nsIRunnable* aInitialEvent = nullptr,
@@ -80,7 +86,7 @@ NS_NewNamedThread(const char (&aName)[LEN],
   NS_SetThreadName<LEN>(thread, aName);
   if (aInitialEvent) {
     rv = thread->Dispatch(aInitialEvent, NS_DISPATCH_NORMAL);
-    NS_WARN_IF_FALSE(NS_SUCCEEDED(rv), "Initial event dispatch failed");
+    NS_WARNING_ASSERTION(NS_SUCCEEDED(rv), "Initial event dispatch failed");
   }
 
   *aResult = nullptr;
@@ -94,7 +100,7 @@ NS_NewNamedThread(const char (&aName)[LEN],
  * @param aResult
  *   The resulting nsIThread object.
  */
-extern NS_METHOD NS_GetCurrentThread(nsIThread** aResult);
+extern nsresult NS_GetCurrentThread(nsIThread** aResult);
 
 /**
  * Dispatch the given event to the current thread.
@@ -105,8 +111,8 @@ extern NS_METHOD NS_GetCurrentThread(nsIThread** aResult);
  * @returns NS_ERROR_INVALID_ARG
  *   If event is null.
  */
-extern NS_METHOD NS_DispatchToCurrentThread(nsIRunnable* aEvent);
-extern NS_METHOD
+extern nsresult NS_DispatchToCurrentThread(nsIRunnable* aEvent);
+extern nsresult
 NS_DispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent);
 
 /**
@@ -120,13 +126,20 @@ NS_DispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent);
  * @returns NS_ERROR_INVALID_ARG
  *   If event is null.
  */
-extern NS_METHOD
+extern nsresult
 NS_DispatchToMainThread(nsIRunnable* aEvent,
                         uint32_t aDispatchFlags = NS_DISPATCH_NORMAL);
-extern NS_METHOD
+extern nsresult
 NS_DispatchToMainThread(already_AddRefed<nsIRunnable>&& aEvent,
                         uint32_t aDispatchFlags = NS_DISPATCH_NORMAL);
 
+extern nsresult
+NS_DelayedDispatchToCurrentThread(
+  already_AddRefed<nsIRunnable>&& aEvent, uint32_t aDelayMs);
+
+extern nsresult
+NS_IdleDispatchToCurrentThread(already_AddRefed<nsIRunnable>&& aEvent);
+
 #ifndef XPCOM_GLUE_AVOID_NSPR
 /**
  * Process all pending events for the given thread before returning.  This
@@ -143,7 +156,7 @@ NS_DispatchToMainThread(already_AddRefed<nsIRunnable>&& aEvent,
  *   value is simply used to determine whether or not to process another event.
  *   Pass PR_INTERVAL_NO_TIMEOUT to specify no timeout.
  */
-extern NS_METHOD
+extern nsresult
 NS_ProcessPendingEvents(nsIThread* aThread,
                         PRIntervalTime aTimeout = PR_INTERVAL_NO_TIMEOUT);
 #endif
@@ -217,58 +230,117 @@ extern nsIThread* NS_GetCurrentThread();
 
 #ifndef XPCOM_GLUE_AVOID_NSPR
 
+namespace mozilla {
+
 // This class is designed to be subclassed.
-class nsRunnable : public nsIRunnable
+class IdlePeriod : public nsIIdlePeriod
+{
+public:
+  NS_DECL_THREADSAFE_ISUPPORTS
+  NS_DECL_NSIIDLEPERIOD
+
+  IdlePeriod() {}
+
+protected:
+  virtual ~IdlePeriod() {}
+private:
+  IdlePeriod(const IdlePeriod&) = delete;
+  IdlePeriod& operator=(const IdlePeriod&) = delete;
+  IdlePeriod& operator=(const IdlePeriod&&) = delete;
+};
+
+// This class is designed to be subclassed.
+class Runnable : public nsIRunnable
 {
 public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIRUNNABLE
 
-  nsRunnable() {}
+  Runnable() {}
 
 protected:
-  virtual ~nsRunnable() {}
+  virtual ~Runnable() {}
+private:
+  Runnable(const Runnable&) = delete;
+  Runnable& operator=(const Runnable&) = delete;
+  Runnable& operator=(const Runnable&&) = delete;
 };
 
 // This class is designed to be subclassed.
-class nsCancelableRunnable : public nsICancelableRunnable
+class CancelableRunnable : public Runnable,
+                           public nsICancelableRunnable
 {
 public:
-  NS_DECL_THREADSAFE_ISUPPORTS
-  NS_DECL_NSIRUNNABLE
-  NS_DECL_NSICANCELABLERUNNABLE
+  NS_DECL_ISUPPORTS_INHERITED
+  // nsICancelableRunnable
+  virtual nsresult Cancel() override;
 
-  nsCancelableRunnable() {}
+  CancelableRunnable() {}
 
 protected:
-  virtual ~nsCancelableRunnable() {}
+  virtual ~CancelableRunnable() {}
+private:
+  CancelableRunnable(const CancelableRunnable&) = delete;
+  CancelableRunnable& operator=(const CancelableRunnable&) = delete;
+  CancelableRunnable& operator=(const CancelableRunnable&&) = delete;
 };
 
+// This class is designed to be subclassed.
+class IncrementalRunnable : public CancelableRunnable,
+                            public nsIIncrementalRunnable
+{
+public:
+  NS_DECL_ISUPPORTS_INHERITED
+  // nsIIncrementalRunnable
+  virtual void SetDeadline(TimeStamp aDeadline) override;
+
+  IncrementalRunnable() {}
+
+protected:
+  virtual ~IncrementalRunnable() {}
+private:
+  IncrementalRunnable(const IncrementalRunnable&) = delete;
+  IncrementalRunnable& operator=(const IncrementalRunnable&) = delete;
+  IncrementalRunnable& operator=(const IncrementalRunnable&&) = delete;
+};
+
+namespace detail {
+
 // An event that can be used to call a C++11 functions or function objects,
 // including lambdas. The function must have no required arguments, and must
 // return void.
-template<typename Function>
-class nsRunnableFunction : public nsRunnable
+template<typename StoredFunction>
+class RunnableFunction : public Runnable
 {
 public:
-  explicit nsRunnableFunction(const Function& aFunction)
-    : mFunction(aFunction)
+  template <typename F>
+  explicit RunnableFunction(F&& aFunction)
+    : mFunction(Forward<F>(aFunction))
   { }
 
-  NS_IMETHOD Run() {
-    static_assert(mozilla::IsVoid<decltype(mFunction())>::value,
+  NS_IMETHOD Run() override {
+    static_assert(IsVoid<decltype(mFunction())>::value,
                   "The lambda must return void!");
     mFunction();
     return NS_OK;
   }
 private:
-  Function mFunction;
+  StoredFunction mFunction;
 };
 
+} // namespace detail
+
+} // namespace mozilla
+
 template<typename Function>
-nsRunnableFunction<Function>* NS_NewRunnableFunction(const Function& aFunction)
+already_AddRefed<mozilla::Runnable>
+NS_NewRunnableFunction(Function&& aFunction)
 {
-  return new nsRunnableFunction<Function>(aFunction);
+  return do_AddRef(new mozilla::detail::RunnableFunction
+                   // Make sure we store a non-reference in nsRunnableFunction.
+                   <typename mozilla::RemoveReference<Function>::Type>
+                   // But still forward aFunction to move if possible.
+                   (mozilla::Forward<Function>(aFunction)));
 }
 
 // An event that can be used to call a method on a class.  The class type must
@@ -276,8 +348,11 @@ nsRunnableFunction<Function>* NS_NewRunnableFunction(const Function& aFunction)
 // with nsRevocableEventPtr.
 template<class ClassType,
          typename ReturnType = void,
-         bool Owning = true>
-class nsRunnableMethod : public nsRunnable
+         bool Owning = true,
+         bool Cancelable = false>
+class nsRunnableMethod : public mozilla::Conditional<!Cancelable,
+                                                     mozilla::Runnable,
+                                                     mozilla::CancelableRunnable>::Type
 {
 public:
   virtual void Revoke() = 0;
@@ -321,31 +396,60 @@ struct nsRunnableMethodReceiver<ClassType, false>
   void Revoke() { mObj = nullptr; }
 };
 
-template<typename Method, bool Owning> struct nsRunnableMethodTraits;
+template<typename Method, bool Owning, bool Cancelable> struct nsRunnableMethodTraits;
 
-template<class C, typename R, bool Owning, typename... As>
-struct nsRunnableMethodTraits<R(C::*)(As...), Owning>
+template<class C, typename R, bool Owning, bool Cancelable, typename... As>
+struct nsRunnableMethodTraits<R(C::*)(As...), Owning, Cancelable>
 {
   typedef C class_type;
   typedef R return_type;
-  typedef nsRunnableMethod<C, R, Owning> base_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
+};
+
+template<class C, typename R, bool Owning, bool Cancelable, typename... As>
+struct nsRunnableMethodTraits<R(C::*)(As...) const, Owning, Cancelable>
+{
+  typedef const C class_type;
+  typedef R return_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
 };
 
 #ifdef NS_HAVE_STDCALL
-template<class C, typename R, bool Owning, typename... As>
-struct nsRunnableMethodTraits<R(__stdcall C::*)(As...), Owning>
+template<class C, typename R, bool Owning, bool Cancelable, typename... As>
+struct nsRunnableMethodTraits<R(__stdcall C::*)(As...), Owning, Cancelable>
 {
   typedef C class_type;
   typedef R return_type;
-  typedef nsRunnableMethod<C, R, Owning> base_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
 };
 
-template<class C, typename R, bool Owning>
-struct nsRunnableMethodTraits<R(NS_STDCALL C::*)(), Owning>
+template<class C, typename R, bool Owning, bool Cancelable>
+struct nsRunnableMethodTraits<R(NS_STDCALL C::*)(), Owning, Cancelable>
 {
   typedef C class_type;
   typedef R return_type;
-  typedef nsRunnableMethod<C, R, Owning> base_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
+};
+template<class C, typename R, bool Owning, bool Cancelable, typename... As>
+struct nsRunnableMethodTraits<R(__stdcall C::*)(As...) const, Owning, Cancelable>
+{
+  typedef const C class_type;
+  typedef R return_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
+};
+
+template<class C, typename R, bool Owning, bool Cancelable>
+struct nsRunnableMethodTraits<R(NS_STDCALL C::*)() const, Owning, Cancelable>
+{
+  typedef const C class_type;
+  typedef R return_type;
+  typedef nsRunnableMethod<C, R, Owning, Cancelable> base_type;
+  static const bool can_cancel = Cancelable;
 };
 #endif
 
@@ -369,7 +473,7 @@ struct StoreCopyPassByValue
   typedef T passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByValue(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByValue(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -383,7 +487,7 @@ struct StoreCopyPassByConstLRef
   typedef const T& passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByConstLRef(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByConstLRef(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -397,7 +501,7 @@ struct StoreCopyPassByLRef
   typedef T& passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByLRef(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByLRef(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -411,7 +515,7 @@ struct StoreCopyPassByRRef
   typedef T&& passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByRRef(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByRRef(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return mozilla::Move(m); }
 };
 template<typename S>
@@ -425,7 +529,7 @@ struct StoreRefPassByLRef
   typedef T& passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreRefPassByLRef(A& a) : m(a) {}
+  MOZ_IMPLICIT StoreRefPassByLRef(A& a) : m(a) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -439,7 +543,7 @@ struct StoreConstRefPassByConstLRef
   typedef const T& passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreConstRefPassByConstLRef(const A& a) : m(a) {}
+  MOZ_IMPLICIT StoreConstRefPassByConstLRef(const A& a) : m(a) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -453,7 +557,7 @@ struct StorensRefPtrPassByPtr
   typedef T* passed_type;
   stored_type m;
   template <typename A>
-  explicit StorensRefPtrPassByPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StorensRefPtrPassByPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return m.get(); }
 };
 template<typename S>
@@ -467,7 +571,7 @@ struct StorePtrPassByPtr
   typedef T* passed_type;
   stored_type m;
   template <typename A>
-  explicit StorePtrPassByPtr(A a) : m(a) {}
+  MOZ_IMPLICIT StorePtrPassByPtr(A a) : m(a) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -481,7 +585,7 @@ struct StoreConstPtrPassByConstPtr
   typedef const T* passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreConstPtrPassByConstPtr(A a) : m(a) {}
+  MOZ_IMPLICIT StoreConstPtrPassByConstPtr(A a) : m(a) {}
   passed_type PassAsParameter() { return m; }
 };
 template<typename S>
@@ -495,7 +599,7 @@ struct StoreCopyPassByConstPtr
   typedef const T* passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByConstPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByConstPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return &m; }
 };
 template<typename S>
@@ -509,7 +613,7 @@ struct StoreCopyPassByPtr
   typedef T* passed_type;
   stored_type m;
   template <typename A>
-  explicit StoreCopyPassByPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
+  MOZ_IMPLICIT StoreCopyPassByPtr(A&& a) : m(mozilla::Forward<A>(a)) {}
   passed_type PassAsParameter() { return &m; }
 };
 template<typename S>
@@ -650,217 +754,53 @@ struct ParameterStorage
 
 } /* namespace detail */
 
+namespace mozilla {
+
+namespace detail {
+
 // struct used to store arguments and later apply them to a method.
-template <typename... Ts> struct nsRunnableMethodArguments;
-
-// Specializations for 0-8 arguments, add more as required.
-// TODO Use tuple instead; And/or use lambdas (see bug 1152753)
-template <>
-struct nsRunnableMethodArguments<>
+template <typename... Ts>
+struct RunnableMethodArguments final
 {
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)();
-  }
-};
-template <typename T0>
-struct nsRunnableMethodArguments<T0>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  template<typename A0>
-  explicit nsRunnableMethodArguments(A0&& a0)
-    : m0(mozilla::Forward<A0>(a0))
+  Tuple<typename ::detail::ParameterStorage<Ts>::Type...> mArguments;
+  template <typename... As>
+  explicit RunnableMethodArguments(As&&... aArguments)
+    : mArguments(Forward<As>(aArguments)...)
   {}
-  template<class C, typename M> void apply(C* o, M m)
+  template<typename C, typename M, typename... Args, size_t... Indices>
+  static auto
+  applyImpl(C* o, M m, Tuple<Args...>& args, IndexSequence<Indices...>)
+      -> decltype(((*o).*m)(Get<Indices>(args).PassAsParameter()...))
   {
-    ((*o).*m)(m0.PassAsParameter());
+    return ((*o).*m)(Get<Indices>(args).PassAsParameter()...);
   }
-};
-template <typename T0, typename T1>
-struct nsRunnableMethodArguments<T0, T1>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  template<typename A0, typename A1>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
+  template<class C, typename M> auto apply(C* o, M m)
+      -> decltype(applyImpl(o, m, mArguments,
+                  typename IndexSequenceFor<Ts...>::Type()))
   {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2>
-struct nsRunnableMethodArguments<T0, T1, T2>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  template<typename A0, typename A1, typename A2>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(), m2.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2, typename T3>
-struct nsRunnableMethodArguments<T0, T1, T2, T3>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  typename ::detail::ParameterStorage<T3>::Type m3;
-  template<typename A0, typename A1, typename A2, typename A3>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2, A3&& a3)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-    , m3(mozilla::Forward<A3>(a3))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(),
-              m2.PassAsParameter(), m3.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2, typename T3, typename T4>
-struct nsRunnableMethodArguments<T0, T1, T2, T3, T4>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  typename ::detail::ParameterStorage<T3>::Type m3;
-  typename ::detail::ParameterStorage<T4>::Type m4;
-  template<typename A0, typename A1, typename A2, typename A3, typename A4>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2, A3&& a3, A4&& a4)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-    , m3(mozilla::Forward<A3>(a3))
-    , m4(mozilla::Forward<A4>(a4))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(),
-              m2.PassAsParameter(), m3.PassAsParameter(),
-              m4.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2, typename T3, typename T4,
-          typename T5>
-struct nsRunnableMethodArguments<T0, T1, T2, T3, T4, T5>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  typename ::detail::ParameterStorage<T3>::Type m3;
-  typename ::detail::ParameterStorage<T4>::Type m4;
-  typename ::detail::ParameterStorage<T5>::Type m5;
-  template<typename A0, typename A1, typename A2, typename A3, typename A4,
-           typename A5>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2, A3&& a3, A4&& a4,
-        A5&& a5)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-    , m3(mozilla::Forward<A3>(a3))
-    , m4(mozilla::Forward<A4>(a4))
-    , m5(mozilla::Forward<A5>(a5))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(),
-              m2.PassAsParameter(), m3.PassAsParameter(),
-              m4.PassAsParameter(), m5.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2, typename T3, typename T4,
-          typename T5, typename T6>
-struct nsRunnableMethodArguments<T0, T1, T2, T3, T4, T5, T6>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  typename ::detail::ParameterStorage<T3>::Type m3;
-  typename ::detail::ParameterStorage<T4>::Type m4;
-  typename ::detail::ParameterStorage<T5>::Type m5;
-  typename ::detail::ParameterStorage<T6>::Type m6;
-  template<typename A0, typename A1, typename A2, typename A3, typename A4,
-           typename A5, typename A6>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2, A3&& a3, A4&& a4,
-        A5&& a5, A6&& a6)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-    , m3(mozilla::Forward<A3>(a3))
-    , m4(mozilla::Forward<A4>(a4))
-    , m5(mozilla::Forward<A5>(a5))
-    , m6(mozilla::Forward<A6>(a6))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(),
-              m2.PassAsParameter(), m3.PassAsParameter(),
-              m4.PassAsParameter(), m5.PassAsParameter(),
-              m6.PassAsParameter());
-  }
-};
-template <typename T0, typename T1, typename T2, typename T3, typename T4,
-          typename T5, typename T6, typename T7>
-struct nsRunnableMethodArguments<T0, T1, T2, T3, T4, T5, T6, T7>
-{
-  typename ::detail::ParameterStorage<T0>::Type m0;
-  typename ::detail::ParameterStorage<T1>::Type m1;
-  typename ::detail::ParameterStorage<T2>::Type m2;
-  typename ::detail::ParameterStorage<T3>::Type m3;
-  typename ::detail::ParameterStorage<T4>::Type m4;
-  typename ::detail::ParameterStorage<T5>::Type m5;
-  typename ::detail::ParameterStorage<T6>::Type m6;
-  typename ::detail::ParameterStorage<T7>::Type m7;
-  template<typename A0, typename A1, typename A2, typename A3, typename A4,
-           typename A5, typename A6, typename A7>
-  nsRunnableMethodArguments(A0&& a0, A1&& a1, A2&& a2, A3&& a3, A4&& a4,
-        A5&& a5, A6&& a6, A7&& a7)
-    : m0(mozilla::Forward<A0>(a0))
-    , m1(mozilla::Forward<A1>(a1))
-    , m2(mozilla::Forward<A2>(a2))
-    , m3(mozilla::Forward<A3>(a3))
-    , m4(mozilla::Forward<A4>(a4))
-    , m5(mozilla::Forward<A5>(a5))
-    , m6(mozilla::Forward<A6>(a6))
-    , m7(mozilla::Forward<A7>(a7))
-  {}
-  template<class C, typename M> void apply(C* o, M m)
-  {
-    ((*o).*m)(m0.PassAsParameter(), m1.PassAsParameter(),
-              m2.PassAsParameter(), m3.PassAsParameter(),
-              m4.PassAsParameter(), m5.PassAsParameter(),
-              m6.PassAsParameter(), m7.PassAsParameter());
+    return applyImpl(o, m, mArguments,
+        typename IndexSequenceFor<Ts...>::Type());
   }
 };
 
-template<typename Method, bool Owning, typename... Storages>
-class nsRunnableMethodImpl
-  : public nsRunnableMethodTraits<Method, Owning>::base_type
+template<typename Method, bool Owning, bool Cancelable, typename... Storages>
+class RunnableMethodImpl final
+  : public ::nsRunnableMethodTraits<Method, Owning, Cancelable>::base_type
 {
-  typedef typename nsRunnableMethodTraits<Method, Owning>::class_type
+  typedef typename ::nsRunnableMethodTraits<Method, Owning, Cancelable>::class_type
       ClassType;
-  nsRunnableMethodReceiver<ClassType, Owning> mReceiver;
+  ::nsRunnableMethodReceiver<ClassType, Owning> mReceiver;
   Method mMethod;
-  nsRunnableMethodArguments<Storages...> mArgs;
+  RunnableMethodArguments<Storages...> mArgs;
+private:
+  virtual ~RunnableMethodImpl() { Revoke(); };
 public:
-  virtual ~nsRunnableMethodImpl() { Revoke(); };
   template<typename... Args>
-  explicit nsRunnableMethodImpl(ClassType* aObj, Method aMethod,
-                                Args&&... aArgs)
+  explicit RunnableMethodImpl(ClassType* aObj, Method aMethod,
+                              Args&&... aArgs)
     : mReceiver(aObj)
     , mMethod(aMethod)
-    , mArgs(mozilla::Forward<Args>(aArgs)...)
+    , mArgs(Forward<Args>(aArgs)...)
   {
     static_assert(sizeof...(Storages) == sizeof...(Args), "Storages and Args should have equal sizes");
   }
@@ -871,70 +811,102 @@ public:
     }
     return NS_OK;
   }
+  nsresult Cancel() {
+    static_assert(Cancelable, "Don't use me!");
+    Revoke();
+    return NS_OK;
+  }
   void Revoke() { mReceiver.Revoke(); }
 };
 
+} // namespace detail
+
 // Use this template function like so:
 //
 //   nsCOMPtr<nsIRunnable> event =
-//     NS_NewRunnableMethod(myObject, &MyClass::HandleEvent);
+//     mozilla::NewRunnableMethod(myObject, &MyClass::HandleEvent);
 //   NS_DispatchToCurrentThread(event);
 //
 // Statically enforced constraints:
 //  - myObject must be of (or implicitly convertible to) type MyClass
-//  - MyClass must defined AddRef and Release methods
+//  - MyClass must define AddRef and Release methods
 //
+
 template<typename PtrType, typename Method>
-typename nsRunnableMethodTraits<Method, true>::base_type*
-NS_NewRunnableMethod(PtrType aPtr, Method aMethod)
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, true, false>::base_type>
+NewRunnableMethod(PtrType aPtr, Method aMethod)
 {
-  return new nsRunnableMethodImpl<Method, true>(aPtr, aMethod);
+  return do_AddRef(new detail::RunnableMethodImpl<Method, true, false>(aPtr, aMethod));
 }
 
 template<typename PtrType, typename Method>
-typename nsRunnableMethodTraits<Method, false>::base_type*
-NS_NewNonOwningRunnableMethod(PtrType&& aPtr, Method aMethod)
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, true, true>::base_type>
+NewCancelableRunnableMethod(PtrType aPtr, Method aMethod)
 {
-  return new nsRunnableMethodImpl<Method, false>(aPtr, aMethod);
+  return do_AddRef(new detail::RunnableMethodImpl<Method, true, true>(aPtr, aMethod));
 }
 
-// Similar to NS_NewRunnableMethod. Call like so:
-// nsCOMPtr<nsIRunnable> event =
-//   NS_NewRunnableMethodWithArg<Type>(myObject, &MyClass::HandleEvent, myArg);
-// 'Type' is the stored type for the argument, see ParameterStorage for details.
-template<typename Storage, typename Method, typename PtrType, typename Arg>
-typename nsRunnableMethodTraits<Method, true>::base_type*
-NS_NewRunnableMethodWithArg(PtrType&& aPtr, Method aMethod, Arg&& aArg)
+template<typename PtrType, typename Method>
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, false, false>::base_type>
+NewNonOwningRunnableMethod(PtrType&& aPtr, Method aMethod)
 {
-  return new nsRunnableMethodImpl<Method, true, Storage>(
-      aPtr, aMethod, mozilla::Forward<Arg>(aArg));
+  return do_AddRef(new detail::RunnableMethodImpl<Method, false, false>(aPtr, aMethod));
 }
 
-// Similar to NS_NewRunnableMethod. Call like so:
+template<typename PtrType, typename Method>
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, false, true>::base_type>
+NewNonOwningCancelableRunnableMethod(PtrType&& aPtr, Method aMethod)
+{
+  return do_AddRef(new detail::RunnableMethodImpl<Method, false, true>(aPtr, aMethod));
+}
+
+// Similar to NewRunnableMethod. Call like so:
 // nsCOMPtr<nsIRunnable> event =
-//   NS_NewRunnableMethodWithArg<Types,...>(myObject, &MyClass::HandleEvent, myArg1,...);
+//   NewRunnableMethod<Types,...>(myObject, &MyClass::HandleEvent, myArg1,...);
 // 'Types' are the stored type for each argument, see ParameterStorage for details.
 template<typename... Storages, typename Method, typename PtrType, typename... Args>
-typename nsRunnableMethodTraits<Method, true>::base_type*
-NS_NewRunnableMethodWithArgs(PtrType&& aPtr, Method aMethod, Args&&... aArgs)
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, true, false>::base_type>
+NewRunnableMethod(PtrType&& aPtr, Method aMethod, Args&&... aArgs)
 {
   static_assert(sizeof...(Storages) == sizeof...(Args),
                 "<Storages...> size should be equal to number of arguments");
-  return new nsRunnableMethodImpl<Method, true, Storages...>(
-      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...);
+  return do_AddRef(new detail::RunnableMethodImpl<Method, true, false, Storages...>(
+      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...));
 }
 
 template<typename... Storages, typename Method, typename PtrType, typename... Args>
-typename nsRunnableMethodTraits<Method, false>::base_type*
-NS_NewNonOwningRunnableMethodWithArgs(PtrType&& aPtr, Method aMethod,
-                                      Args&&... aArgs)
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, false, false>::base_type>
+NewNonOwningRunnableMethod(PtrType&& aPtr, Method aMethod, Args&&... aArgs)
 {
   static_assert(sizeof...(Storages) == sizeof...(Args),
                 "<Storages...> size should be equal to number of arguments");
-  return new nsRunnableMethodImpl<Method, false, Storages...>(
-      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...);
+  return do_AddRef(new detail::RunnableMethodImpl<Method, false, false, Storages...>(
+      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...));
 }
 
+template<typename... Storages, typename Method, typename PtrType, typename... Args>
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, true, true>::base_type>
+NewCancelableRunnableMethod(PtrType&& aPtr, Method aMethod, Args&&... aArgs)
+{
+  static_assert(sizeof...(Storages) == sizeof...(Args),
+                "<Storages...> size should be equal to number of arguments");
+  return do_AddRef(new detail::RunnableMethodImpl<Method, true, true, Storages...>(
+      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...));
+}
+
+template<typename... Storages, typename Method, typename PtrType, typename... Args>
+already_AddRefed<typename ::nsRunnableMethodTraits<Method, false, true>::base_type>
+NewNonOwningCancelableRunnableMethod(PtrType&& aPtr, Method aMethod,
+                                                Args&&... aArgs)
+{
+  static_assert(sizeof...(Storages) == sizeof...(Args),
+                "<Storages...> size should be equal to number of arguments");
+  return do_AddRef(new detail::RunnableMethodImpl<Method, false, true, Storages...>(
+      aPtr, aMethod, mozilla::Forward<Args>(aArgs)...));
+}
+
+} // namespace mozilla
+
 #endif  // XPCOM_GLUE_AVOID_NSPR
 
 // This class is designed to be used when you have an event class E that has a
@@ -945,7 +917,7 @@ NS_NewNonOwningRunnableMethodWithArgs(PtrType&& aPtr, Method aMethod,
 //
 //   class R;
 //
-//   class E : public nsRunnable {
+//   class E : public mozilla::Runnable {
 //   public:
 //     void Revoke() {
 //       mResource = nullptr;
@@ -998,6 +970,16 @@ public:
     return *this;
   }
 
+  const nsRevocableEventPtr& operator=(already_AddRefed<T> aEvent)
+  {
+    RefPtr<T> event = aEvent;
+    if (mEvent != event) {
+      Revoke();
+      mEvent = event.forget();
+    }
+    return *this;
+  }
+
   void Revoke()
   {
     if (mEvent) {
