@@ -21,7 +21,7 @@
 #include "nsIInputStream.h"
 #include "nsIIPCSerializableInputStream.h"
 #include "nsISeekableStream.h"
-#include "prlog.h"
+#include "mozilla/Logging.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/Likely.h"
 #include "mozilla/MathAlgorithms.h"
@@ -29,33 +29,25 @@
 
 using mozilla::ipc::InputStreamParams;
 using mozilla::ipc::StringInputStreamParams;
+using mozilla::Maybe;
+using mozilla::Some;
 
-#if defined(PR_LOGGING)
 //
 // Log module for StorageStream logging...
 //
 // To enable logging (see prlog.h for full details):
 //
-//    set NSPR_LOG_MODULES=StorageStreamLog:5
-//    set NSPR_LOG_FILE=nspr.log
+//    set MOZ_LOG=StorageStreamLog:5
+//    set MOZ_LOG_FILE=storage.log
 //
-// this enables PR_LOG_DEBUG level information and places all output in
-// the file nspr.log
+// This enables LogLevel::Debug level information and places all output in
+// the file storage.log.
 //
-static PRLogModuleInfo*
-GetStorageStreamLog()
-{
-  static PRLogModuleInfo* sLog;
-  if (!sLog) {
-    sLog = PR_NewLogModule("nsStorageStream");
-  }
-  return sLog;
-}
-#endif
+static LazyLogModule sStorageStreamLog("nsStorageStream");
 #ifdef LOG
 #undef LOG
 #endif
-#define LOG(args) PR_LOG(GetStorageStreamLog(), PR_LOG_DEBUG, args)
+#define LOG(args) MOZ_LOG(sStorageStreamLog, mozilla::LogLevel::Debug, args)
 
 nsStorageStream::nsStorageStream()
   : mSegmentedBuffer(0), mSegmentSize(0), mWriteInProgress(false),
@@ -77,10 +69,6 @@ NS_IMETHODIMP
 nsStorageStream::Init(uint32_t aSegmentSize, uint32_t aMaxSize)
 {
   mSegmentedBuffer = new nsSegmentedBuffer();
-  if (!mSegmentedBuffer) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
   mSegmentSize = aSegmentSize;
   mSegmentSizeLog2 = mozilla::FloorLog2(aSegmentSize);
 
@@ -213,7 +201,7 @@ nsStorageStream::Write(const char* aBuffer, uint32_t aCount,
     mWriteCursor += count;
     LOG(("nsStorageStream [%p] Writing mWriteCursor=%x mSegmentEnd=%x count=%d\n",
          this, mWriteCursor, mSegmentEnd, count));
-  };
+  }
 
 out:
   *aNumWritten = aCount - remaining;
@@ -290,7 +278,7 @@ nsStorageStream::GetWriteInProgress(bool* aWriteInProgress)
   return NS_OK;
 }
 
-NS_METHOD
+nsresult
 nsStorageStream::Seek(int32_t aPosition)
 {
   if (NS_WARN_IF(!mSegmentedBuffer)) {
@@ -355,7 +343,6 @@ public:
       mSegmentSize(aSegmentSize), mLogicalCursor(0),
       mStatus(NS_OK)
   {
-    NS_ADDREF(mStorageStream);
   }
 
   NS_DECL_THREADSAFE_ISUPPORTS
@@ -367,16 +354,15 @@ public:
 private:
   ~nsStorageInputStream()
   {
-    NS_IF_RELEASE(mStorageStream);
   }
 
 protected:
-  NS_METHOD Seek(uint32_t aPosition);
+  nsresult Seek(uint32_t aPosition);
 
   friend class nsStorageStream;
 
 private:
-  nsStorageStream* mStorageStream;
+  RefPtr<nsStorageStream> mStorageStream;
   uint32_t         mReadCursor;    // Next memory location to read byte, or 0
   uint32_t         mSegmentEnd;    // One byte past end of current buffer segment
   uint32_t         mSegmentNum;    // Segment number containing read cursor
@@ -408,21 +394,15 @@ nsStorageStream::NewInputStream(int32_t aStartingOffset,
     return NS_ERROR_NOT_INITIALIZED;
   }
 
-  nsStorageInputStream* inputStream =
+  RefPtr<nsStorageInputStream> inputStream =
     new nsStorageInputStream(this, mSegmentSize);
-  if (!inputStream) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  NS_ADDREF(inputStream);
 
   nsresult rv = inputStream->Seek(aStartingOffset);
   if (NS_FAILED(rv)) {
-    NS_RELEASE(inputStream);
     return rv;
   }
 
-  *aInputStream = inputStream;
+  inputStream.forget(aInputStream);
   return NS_OK;
 }
 
@@ -474,7 +454,15 @@ nsStorageInputStream::ReadSegments(nsWriteSegmentFun aWriter, void* aClosure,
         goto out;
       }
 
-      mSegmentNum++;
+      // We have data in the stream, but if mSegmentEnd is zero, then we
+      // were likely constructed prior to any data being written into
+      // the stream.  Therefore, if mSegmentEnd is non-zero, we should
+      // move into the next segment; otherwise, we should stay in this
+      // segment so our input state can be updated and we can properly
+      // perform the initial read.
+      if (mSegmentEnd > 0) {
+        mSegmentNum++;
+      }
       mReadCursor = 0;
       mSegmentEnd = XPCOM_MIN(mSegmentSize, available);
       availableInSegment = mSegmentEnd;
@@ -490,7 +478,7 @@ nsStorageInputStream::ReadSegments(nsWriteSegmentFun aWriter, void* aClosure,
     remainingCapacity -= bytesConsumed;
     mReadCursor += bytesConsumed;
     mLogicalCursor += bytesConsumed;
-  };
+  }
 
 out:
   *aNumRead = aCount - remainingCapacity;
@@ -564,7 +552,7 @@ nsStorageInputStream::SetEOF()
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_METHOD
+nsresult
 nsStorageInputStream::Seek(uint32_t aPosition)
 {
   uint32_t length = mStorageStream->mLogicalLength;
@@ -612,6 +600,15 @@ nsStorageInputStream::Serialize(InputStreamParams& aParams, FileDescriptorArray&
   aParams = params;
 }
 
+Maybe<uint64_t>
+nsStorageInputStream::ExpectedSerializedLength()
+{
+  uint64_t remaining = 0;
+  DebugOnly<nsresult> rv = Available(&remaining);
+  MOZ_ASSERT(NS_SUCCEEDED(rv));
+  return Some(remaining);
+}
+
 bool
 nsStorageInputStream::Deserialize(const InputStreamParams& aParams,
                                   const FileDescriptorArray&)
@@ -637,18 +634,12 @@ nsresult
 NS_NewStorageStream(uint32_t aSegmentSize, uint32_t aMaxSize,
                     nsIStorageStream** aResult)
 {
-  nsStorageStream* storageStream = new nsStorageStream();
-  if (!storageStream) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  NS_ADDREF(storageStream);
+  RefPtr<nsStorageStream> storageStream = new nsStorageStream();
   nsresult rv = storageStream->Init(aSegmentSize, aMaxSize);
   if (NS_FAILED(rv)) {
-    NS_RELEASE(storageStream);
     return rv;
   }
-  *aResult = storageStream;
+  storageStream.forget(aResult);
   return NS_OK;
 }
 
