@@ -315,13 +315,13 @@ public:
    * for wide strings. Call this version when you know the
    * length of 'data'.
    */
-  bool NS_FASTCALL B2G_ACL_EXPORT EqualsASCII(const char* aData, size_type aLen) const;
+  bool NS_FASTCALL EqualsASCII(const char* aData, size_type aLen) const;
   /**
    * An efficient comparison with ASCII that can be used even
    * for wide strings. Call this version when 'data' is
    * null-terminated.
    */
-  bool NS_FASTCALL B2G_ACL_EXPORT EqualsASCII(const char* aData) const;
+  bool NS_FASTCALL EqualsASCII(const char* aData) const;
 
   // EqualsLiteral must ONLY be applied to an actual literal string, or
   // a char array *constant* declared without an explicit size.
@@ -361,22 +361,22 @@ public:
    */
 
   void NS_FASTCALL Assign(char_type aChar);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Assign(char_type aChar,
-                                                 const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Assign(char_type aChar, const fallible_t&);
 
   void NS_FASTCALL Assign(const char_type* aData);
+  MOZ_MUST_USE bool NS_FASTCALL Assign(const char_type* aData,
+                                       const fallible_t&);
+
   void NS_FASTCALL Assign(const char_type* aData, size_type aLength);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Assign(const char_type* aData,
-                                                 size_type aLength,
-                                                 const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Assign(const char_type* aData,
+                                       size_type aLength, const fallible_t&);
 
   void NS_FASTCALL Assign(const self_type&);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Assign(const self_type&,
-                                                 const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Assign(const self_type&, const fallible_t&);
 
   void NS_FASTCALL Assign(const substring_tuple_type&);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Assign(const substring_tuple_type&,
-                                                 const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Assign(const substring_tuple_type&,
+                                       const fallible_t&);
 
 #if defined(CharT_is_PRUnichar) && defined(MOZ_USE_CHAR16_WRAPPER)
   void Assign(char16ptr_t aData)
@@ -384,19 +384,13 @@ public:
     Assign(static_cast<const char16_t*>(aData));
   }
 
-  MOZ_WARN_UNUSED_RESULT bool Assign(char16ptr_t aData,
-                                     const fallible_t& aFallible)
-  {
-    return Assign(static_cast<const char16_t*>(aData), aFallible);
-  }
-
   void Assign(char16ptr_t aData, size_type aLength)
   {
     Assign(static_cast<const char16_t*>(aData), aLength);
   }
 
-  MOZ_WARN_UNUSED_RESULT bool Assign(char16ptr_t aData, size_type aLength,
-                                     const fallible_t& aFallible)
+  MOZ_MUST_USE bool Assign(char16ptr_t aData, size_type aLength,
+                           const fallible_t& aFallible)
   {
     return Assign(static_cast<const char16_t*>(aData), aLength,
                   aFallible);
@@ -404,16 +398,16 @@ public:
 #endif
 
   void NS_FASTCALL AssignASCII(const char* aData, size_type aLength);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL AssignASCII(const char* aData,
-                                                      size_type aLength,
-                                                      const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL AssignASCII(const char* aData,
+                                            size_type aLength,
+                                            const fallible_t&);
 
   void NS_FASTCALL AssignASCII(const char* aData)
   {
     AssignASCII(aData, mozilla::AssertedCast<size_type, size_t>(strlen(aData)));
   }
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL AssignASCII(const char* aData,
-                                                      const fallible_t& aFallible)
+  MOZ_MUST_USE bool NS_FASTCALL AssignASCII(const char* aData,
+                                            const fallible_t& aFallible)
   {
     return AssignASCII(aData,
                        mozilla::AssertedCast<size_type, size_t>(strlen(aData)),
@@ -476,27 +470,27 @@ public:
 
   void NS_FASTCALL Replace(index_type aCutStart, size_type aCutLength,
                            char_type aChar);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Replace(index_type aCutStart,
-                                                  size_type aCutLength,
-                                                  char_type aChar,
-                                                  const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Replace(index_type aCutStart,
+                                        size_type aCutLength,
+                                        char_type aChar,
+                                        const fallible_t&);
   void NS_FASTCALL Replace(index_type aCutStart, size_type aCutLength,
                            const char_type* aData,
                            size_type aLength = size_type(-1));
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL Replace(index_type aCutStart,
-                                                  size_type aCutLength,
-                                                  const char_type* aData,
-                                                  size_type aLength,
-                                                  const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL Replace(index_type aCutStart,
+                                        size_type aCutLength,
+                                        const char_type* aData,
+                                        size_type aLength,
+                                        const fallible_t&);
   void Replace(index_type aCutStart, size_type aCutLength,
                const self_type& aStr)
   {
     Replace(aCutStart, aCutLength, aStr.Data(), aStr.Length());
   }
-  MOZ_WARN_UNUSED_RESULT bool Replace(index_type aCutStart,
-                                      size_type aCutLength,
-                                      const self_type& aStr,
-                                      const fallible_t& aFallible)
+  MOZ_MUST_USE bool Replace(index_type aCutStart,
+                            size_type aCutLength,
+                            const self_type& aStr,
+                            const fallible_t& aFallible)
   {
     return Replace(aCutStart, aCutLength, aStr.Data(), aStr.Length(),
                    aFallible);
@@ -508,10 +502,10 @@ public:
                                 const char* aData,
                                 size_type aLength = size_type(-1));
 
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL ReplaceASCII(index_type aCutStart, size_type aCutLength,
-                                                       const char* aData,
-                                                       size_type aLength,
-                                                       const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL ReplaceASCII(index_type aCutStart, size_type aCutLength,
+                                             const char* aData,
+                                             size_type aLength,
+                                             const fallible_t&);
 
   // ReplaceLiteral must ONLY be applied to an actual literal string.
   // Do not attempt to use it with a regular char* pointer, or with a char
@@ -527,8 +521,7 @@ public:
   {
     Replace(mLength, 0, aChar);
   }
-  MOZ_WARN_UNUSED_RESULT bool Append(char_type aChar,
-                                     const fallible_t& aFallible)
+  MOZ_MUST_USE bool Append(char_type aChar, const fallible_t& aFallible)
   {
     return Replace(mLength, 0, aChar, aFallible);
   }
@@ -536,8 +529,8 @@ public:
   {
     Replace(mLength, 0, aData, aLength);
   }
-  MOZ_WARN_UNUSED_RESULT bool Append(const char_type* aData, size_type aLength,
-                                     const fallible_t& aFallible)
+  MOZ_MUST_USE bool Append(const char_type* aData, size_type aLength,
+                           const fallible_t& aFallible)
   {
     return Replace(mLength, 0, aData, aLength, aFallible);
   }
@@ -553,7 +546,7 @@ public:
   {
     Replace(mLength, 0, aStr);
   }
-  MOZ_WARN_UNUSED_RESULT bool Append(const self_type& aStr, const fallible_t& aFallible)
+  MOZ_MUST_USE bool Append(const self_type& aStr, const fallible_t& aFallible)
   {
     return Replace(mLength, 0, aStr, aFallible);
   }
@@ -567,12 +560,12 @@ public:
     ReplaceASCII(mLength, 0, aData, aLength);
   }
 
-  MOZ_WARN_UNUSED_RESULT bool AppendASCII(const char* aData, const fallible_t& aFallible)
+  MOZ_MUST_USE bool AppendASCII(const char* aData, const fallible_t& aFallible)
   {
     return ReplaceASCII(mLength, 0, aData, size_type(-1), aFallible);
   }
 
-  MOZ_WARN_UNUSED_RESULT bool AppendASCII(const char* aData, size_type aLength, const fallible_t& aFallible)
+  MOZ_MUST_USE bool AppendASCII(const char* aData, size_type aLength, const fallible_t& aFallible)
   {
     return ReplaceASCII(mLength, 0, aData, aLength, aFallible);
   }
@@ -643,7 +636,7 @@ public:
   }
 
   template<int N>
-  MOZ_WARN_UNUSED_RESULT     bool AppendLiteral(const char (&aStr)[N], const fallible_t& aFallible)
+  MOZ_MUST_USE bool AppendLiteral(const char (&aStr)[N], const fallible_t& aFallible)
   {
     return AppendASCII(aStr, N - 1, aFallible);
   }
@@ -729,12 +722,12 @@ public:
    * Also ensures that the buffer is mutable.
    */
   void NS_FASTCALL SetCapacity(size_type aNewCapacity);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL SetCapacity(size_type aNewCapacity,
-                                                      const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL SetCapacity(size_type aNewCapacity,
+                                            const fallible_t&);
 
   void NS_FASTCALL SetLength(size_type aNewLength);
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL SetLength(size_type aNewLength,
-                                                    const fallible_t&);
+  MOZ_MUST_USE bool NS_FASTCALL SetLength(size_type aNewLength,
+                                          const fallible_t&);
 
   void Truncate(size_type aNewLength = 0)
   {
@@ -951,7 +944,7 @@ protected:
    * any of its member variables.  in other words, this function acts
    * like a destructor.
    */
-  void NS_FASTCALL B2G_ACL_EXPORT Finalize();
+  void NS_FASTCALL Finalize();
 
   /**
    * this function prepares mData to be mutated.
@@ -994,11 +987,11 @@ protected:
    * this function returns false if is unable to allocate sufficient
    * memory.
    */
-  MOZ_WARN_UNUSED_RESULT bool ReplacePrep(index_type aCutStart,
-                                          size_type aCutLength,
-                                          size_type aNewLength);
+  MOZ_MUST_USE bool ReplacePrep(index_type aCutStart,
+                                size_type aCutLength,
+                                size_type aNewLength);
 
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL ReplacePrepInternal(
+  MOZ_MUST_USE bool NS_FASTCALL ReplacePrepInternal(
     index_type aCutStart,
     size_type aCutLength,
     size_type aNewFragLength,
@@ -1017,7 +1010,7 @@ protected:
    * this helper function can be called prior to directly manipulating
    * the contents of mData.  see, for example, BeginWriting.
    */
-  MOZ_WARN_UNUSED_RESULT bool NS_FASTCALL EnsureMutable(
+  MOZ_MUST_USE bool NS_FASTCALL EnsureMutable(
     size_type aNewLen = size_type(-1));
 
   /**
@@ -1036,6 +1029,20 @@ protected:
     return (aStart < (mData + mLength) && aEnd > mData);
   }
 
+  /**
+   * Checks if the given capacity is valid for this string type.
+   */
+  static MOZ_MUST_USE bool CheckCapacity(size_type aCapacity) {
+    if (aCapacity > kMaxCapacity) {
+      // Also assert for |aCapacity| equal to |size_type(-1)|, since we used to
+      // use that value to flag immutability.
+      NS_ASSERTION(aCapacity != size_type(-1), "Bogus capacity");
+      return false;
+    }
+
+    return true;
+  }
+
   /**
    * this helper function stores the specified dataFlags in mFlags
    */
@@ -1050,6 +1057,7 @@ protected:
 
   static int AppendFunc(void* aArg, const char* aStr, uint32_t aLen);
 
+  static const size_type kMaxCapacity;
 public:
 
   // NOTE: this method is declared public _only_ for convenience for
@@ -1126,6 +1134,13 @@ operator!=(const nsTSubstring_CharT::base_string_type& aLhs,
   return !aLhs.Equals(aRhs);
 }
 
+inline bool
+operator!=(const nsTSubstring_CharT::base_string_type& aLhs,
+           const nsTSubstring_CharT::char_type* aRhs)
+{
+  return !aLhs.Equals(aRhs);
+}
+
 inline bool
 operator<(const nsTSubstring_CharT::base_string_type& aLhs,
           const nsTSubstring_CharT::base_string_type& aRhs)
